@@ -1,6 +1,8 @@
 #ifndef __MDFN_PSFLOADER_H
 #define __MDFN_PSFLOADER_H
 
+#include <mednafen/Stream.h>
+
 #include <map>
 #include <string.h>
 #include <vector>
@@ -17,7 +19,7 @@ class PSFTags
  std::string GetTag(const char *name);
  bool TagExists(const char *name);
 
- void LoadTags(const uint8 *data, uint32 size);
+ void LoadTags(Stream* fp);
  void EraseTag(const char *name);
 
 
@@ -31,18 +33,18 @@ class PSFLoader
 {
  public:
  PSFLoader();
- ~PSFLoader();
+ virtual ~PSFLoader();
 
- static bool TestMagic(uint8 version, MDFNFILE *fp);
+ static bool TestMagic(uint8 version, Stream *fp);
 
- PSFTags Load(uint8 version, uint32 max_exe_size, MDFNFILE *fp);
+ PSFTags Load(uint8 version, uint32 max_exe_size, Stream *fp);
 
- virtual void HandleReserved(const uint8 *data, uint32 len);
- virtual void HandleEXE(const uint8 *data, uint32 len, bool ignore_pcsp = false);
+ virtual void HandleReserved(Stream* fp, uint32 len);
+ virtual void HandleEXE(Stream* fp, bool ignore_pcsp = false);
 
  private:
 
- PSFTags LoadInternal(uint8 version, uint32 max_exe_size, MDFNFILE *fp, uint32 level, bool force_ignore_pcsp = false);
+ PSFTags LoadInternal(uint8 version, uint32 max_exe_size, Stream *fp, uint32 level, bool force_ignore_pcsp = false);
 };
 
 
