@@ -20,10 +20,6 @@
 #include "CDAccess_Image.h"
 #include "CDAccess_CCD.h"
 
-#ifdef HAVE_LIBCDIO
-#include "CDAccess_Physical.h"
-#endif
-
 using namespace CDUtility;
 
 CDAccess::CDAccess()
@@ -36,11 +32,11 @@ CDAccess::~CDAccess()
 
 }
 
-CDAccess *cdaccess_open_image(const char *path, bool image_memcache)
+CDAccess* CDAccess_Open(const std::string& path, bool image_memcache)
 {
  CDAccess *ret = NULL;
 
- if(strlen(path) >= 4 && !strcasecmp(path + strlen(path) - 4, ".ccd"))
+ if(path.size() >= 4 && !strcasecmp(path.c_str() + path.size() - 4, ".ccd"))
   ret = new CDAccess_CCD(path, image_memcache);
  else
   ret = new CDAccess_Image(path, image_memcache);
@@ -48,11 +44,3 @@ CDAccess *cdaccess_open_image(const char *path, bool image_memcache)
  return ret;
 }
 
-CDAccess *cdaccess_open_phys(const char *devicename)
-{
- #ifdef HAVE_LIBCDIO
- return new CDAccess_Physical(devicename);
- #else
- throw MDFN_Error(0, _("Physical CD access support not compiled in."));
- #endif
-}
