@@ -26,8 +26,6 @@
 
 	A PREGAP statement in the first track definition in a CUE sheet may not work properly(depends on what is proper);
 	it will be added onto the implicit default 00:02:00 of pregap.
-
-	Trying to read sectors at an LBA of less than 0 is not supported.  TODO: support it(at least up to -150).
 */
 
 #include "../mednafen.h"
@@ -38,6 +36,7 @@
 #include <errno.h>
 #include <time.h>
 #include <trio/trio.h>
+#include <memory>
 
 #include "../general.h"
 #include "../string/trim.h"
@@ -48,7 +47,7 @@
 #include "CDAccess.h"
 #include "CDAccess_Image.h"
 
-#include "audioreader.h"
+#include "CDAFReader.h"
 
 #include <map>
 
@@ -71,10 +70,11 @@ enum
  DI_FORMAT_MODE2_FORM1 = 0x04,
  DI_FORMAT_MODE2_FORM2 = 0x05,
  DI_FORMAT_MODE2_RAW   = 0x06,
+ DI_FORMAT_CDI_RAW     = 0x07,
  _DI_FORMAT_COUNT
 };
 
-static const int32 DI_Size_Table[7] =
+static const int32 DI_Size_Table[8] =
 {
  2352, // Audio
  2048, // MODE1
@@ -82,10 +82,11 @@ static const int32 DI_Size_Table[7] =
  2336, // MODE2
  2048, // MODE2 Form 1
  2324, // Mode 2 Form 2
- 2352
+ 2352, // MODE2 RAW
+ 2352, // CD-I RAW
 };
 
-static const char *DI_CDRDAO_Strings[7] = 
+static const char *DI_CDRDAO_Strings[8] = 
 {
  "AUDIO",
  "MODE1",
@@ -93,20 +94,20 @@ static const char *DI_CDRDAO_Strings[7] =
  "MODE2",
  "MODE2_FORM1",
  "MODE2_FORM2",
- "MODE2_RAW"
+ "MODE2_RAW",
+ "CDI_RAW"
 };
 
-static const char *DI_CUE_Strings[7] = 
+static const char *DI_CUE_Strings[8] = 
 {
  "AUDIO",
  "MODE1/2048",
  "MODE1/2352",
-
- // FIXME: These are just guesses:
- "MODE2/2336",
- "MODE2/2048",
- "MODE2/2324",
- "MODE2/2352"
+ "MODE2/2336",	// FIXME: A guess
+ "MODE2/2048",	// FIXME: A guess
+ "MODE2/2324",	// FIXME: A guess
+ "MODE2/2352",	// FIXME: A guess
+ "CDI/2352",
 };
 
 // Should return an offset to the start of the next argument(past any whitespace), or if there isn't a next argument,
@@ -223,16 +224,16 @@ void CDAccess_Image::ParseTOCFileLineInfo(CDRFILE_TRACK_INFO *track, const int t
   efn = MDFN_EvalFIP(base_dir, filename);
 
   if(image_memcache)
-   track->fp = new MemoryStream(new FileStream(efn.c_str(), FileStream::MODE_READ));
+   track->fp = new MemoryStream(new FileStream(efn, FileStream::MODE_READ));
   else
-   track->fp = new FileStream(efn.c_str(), FileStream::MODE_READ);
+   track->fp = new FileStream(efn, FileStream::MODE_READ);
 
   toc_streamcache[filename] = track->fp;
  }
 
  if(filename.length() >= 4 && !strcasecmp(filename.c_str() + filename.length() - 4, ".wav"))
  {
-  track->AReader = AR_Open(track->fp);
+  track->AReader = CDAFR_Open(track->fp);
 
   if(!track->AReader)
    throw MDFN_Error(0, "TODO ERROR");
@@ -289,18 +290,6 @@ void CDAccess_Image::ParseTOCFileLineInfo(CDRFILE_TRACK_INFO *track, const int t
  track->sectors = sectors;
 }
 
-
-static void MDFN_strtoupper(char *str)
-{
- for(size_t x = 0; str[x]; x++)
- {
-  if(str[x] >= 'a' && str[x] <= 'z')
-  {
-   str[x] = str[x] - 'a' + 'A';
-  }
- }
-}
-
 static void MDFN_strtoupper(std::string &str)
 {
  const size_t len = str.length();
@@ -334,7 +323,75 @@ std::string MDFN_toupper(const std::string &str)
 }
 #endif
 
-void CDAccess_Image::ImageOpen(const char *path, bool image_memcache)
+void CDAccess_Image::LoadSBI(const std::string& sbi_path)
+{
+ MDFN_printf(_("Loading SBI file \"%s\"...\n"), sbi_path.c_str());
+ {
+  MDFN_AutoIndent aind(1);
+
+  try
+  {
+   FileStream sbis(sbi_path, FileStream::MODE_READ);
+   uint8 header[4];
+   uint8 ed[4 + 10];
+   uint8 tmpq[12];
+
+   sbis.read(header, 4);
+
+   if(memcmp(header, "SBI\0", 4))
+    throw MDFN_Error(0, _("Not recognized a valid SBI file."));
+
+   while(sbis.read(ed, sizeof(ed), false) == sizeof(ed))
+   {
+    if(!BCD_is_valid(ed[0]) || !BCD_is_valid(ed[1]) || !BCD_is_valid(ed[2]))
+     throw MDFN_Error(0, _("Bad BCD MSF offset in SBI file: %02x:%02x:%02x"), ed[0], ed[1], ed[2]);
+
+    if(ed[3] != 0x01)
+     throw MDFN_Error(0, _("Unrecognized boogly oogly in SBI file: %02x"), ed[3]);
+
+    memcpy(tmpq, &ed[4], 10);
+
+    //
+    subq_generate_checksum(tmpq);
+    tmpq[10] ^= 0xFF;
+    tmpq[11] ^= 0xFF;
+    //
+
+    //printf("%02x:%02x:%02x --- ", ed[0], ed[1], ed[2]);
+    //for(unsigned i = 0; i < 12; i++)
+    // printf("%02x ", tmpq[i]);
+    //printf("\n");
+
+    uint32 aba = AMSF_to_ABA(BCD_to_U8(ed[0]), BCD_to_U8(ed[1]), BCD_to_U8(ed[2]));
+
+    memcpy(SubQReplaceMap[aba].data(), tmpq, 12);
+   }
+   MDFN_printf(_("Loaded Q subchannel replacements for %zu sectors.\n"), SubQReplaceMap.size());
+  }
+  catch(MDFN_Error &e)
+  {
+   if(e.GetErrno() != ENOENT)
+    throw;
+   else
+    MDFN_printf(_("Error: %s\n"), e.what());
+  }
+  catch(std::exception &e)
+  {
+   throw;
+  }
+ }
+}
+
+static void StringToMSF(const char* str, unsigned* m, unsigned* s, unsigned* f)
+{
+ if(trio_sscanf(str, "%u:%u:%u", m, s, f) != 3)
+  throw MDFN_Error(0, _("M:S:F time \"%s\" is malformed."), str);
+
+ if(*m > 99 || *s > 59 || *f > 74)
+  throw MDFN_Error(0, _("M:S:F time \"%s\" contains component(s) out of range."), str);
+}
+
+void CDAccess_Image::ImageOpen(const std::string& path, bool image_memcache)
 {
  MemoryStream fp(new FileStream(path, FileStream::MODE_READ));
  static const unsigned max_args = 4;
@@ -520,8 +577,11 @@ void CDAccess_Image::ImageOpen(const char *path, bool image_memcache)
      {
       throw(MDFN_Error(0, _("Command %s is outside of a TRACK definition!\n"), cmdbuf.c_str()));
      }
-     int m,s,f;
-     trio_sscanf(args[0].c_str(), "%d:%d:%d", &m, &s, &f);
+
+     unsigned int m,s,f;
+
+     StringToMSF(args[0].c_str(), &m, &s, &f);
+
      TmpTrack.pregap = (m * 60 + s) * 75 + f;
     } // end to PREGAP
     else if(cmdbuf == "START")
@@ -530,8 +590,11 @@ void CDAccess_Image::ImageOpen(const char *path, bool image_memcache)
      {
       throw(MDFN_Error(0, _("Command %s is outside of a TRACK definition!\n"), cmdbuf.c_str()));
      }
-     int m,s,f;
-     trio_sscanf(args[0].c_str(), "%d:%d:%d", &m, &s, &f);
+
+     unsigned int m,s,f;
+
+     StringToMSF(args[0].c_str(), &m, &s, &f);
+
      TmpTrack.pregap = (m * 60 + s) * 75 + f;
     }
     else if(cmdbuf == "TWO_CHANNEL_AUDIO")
@@ -592,13 +655,8 @@ void CDAccess_Image::ImageOpen(const char *path, bool image_memcache)
       active_track = -1;
      }
 
-     if(!MDFN_IsFIROPSafe(args[0]))
-     {
-      throw(MDFN_Error(0, _("Referenced path \"%s\" is potentially unsafe.  See \"filesys.untrusted_fip_check\" setting.\n"), args[0].c_str()));
-     }
-
      std::string efn = MDFN_EvalFIP(base_dir, args[0]);
-     TmpTrack.fp = new FileStream(efn.c_str(), FileStream::MODE_READ);
+     TmpTrack.fp = new FileStream(efn, FileStream::MODE_READ);
      TmpTrack.FirstFileInstance = 1;
 
      if(image_memcache)
@@ -614,7 +672,7 @@ void CDAccess_Image::ImageOpen(const char *path, bool image_memcache)
      else if(!strcasecmp(args[1].c_str(), "OGG") || !strcasecmp(args[1].c_str(), "VORBIS") || !strcasecmp(args[1].c_str(), "WAVE") || !strcasecmp(args[1].c_str(), "WAV") || !strcasecmp(args[1].c_str(), "PCM")
 	|| !strcasecmp(args[1].c_str(), "MPC") || !strcasecmp(args[1].c_str(), "MP+"))
      {
-      TmpTrack.AReader = AR_Open(TmpTrack.fp);
+      TmpTrack.AReader = CDAFR_Open(TmpTrack.fp);
       if(!TmpTrack.AReader)
       {
        throw(MDFN_Error(0, _("Unsupported audio track file format: %s\n"), args[0].c_str()));
@@ -639,6 +697,11 @@ void CDAccess_Image::ImageOpen(const char *path, bool image_memcache)
      }
      active_track = atoi(args[0].c_str());
 
+     if(active_track < 1 || active_track > 99)
+     {
+      throw(MDFN_Error(0, _("Invalid track number: %d\n"), active_track));
+     }
+
      if(active_track < FirstTrack)
       FirstTrack = active_track;
      if(active_track > LastTrack)
@@ -658,11 +721,6 @@ void CDAccess_Image::ImageOpen(const char *path, bool image_memcache)
      {
       throw(MDFN_Error(0, _("Invalid track format: %s\n"), args[1].c_str()));
      }
-
-     if(active_track < 0 || active_track > 99)
-     {
-      throw(MDFN_Error(0, _("Invalid track number: %d\n"), active_track));
-     }
     }
     else if(cmdbuf == "INDEX")
     {
@@ -670,10 +728,7 @@ void CDAccess_Image::ImageOpen(const char *path, bool image_memcache)
      {
       unsigned int m,s,f;
 
-      if(trio_sscanf(args[1].c_str(), "%u:%u:%u", &m, &s, &f) != 3)
-      {
-       throw MDFN_Error(0, _("Malformed m:s:f time in \"%s\" directive: %s"), cmdbuf.c_str(), args[0].c_str());
-      }
+      StringToMSF(args[1].c_str(), &m, &s, &f);
 
       if(!strcasecmp(args[0].c_str(), "01") || !strcasecmp(args[0].c_str(), "1"))
        TmpTrack.index[1] = (m * 60 + s) * 75 + f;
@@ -687,10 +742,7 @@ void CDAccess_Image::ImageOpen(const char *path, bool image_memcache)
      {
       unsigned int m,s,f;
 
-      if(trio_sscanf(args[0].c_str(), "%u:%u:%u", &m, &s, &f) != 3)
-      {
-       throw MDFN_Error(0, _("Malformed m:s:f time in \"%s\" directive: %s"), cmdbuf.c_str(), args[0].c_str());
-      }
+      StringToMSF(args[0].c_str(), &m, &s, &f);
 
       TmpTrack.pregap = (m * 60 + s) * 75 + f;
      }
@@ -701,10 +753,7 @@ void CDAccess_Image::ImageOpen(const char *path, bool image_memcache)
      {
       unsigned int m,s,f;
 
-      if(trio_sscanf(args[0].c_str(), "%u:%u:%u", &m, &s, &f) != 3)
-      {
-       throw MDFN_Error(0, _("Malformed m:s:f time in \"%s\" directive: %s"), cmdbuf.c_str(), args[0].c_str());
-      }      
+      StringToMSF(args[0].c_str(), &m, &s, &f);
 
       TmpTrack.postgap = (m * 60 + s) * 75 + f;
      }
@@ -768,8 +817,14 @@ void CDAccess_Image::ImageOpen(const char *path, bool image_memcache)
  int32 LastIndex = 0;
  long FileOffset = 0;
 
+ RunningLBA -= 150;
+ Tracks[FirstTrack].pregap += 150;
+
  for(int x = FirstTrack; x < (FirstTrack + NumTracks); x++)
  {
+  if(!Tracks[x].fp && !Tracks[x].AReader)
+   throw MDFN_Error(0, _("Missing track %u."), x);
+
   if(Tracks[x].DIFormat == DI_FORMAT_AUDIO)
    Tracks[x].subq_control &= ~SUBQ_CTRLF_DATA;
   else
@@ -777,16 +832,23 @@ void CDAccess_Image::ImageOpen(const char *path, bool image_memcache)
 
   if(!IsTOC)	// TOC-format disc_type calculation is handled differently.
   {
-   switch(Tracks[x].DIFormat)
+   if(disc_type != DISC_TYPE_CD_I)
    {
-    default: break;
+    switch(Tracks[x].DIFormat)
+    {
+     default: break;
 
-    case DI_FORMAT_MODE2:
-    case DI_FORMAT_MODE2_FORM1:
-    case DI_FORMAT_MODE2_FORM2:
-    case DI_FORMAT_MODE2_RAW:
+     case DI_FORMAT_MODE2:
+     case DI_FORMAT_MODE2_FORM1:
+     case DI_FORMAT_MODE2_FORM2:
+     case DI_FORMAT_MODE2_RAW:
 	disc_type = DISC_TYPE_CD_XA;	
 	break;
+
+     case DI_FORMAT_CDI_RAW:
+	disc_type = DISC_TYPE_CD_I;
+	break;
+    }
    }
   }
 
@@ -846,6 +908,27 @@ void CDAccess_Image::ImageOpen(const char *path, bool image_memcache)
  } // end to track loop
 
  total_sectors = RunningLBA;
+
+ //
+ // Load SBI file, if present
+ //
+ if(!IsTOC)
+ {
+  char sbi_ext[4] = { 's', 'b', 'i', 0 };
+
+  if(file_ext.length() == 4 && file_ext[0] == '.')
+  {
+   for(unsigned i = 0; i < 3; i++)
+   {
+    if(file_ext[1 + i] >= 'A' && file_ext[1 + i] <= 'Z')
+     sbi_ext[i] += 'A' - 'a';
+   }
+  }
+
+  LoadSBI(MDFN_EvalFIP(base_dir, file_base + std::string(".") + std::string(sbi_ext), true).c_str());
+ }
+
+ GenerateTOC();
 }
 
 void CDAccess_Image::Cleanup(void)
@@ -871,7 +954,7 @@ void CDAccess_Image::Cleanup(void)
  }
 }
 
-CDAccess_Image::CDAccess_Image(const char *path, bool image_memcache) : NumTracks(0), FirstTrack(0), LastTrack(0), total_sectors(0)
+CDAccess_Image::CDAccess_Image(const std::string& path, bool image_memcache) : NumTracks(0), FirstTrack(0), LastTrack(0), total_sectors(0)
 {
  memset(Tracks, 0, sizeof(Tracks));
 
@@ -893,64 +976,122 @@ CDAccess_Image::~CDAccess_Image()
 
 void CDAccess_Image::Read_Raw_Sector(uint8 *buf, int32 lba)
 {
-  bool TrackFound = FALSE;
   uint8 SimuQ[0xC];
+  int32 track;
+  CDRFILE_TRACK_INFO *ct;
+
+  //
+  // Leadout synthesis
+  //
+  if(lba >= total_sectors)
+  {
+   uint8 data_synth_mode = (disc_type == DISC_TYPE_CD_XA ? 0x02 : 0x01);
+
+   switch(Tracks[LastTrack].DIFormat)
+   {
+    case DI_FORMAT_AUDIO:
+		break;
+
+    case DI_FORMAT_MODE1_RAW:
+    case DI_FORMAT_MODE1:
+		data_synth_mode = 0x01;
+		break;
+
+    case DI_FORMAT_MODE2_RAW:
+    case DI_FORMAT_MODE2_FORM1:
+    case DI_FORMAT_MODE2_FORM2:
+    case DI_FORMAT_MODE2:
+    case DI_FORMAT_CDI_RAW:
+		data_synth_mode = 0x02;
+		break;
+   }
+
+   synth_leadout_sector_lba(data_synth_mode, toc, lba, buf);
+   return;
+  }
+  //
+  //
+  //
 
   memset(buf + 2352, 0, 96);
-
-  MakeSubPQ(lba, buf + 2352);
-
+  track = MakeSubPQ(lba, buf + 2352);
   subq_deinterleave(buf + 2352, SimuQ);
 
-  for(int32 track = FirstTrack; track < (FirstTrack + NumTracks); track++)
+  ct = &Tracks[track];
+
+  //
+  // Handle pregap and postgap reading
+  //
+  if(lba < (ct->LBA - ct->pregap_dv) || lba >= (ct->LBA + ct->sectors))
   {
-   CDRFILE_TRACK_INFO *ct = &Tracks[track];
+   int32 pg_offset = lba - ct->LBA;
+   CDRFILE_TRACK_INFO* et = ct;
 
-   if(lba >= (ct->LBA - ct->pregap_dv - ct->pregap) && lba < (ct->LBA + ct->sectors + ct->postgap))
+   if(pg_offset < -150)
    {
-    TrackFound = TRUE;
+    if((Tracks[track].subq_control & SUBQ_CTRLF_DATA) && (FirstTrack < track) && !(Tracks[track - 1].subq_control & SUBQ_CTRLF_DATA))
+     et = &Tracks[track - 1];
+   }
 
-    // Handle pregap and postgap reading
-    if(lba < (ct->LBA - ct->pregap_dv) || lba >= (ct->LBA + ct->sectors))
+   memset(buf, 0, 2352);
+   switch(et->DIFormat)
+   {
+    case DI_FORMAT_AUDIO:
+		break;
+
+    case DI_FORMAT_MODE1_RAW:
+    case DI_FORMAT_MODE1:
+		encode_mode1_sector(lba + 150, buf);
+		break;
+
+    case DI_FORMAT_MODE2_RAW:
+    case DI_FORMAT_MODE2_FORM1:
+    case DI_FORMAT_MODE2_FORM2:
+    case DI_FORMAT_MODE2:
+    case DI_FORMAT_CDI_RAW:
+	 	buf[12 +  6] = 0x20;
+		buf[12 + 10] = 0x20;
+		encode_mode2_form2_sector(lba + 150, buf);
+		// TODO: Zero out optional(?) checksum bytes?
+		break;
+   }
+   //printf("Pre/post-gap read, LBA=%d(LBA-track_start_LBA=%d)\n", lba, lba - ct->LBA);
+  }
+  else
+  {
+   if(ct->AReader)
+   {
+    int16 AudioBuf[588 * 2];
+    uint64 frames_read = ct->AReader->Read((ct->FileOffset / 4) + (lba - ct->LBA) * 588, AudioBuf, 588);
+
+    ct->LastSamplePos += frames_read;
+
+    if(frames_read > 588)	// This shouldn't happen.
     {
-     //printf("Pre/post-gap read, LBA=%d(LBA-track_start_LBA=%d)\n", lba, lba - ct->LBA);
-     memset(buf, 0, 2352);	// Null sector data, per spec
+     printf("Error: frames_read out of range: %llu\n", (unsigned long long)frames_read);
+     frames_read = 0;
     }
-    else
+
+    if(frames_read < 588)
+     memset((uint8 *)AudioBuf + frames_read * 2 * sizeof(int16), 0, (588 - frames_read) * 2 * sizeof(int16));
+
+    for(int i = 0; i < 588 * 2; i++)
+     MDFN_en16lsb(buf + i * 2, AudioBuf[i]);
+   }
+   else	// Binary, woo.
+   {
+    long SeekPos = ct->FileOffset;
+    long LBARelPos = lba - ct->LBA;
+
+    SeekPos += LBARelPos * DI_Size_Table[ct->DIFormat];
+
+    if(ct->SubchannelMode)
+     SeekPos += 96 * (lba - ct->LBA);
+
+    ct->fp->seek(SeekPos, SEEK_SET);
+
+    switch(ct->DIFormat)
     {
-     if(ct->AReader)
-     {
-      int16 AudioBuf[588 * 2];
-      int frames_read = ct->AReader->Read((ct->FileOffset / 4) + (lba - ct->LBA) * 588, AudioBuf, 588);
-
-      ct->LastSamplePos += frames_read;
-
-      if(frames_read < 0 || frames_read > 588)	// This shouldn't happen.
-      {
-       printf("Error: frames_read out of range: %d\n", frames_read);
-       frames_read = 0;
-      }
-
-      if(frames_read < 588)
-       memset((uint8 *)AudioBuf + frames_read * 2 * sizeof(int16), 0, (588 - frames_read) * 2 * sizeof(int16));
-
-      for(int i = 0; i < 588 * 2; i++)
-       MDFN_en16lsb(buf + i * 2, AudioBuf[i]);
-     }
-     else	// Binary, woo.
-     {
-      long SeekPos = ct->FileOffset;
-      long LBARelPos = lba - ct->LBA;
-
-      SeekPos += LBARelPos * DI_Size_Table[ct->DIFormat];
-
-      if(ct->SubchannelMode)
-       SeekPos += 96 * (lba - ct->LBA);
-
-      ct->fp->seek(SeekPos, SEEK_SET);
-
-      switch(ct->DIFormat)
-      {
 	case DI_FORMAT_AUDIO:
 		ct->fp->read(buf, 2352);
 
@@ -965,6 +1106,7 @@ void CDAccess_Image::Read_Raw_Sector(uint8 *buf, int32 lba)
 
 	case DI_FORMAT_MODE1_RAW:
 	case DI_FORMAT_MODE2_RAW:
+	case DI_FORMAT_CDI_RAW:
 		ct->fp->read(buf, 2352);
 		break;
 
@@ -986,55 +1128,47 @@ void CDAccess_Image::Read_Raw_Sector(uint8 *buf, int32 lba)
 		//encode_mode2_form2_sector(lba + 150, buf);
 		break;
 
-      }
+    }
 
-      if(ct->SubchannelMode)
-       ct->fp->read(buf + 2352, 96);
-     }
-    } // end if audible part of audio track read.
-    break;
-   } // End if LBA is in range
-  } // end track search loop
-
-  if(!TrackFound)
-  {
-   throw(MDFN_Error(0, _("Could not find track for sector %u!"), lba));
-  }
-
-#if 0
- if(qbuf[0] & 0x40)
- {
-  uint8 dummy_buf[2352 + 96];
-  bool any_mismatch = FALSE;
-
-  memcpy(dummy_buf + 16, buf + 16, 2048); 
-  memset(dummy_buf + 2352, 0, 96);
-
-  MakeSubPQ(lba, dummy_buf + 2352);
-  encode_mode1_sector(lba + 150, dummy_buf);
-
-  for(int i = 0; i < 2352 + 96; i++)
-  {
-   if(dummy_buf[i] != buf[i])
-   {
-    printf("Mismatch at %d, %d: %02x:%02x; ", lba, i, dummy_buf[i], buf[i]);
-    any_mismatch = TRUE;
+    if(ct->SubchannelMode)
+     ct->fp->read(buf + 2352, 96);
    }
-  }
-  if(any_mismatch)
-   puts("\n");
- }
-#endif
+  } // end if audible part of audio track read.
+}
 
- //subq_deinterleave(buf + 2352, qbuf);
- //printf("%02x\n", qbuf[0]);
- //printf("%02x\n", buf[12 + 3]);
+bool CDAccess_Image::Fast_Read_Raw_PW_TSRE(uint8* pwbuf, int32 lba) const noexcept
+{
+ int32 track;
+
+ if(lba >= total_sectors)
+ {
+  subpw_synth_leadout_lba(toc, lba, pwbuf);
+  return(true);
+ }
+
+ memset(pwbuf, 0, 96);
+ try
+ {
+  track = MakeSubPQ(lba, pwbuf);
+ }
+ catch(...)
+ {
+  return(false);
+ }
+
+ //
+ // If TOC+BIN has embedded subchannel data, we can't fast-read(synthesize) it...
+ //
+ if(Tracks[track].SubchannelMode && lba >= (Tracks[track].LBA - Tracks[track].pregap_dv) && (lba < Tracks[track].LBA + Tracks[track].sectors))
+  return(false);
+
+ return(true);
 }
 
 //
 // Note: this function makes use of the current contents(as in |=) in SubPWBuf.
 //
-void CDAccess_Image::MakeSubPQ(int32 lba, uint8 *SubPWBuf)
+int32 CDAccess_Image::MakeSubPQ(int32 lba, uint8 *SubPWBuf) const
 {
  uint8 buf[0xC];
  int32 track;
@@ -1042,26 +1176,24 @@ void CDAccess_Image::MakeSubPQ(int32 lba, uint8 *SubPWBuf)
  uint32 ma, sa, fa;
  uint32 m, s, f;
  uint8 pause_or = 0x00;
- bool track_found = FALSE;
+ bool track_found = false;
 
  for(track = FirstTrack; track < (FirstTrack + NumTracks); track++)
  {
   if(lba >= (Tracks[track].LBA - Tracks[track].pregap_dv - Tracks[track].pregap) && lba < (Tracks[track].LBA + Tracks[track].sectors + Tracks[track].postgap))
   {
-   track_found = TRUE;
+   track_found = true;
    break;
   }
  }
 
- //printf("%d %d\n", Tracks[1].LBA, Tracks[1].sectors);
-
  if(!track_found)
- {
-  printf("MakeSubPQ error for sector %u!", lba);
-  track = FirstTrack;
- }
+  throw(MDFN_Error(0, _("Could not find track for sector %u!"), lba));
 
- lba_relative = abs((int32)lba - Tracks[track].LBA);
+ if(lba < Tracks[track].LBA)
+  lba_relative = Tracks[track].LBA - 1 - lba;
+ else
+  lba_relative = lba - Tracks[track].LBA;
 
  f = (lba_relative % 75);
  s = ((lba_relative / 75) % 60);
@@ -1124,41 +1256,55 @@ void CDAccess_Image::MakeSubPQ(int32 lba, uint8 *SubPWBuf)
 
  subq_generate_checksum(buf);
 
- for(int i = 0; i < 96; i++)
-  SubPWBuf[i] |= (((buf[i >> 3] >> (7 - (i & 0x7))) & 1) ? 0x40 : 0x00) | pause_or;
-}
-
-void CDAccess_Image::Read_TOC(TOC *toc)
-{
- toc->Clear();
-
- toc->first_track = FirstTrack;
- toc->last_track = FirstTrack + NumTracks - 1;
- toc->disc_type = disc_type;
-
- for(int i = toc->first_track; i <= toc->last_track; i++)
+ if(!SubQReplaceMap.empty())
  {
-  toc->tracks[i].lba = Tracks[i].LBA;
-  toc->tracks[i].adr = ADR_CURPOS;
-  toc->tracks[i].control = Tracks[i].subq_control;
+  //printf("%d\n", lba);
+  auto it = SubQReplaceMap.find(LBA_to_ABA(lba));
+
+  if(it != SubQReplaceMap.end())
+  {
+   //printf("Replace: %d\n", lba);
+   memcpy(buf, it->second.data(), 12);
+  }
  }
 
- toc->tracks[100].lba = total_sectors;
- toc->tracks[100].adr = ADR_CURPOS;
- toc->tracks[100].control = toc->tracks[toc->last_track].control & 0x4;
+ for(int i = 0; i < 96; i++)
+  SubPWBuf[i] |= (((buf[i >> 3] >> (7 - (i & 0x7))) & 1) ? 0x40 : 0x00) | pause_or;
 
- // Convenience leadout track duplication.
- if(toc->last_track < 99)
-  toc->tracks[toc->last_track + 1] = toc->tracks[100];
+ return track;
 }
 
-bool CDAccess_Image::Is_Physical(void) throw()
+void CDAccess_Image::Read_TOC(TOC *rtoc)
 {
- return(false);
+ *rtoc = toc;
 }
 
-void CDAccess_Image::Eject(bool eject_status)
+void CDAccess_Image::GenerateTOC(void)
 {
+ toc.Clear();
 
+ toc.first_track = FirstTrack;
+ toc.last_track = FirstTrack + NumTracks - 1;
+ toc.disc_type = disc_type;
+
+ for(int i = FirstTrack; i < FirstTrack + NumTracks; i++)
+ {
+  if(Tracks[i].DIFormat == DI_FORMAT_CDI_RAW)
+  {
+   toc.first_track = std::min<int>(99, i + 1);
+   toc.last_track = std::max<int>(toc.first_track, toc.last_track);
+  }
+
+  toc.tracks[i].lba = Tracks[i].LBA;
+  toc.tracks[i].adr = ADR_CURPOS;
+  toc.tracks[i].control = Tracks[i].subq_control;
+  toc.tracks[i].valid = true;
+ }
+
+ toc.tracks[100].lba = total_sectors;
+ toc.tracks[100].adr = ADR_CURPOS;
+ toc.tracks[100].control = Tracks[FirstTrack + NumTracks - 1].subq_control & 0x4;
+ toc.tracks[100].valid = true;
 }
 
+
