@@ -16,14 +16,50 @@ class SimpleFIFO
  {
   data.resize(round_up_pow2(the_size));
   size = the_size;
+  read_pos = 0;
+  write_pos = 0;
+  in_count = 0;
  }
 
  // Destructor
- ~SimpleFIFO()
+ INLINE ~SimpleFIFO()
  {
 
  }
 
+ INLINE void SaveStatePostLoad(void)
+ {
+  read_pos %= data.size();
+  write_pos %= data.size();
+  in_count %= (data.size() + 1);
+ }
+
+#if 0
+ INLINE int StateAction(StateMem *sm, int load, int data_only, const char* sname)
+ {
+  SFORMAT StateRegs[] =
+  {
+   std::vector<T> data;
+   uint32 size;
+
+   SFVAR(read_pos),
+   SFVAR(write_pos),
+   SFVAR(in_count),
+   SFEND;
+  }
+  int ret = MDFNSS_StateAction(sm, load, data_only, sname);
+
+  if(load)
+  {
+   read_pos %= data.size();
+   write_pos %= data.size();
+   in_count %= (data.size() + 1);
+  }
+
+  return(ret);
+ }
+#endif
+
  INLINE uint32 CanRead(void)
  {
   return(in_count);
@@ -58,7 +94,7 @@ class SimpleFIFO
   return(ReadUnit(peek));
  }
 
- inline void Write(const T *happy_data, uint32 happy_count)
+ INLINE void Write(const T *happy_data, uint32 happy_count)
  {
   assert(CanWrite() >= happy_count);
 
@@ -73,19 +109,19 @@ class SimpleFIFO
   }
  }
 
- void WriteUnit(T wr_data)
+ INLINE void WriteUnit(const T& wr_data)
  {
   Write(&wr_data, 1);
  }
 
- void WriteByte(T wr_data)
+ INLINE void WriteByte(const T& wr_data)
  {
   assert(sizeof(T) == 1);
   Write(&wr_data, 1);
  }
 
 
- void Flush(void)
+ INLINE void Flush(void)
  {
   read_pos = 0;
   write_pos = 0;
@@ -97,7 +133,7 @@ class SimpleFIFO
  uint32 size;
  uint32 read_pos; // Read position
  uint32 write_pos; // Write position
- uint32 in_count; // Number of bytes in the FIFO
+ uint32 in_count; // Number of units in the FIFO
 };
 
 
