@@ -378,9 +378,23 @@ static const ButtConfig MDPad3Config[2][8] =
 {
  { MK(w), MK(s), MK(a), MK(d), MK(KP2), MK(KP3), MK(KP1), MK(RETURN) },
  { MKZ(), MKZ(), MKZ(), MKZ(), MKZ(),   MKZ(),   MKZ(),   MKZ() },
-
 };
 
+static const ButtConfig MDPad6Config[2][12] =
+{
+ { MK(w), MK(s), MK(a), MK(d), MK(KP2), MK(KP3), MK(KP1), MK(RETURN), MK(KP6), MK(KP5), MK(KP4), MK(m) },
+ { MKZ(), MKZ(), MKZ(), MKZ(), MKZ(),   MKZ(),   MKZ(),   MKZ(),      MKZ(),   MKZ(),   MKZ(),   MKZ() },
+};
+
+static ButtConfig MDMegaMouseConfig[4] =
+{
+ { BUTTC_MOUSE, 0, 0, 0 },
+ { BUTTC_MOUSE, 0, 2, 0 },
+ { BUTTC_MOUSE, 0, 1, 0 },
+ MK(RETURN),
+};
+
+
 static const ButtConfig SNESPadConfig[12] =
 {
  MK(KP2),
@@ -416,26 +430,6 @@ static ButtConfig SNESSuperScopeConfig[7] =
  { BUTTC_MOUSE, 0, 2, 0 },	// Cursor
 };
 
-/*
- { "select", "SELECT", 4, IDIT_BUTTON, NULL },
- { NULL, "empty", 0, IDIT_BUTTON },
- { NULL, "empty", 0, IDIT_BUTTON },
- { "start", "START", 5, IDIT_BUTTON, NULL },
- { "up", "UP ↑", 0, IDIT_BUTTON, "down" },
- { "right", "RIGHT →", 3, IDIT_BUTTON, "left" },
- { "down", "DOWN ↓", 1, IDIT_BUTTON, "up" },
- { "left", "LEFT ←", 2, IDIT_BUTTON, "right" },
-
- { "l2", "L2 (rear left shoulder)", 11, IDIT_BUTTON, NULL },
- { "r2", "R2 (rear right shoulder)", 13, IDIT_BUTTON, NULL },
- { "l1", "L1 (front left shoulder)", 10, IDIT_BUTTON, NULL },
- { "r1", "R1 (front right shoulder)", 12, IDIT_BUTTON, NULL },
-
- { "triangle", "△ (upper)", 6, IDIT_BUTTON_CAN_RAPID, NULL },
- { "circle", "○ (right)", 9, IDIT_BUTTON_CAN_RAPID, NULL },
- { "cross", "x (lower)", 7, IDIT_BUTTON_CAN_RAPID, NULL },
- { "square", "□ (left)", 8, IDIT_BUTTON_CAN_RAPID, NULL },
-*/
 static const ButtConfig PSXPadConfig[14] =
 {
  MK(TAB),
@@ -600,6 +594,25 @@ static DefaultSettingsMeow defset[] =
  { "md.input.port7.gamepad", MDPad3Config[1], sizeof(MDPad3Config[1]) / sizeof(ButtConfig) },
  { "md.input.port8.gamepad", MDPad3Config[1], sizeof(MDPad3Config[1]) / sizeof(ButtConfig) },
 
+ { "md.input.port1.gamepad6", MDPad6Config[0], sizeof(MDPad6Config[0]) / sizeof(ButtConfig) },
+ { "md.input.port2.gamepad6", MDPad6Config[1], sizeof(MDPad6Config[1]) / sizeof(ButtConfig) }, 
+ { "md.input.port3.gamepad6", MDPad6Config[1], sizeof(MDPad6Config[1]) / sizeof(ButtConfig) },
+ { "md.input.port4.gamepad6", MDPad6Config[1], sizeof(MDPad6Config[1]) / sizeof(ButtConfig) },
+ { "md.input.port5.gamepad6", MDPad6Config[1], sizeof(MDPad6Config[1]) / sizeof(ButtConfig) },
+ { "md.input.port6.gamepad6", MDPad6Config[1], sizeof(MDPad6Config[1]) / sizeof(ButtConfig) },
+ { "md.input.port7.gamepad6", MDPad6Config[1], sizeof(MDPad6Config[1]) / sizeof(ButtConfig) },
+ { "md.input.port8.gamepad6", MDPad6Config[1], sizeof(MDPad6Config[1]) / sizeof(ButtConfig) },
+
+ { "md.input.port1.megamouse", MDMegaMouseConfig, sizeof(MDMegaMouseConfig) / sizeof(ButtConfig) },
+ { "md.input.port2.megamouse", MDMegaMouseConfig, sizeof(MDMegaMouseConfig) / sizeof(ButtConfig) },
+ { "md.input.port3.megamouse", MDMegaMouseConfig, sizeof(MDMegaMouseConfig) / sizeof(ButtConfig) },
+ { "md.input.port4.megamouse", MDMegaMouseConfig, sizeof(MDMegaMouseConfig) / sizeof(ButtConfig) },
+ { "md.input.port5.megamouse", MDMegaMouseConfig, sizeof(MDMegaMouseConfig) / sizeof(ButtConfig) },
+ { "md.input.port6.megamouse", MDMegaMouseConfig, sizeof(MDMegaMouseConfig) / sizeof(ButtConfig) },
+ { "md.input.port7.megamouse", MDMegaMouseConfig, sizeof(MDMegaMouseConfig) / sizeof(ButtConfig) },
+ { "md.input.port8.megamouse", MDMegaMouseConfig, sizeof(MDMegaMouseConfig) / sizeof(ButtConfig) },
+
+
  { "snes.input.port1.gamepad", SNESPadConfig, sizeof(SNESPadConfig) / sizeof(ButtConfig) },
  { "snes.input.port1.mouse", SNESMouseConfig, sizeof(SNESMouseConfig) / sizeof(ButtConfig) },
  { "snes.input.port2.mouse", SNESMouseConfig, sizeof(SNESMouseConfig) / sizeof(ButtConfig) },
