@@ -36,8 +36,8 @@
 #include <pwd.h>
 #endif
 
-#ifdef HAVE_LIBCDIO
-#include <cdio/version.h>
+#ifdef HAVE_ICONV
+#include <iconv.h>
 #endif
 
 #include <algorithm>
@@ -56,7 +56,8 @@
 #include "video-state.h"
 #include "remote.h"
 #include "ers.h"
-#include "../qtrecord.h"
+#include "rmdui.h"
+#include <mednafen/qtrecord.h>
 #include <math.h>
 
 JoystickManager *joy_manager = NULL;
@@ -66,45 +67,6 @@ bool pending_save_state, pending_snapshot, pending_ssnapshot, pending_save_movie
 static uint32 volatile MainThreadID = 0;
 static bool ffnosound;
 
-static const char *CSD_xres = gettext_noop("Full-screen horizontal resolution.");
-static const char *CSD_yres = gettext_noop("Full-screen vertical resolution.");
-static const char *CSDE_xres = gettext_noop("A value of \"0\" will cause the desktop horizontal resolution to be used.");
-static const char *CSDE_yres = gettext_noop("A value of \"0\" will cause the desktop vertical resolution to be used.");
-
-static const char *CSD_xscale = gettext_noop("Scaling factor for the X axis in windowed mode.");
-static const char *CSD_yscale = gettext_noop("Scaling factor for the Y axis in windowed mode.");
-
-static const char *CSD_xscalefs = gettext_noop("Scaling factor for the X axis in fullscreen mode.");
-static const char *CSD_yscalefs = gettext_noop("Scaling factor for the Y axis in fullscreen mode.");
-static const char *CSDE_xyscalefs = gettext_noop("For this settings to have any effect, the \"<system>.stretch\" setting must be set to \"0\".");
-
-static const char *CSD_scanlines = gettext_noop("Enable scanlines with specified opacity.");
-static const char *CSDE_scanlines = gettext_noop("Opacity is specified in %; IE a value of \"100\" will give entirely black scanlines.\n\nNegative values are the same as positive values for non-interlaced video, but for interlaced video will cause the scanlines to be overlaid over the previous field's lines(only if the video.deinterlacer setting is set to \"weave\", the default).");
-
-static const char *CSD_stretch = gettext_noop("Stretch to fill screen.");
-static const char *CSD_videoip = gettext_noop("Enable (bi)linear interpolation.");
-
-
-static const char *CSD_special = gettext_noop("Enable specified special video scaler.");
-static const char *CSDE_special = gettext_noop("The destination rectangle is NOT altered by this setting, so if you have xscale and yscale set to \"2\", and try to use a 3x scaling filter like hq3x, the image is not going to look that great. The nearest-neighbor scalers are intended for use with bilinear interpolation enabled, at high resolutions(such as 1280x1024; nn2x(or nny2x) + bilinear interpolation + fullscreen stretching at this resolution looks quite nice).");
-
-static const char *CSD_pixshader = gettext_noop("Enable specified OpenGL pixel shader.");
-static const char *CSDE_pixshader = gettext_noop("Obviously, this will only work with the OpenGL \"video.driver\" setting, and only on cards and OpenGL implementations that support pixel shaders, otherwise you will get a black screen, or Mednafen may display an error message when starting up. Bilinear interpolation is disabled with pixel shaders, and any interpolation, if present, will be noted in the description of each pixel shader.");
-
-static MDFNSetting_EnumList VDriver_List[] =
-{
- // Legacy:
- { "0", VDRIVER_OPENGL },
- { "1", VDRIVER_SOFTSDL },
-
-
- { "opengl", VDRIVER_OPENGL, "OpenGL + SDL", gettext_noop("This output method is preferred, as all features are available with it.") },
- { "sdl", VDRIVER_SOFTSDL, "SDL Surface", gettext_noop("Slower with lower-quality scaling than OpenGL, but if you don't have hardware-accelerated OpenGL rendering, it will be faster than software OpenGL rendering. Bilinear interpolation not available. Pixel shaders do not work with this output method, of course.") },
- { "overlay", VDRIVER_OVERLAY, "SDL Overlay", gettext_noop("As fast as OpenGL, perhaps faster in some situations, *if* it's hardware-accelerated. Scanline effects are not available. hq2x, hq3x, hq4x are not available. The OSD may be missing or glitchy. PSX emulation will not display properly. Bilinear interpolation can't be turned off. Harsh chroma subsampling blurring in some picture types.  If you use this output method, it is strongly recommended to use a special scaler with it, such as nn2x.") },
-
- { NULL, 0 },
-};
-
 static MDFNSetting_EnumList SDriver_List[] =
 {
  { "default", -1, "Default", gettext_noop("Selects the default sound driver.") },
@@ -125,50 +87,6 @@ static MDFNSetting_EnumList SDriver_List[] =
  { NULL, 0 },
 };
 
-static MDFNSetting_EnumList Special_List[] =
-{
-    { "0", 	-1 },
-    { "none", 	-1, "None/Disabled" },
-
-#ifdef WANT_FANCY_SCALERS
-    { "hq2x", 	-1, "hq2x" },
-    { "hq3x", 	-1, "hq3x" },
-    { "hq4x", 	-1, "hq4x" },
-    { "scale2x",-1, "scale2x" },
-    { "scale3x",-1, "scale3x" },
-    { "scale4x",-1, "scale4x" },
-
-    { "2xsai", 	-1, "2xSaI" },
-    { "super2xsai", -1, "Super 2xSaI" },
-    { "supereagle", -1, "Super Eagle" },
-#endif
-
-    { "nn2x",	-1, "Nearest-neighbor 2x" },
-    { "nn3x",	-1, "Nearest-neighbor 3x" },
-    { "nn4x",	-1, "Nearest-neighbor 4x" },
-    { "nny2x",	-1, "Nearest-neighbor 2x, y axis only" },
-    { "nny3x",	-1, "Nearest-neighbor 3x, y axis only" }, 
-    { "nny4x",	-1, "Nearest-neighbor 4x, y axis only" },
-//    { "scanlines", -1, "Scanlines" },
-
-    { NULL, 0 },
-};
-
-static MDFNSetting_EnumList Pixshader_List[] =
-{
-    { "none",		SHADER_NONE,		"None/Disabled" },
-    { "autoip", 	SHADER_AUTOIP,	"Auto Interpolation", gettext_noop("Will automatically interpolate on each axis if the corresponding effective scaling factor is not an integer.") },
-    { "autoipsharper",	SHADER_AUTOIPSHARPER,	"Sharper Auto Interpolation", gettext_noop("Same as \"autoip\", but when interpolation is done, it is done in a manner that will reduce blurriness if possible.") },
-    { "scale2x", 	SHADER_SCALE2X,    "Scale2x" },
-    { "sabr",		SHADER_SABR,	"SABR v3.0", gettext_noop("GPU-intensive.") },
-    { "ipsharper", 	SHADER_IPSHARPER,  "Sharper bilinear interpolation." },
-    { "ipxnoty", 	SHADER_IPXNOTY,    "Linear interpolation on X axis only." },
-    { "ipynotx", 	SHADER_IPYNOTX,    "Linear interpolation on Y axis only." },
-    { "ipxnotysharper", SHADER_IPXNOTYSHARPER, "Sharper version of \"ipxnoty\"." },
-    { "ipynotxsharper", SHADER_IPYNOTXSHARPER, "Sharper version of \"ipynotx\"." },
-
-    { NULL, 0 },
-};
 
 static std::vector <MDFNSetting> NeoDriverSettings;
 static MDFNSetting DriverSettings[] =
@@ -182,12 +100,6 @@ static MDFNSetting DriverSettings[] =
   { "netplay.port", MDFNSF_NOFLAGS, gettext_noop("Server port."), NULL, MDFNST_UINT, "4046", "1", "65535" },
   { "netplay.smallfont", MDFNSF_NOFLAGS, gettext_noop("Use small(tiny!) font for netplay chat console."), NULL, MDFNST_BOOL, "0" },
 
-  { "video.fs", MDFNSF_NOFLAGS, gettext_noop("Enable fullscreen mode."), NULL, MDFNST_BOOL, "0", },
-  { "video.driver", MDFNSF_NOFLAGS, gettext_noop("Video output method/driver."), NULL, MDFNST_ENUM, "opengl", NULL, NULL, NULL,NULL, VDriver_List },
-  { "video.glvsync", MDFNSF_NOFLAGS, gettext_noop("Attempt to synchronize OpenGL page flips to vertical retrace period."), 
-			       gettext_noop("Note: Additionally, if the environment variable \"__GL_SYNC_TO_VBLANK\" does not exist, then it will be created and set to the value specified for this setting.  This has the effect of forcibly enabling or disabling vblank synchronization when running under Linux with NVidia's drivers."),
-				MDFNST_BOOL, "1" },
-
   { "video.frameskip", MDFNSF_NOFLAGS, gettext_noop("Enable frameskip during emulation rendering."), 
 					gettext_noop("Disable for rendering code performance testing."), MDFNST_BOOL, "1" },
 
@@ -195,8 +107,6 @@ static MDFNSetting DriverSettings[] =
 					gettext_noop("Disable to reduce latency, at the cost of potentially increased video \"juddering\", with the maximum reduction in latency being about 1 video frame's time.\nWill work best with emulated systems that are not very computationally expensive to emulate, combined with running on a relatively fast CPU."),
 					MDFNST_BOOL, "1" },
 
-  { "video.disable_composition", MDFNSF_NOFLAGS, gettext_noop("Attempt to disable desktop composition."), gettext_noop("Currently, this setting only has an effect on Windows Vista and Windows 7(and probably the equivalent server versions as well)."), MDFNST_BOOL, "1" },
-
   { "ffspeed", MDFNSF_NOFLAGS, gettext_noop("Fast-forwarding speed multiplier."), NULL, MDFNST_FLOAT, "4", "1", "15" },
   { "fftoggle", MDFNSF_NOFLAGS, gettext_noop("Treat the fast-forward button as a toggle."), NULL, MDFNST_BOOL, "0" },
   { "ffnosound", MDFNSF_NOFLAGS, gettext_noop("Silence sound output when fast-forwarding."), NULL, MDFNST_BOOL, "0" },
@@ -218,14 +128,17 @@ static MDFNSetting DriverSettings[] =
   { "debugger.autostepmode", MDFNSF_NOFLAGS, gettext_noop("Automatically go into the debugger's step mode after a game is loaded."), NULL, MDFNST_BOOL, "0" },
   #endif
 
-  { "osd.state_display_time", MDFNSF_NOFLAGS, gettext_noop("The length of time, in milliseconds, to display the save state or the movie selector after selecting a state or movie."),  NULL, MDFNST_UINT, "2000", "0", "15000" },
+  { "osd.message_display_time", MDFNSF_NOFLAGS, gettext_noop("Length of time, in milliseconds, to display internal status and error messages"), gettext_noop("Time lengths less than 100ms are recommended against unless you understand you may miss important non-fatal error messages, and that the input configuration process may become unusable."), MDFNST_UINT, "2500", "0", "15000" },
+  { "osd.state_display_time", MDFNSF_NOFLAGS, gettext_noop("Length of time, in milliseconds, to display the save state or the movie selector after selecting a state or movie."),  NULL, MDFNST_UINT, "2000", "0", "15000" },
   { "osd.alpha_blend", MDFNSF_NOFLAGS, gettext_noop("Enable alpha blending for OSD elements."), NULL, MDFNST_BOOL, "1" },
+
+  { "srwautoenable", MDFNSF_SUPPRESS_DOC, gettext_noop("DO NOT USE UNLESS YOU'RE A SPACE GOAT"/*"Automatically enable state rewinding functionality on game load."*/), gettext_noop("Use this setting with caution, as save state rewinding can have widely variable memory and CPU usage requirements among different games and different emulated systems."), MDFNST_BOOL, "0" },
 };
 
-static void BuildSystemSetting(MDFNSetting *setting, const char *system_name, const char *name, const char *description, const char *description_extra, MDFNSettingType type, 
-	const char *default_value, const char *minimum = NULL, const char *maximum = NULL,
-	bool (*validate_func)(const char *name, const char *value) = NULL, void (*ChangeNotification)(const char *name) = NULL, 
-        const MDFNSetting_EnumList *enum_list = NULL)
+void BuildSystemSetting(MDFNSetting *setting, const char *system_name, const char *name, const char *description, const char *description_extra, MDFNSettingType type, 
+	const char *default_value, const char *minimum, const char *maximum,
+	bool (*validate_func)(const char *name, const char *value), void (*ChangeNotification)(const char *name), 
+        const MDFNSetting_EnumList *enum_list)
 {
  char setting_name[256];
 
@@ -256,35 +169,6 @@ static const MDFNSetting_EnumList DisFontSize_List[] =
  { NULL, 0 },
 };
 
-static const MDFNSetting_EnumList StretchMode_List[] =
-{
- { "0", 0, gettext_noop("Disabled") },
- { "off", 0 },
-
- { "1", 1 },
- { "full", 1, gettext_noop("Full"), gettext_noop("Full-screen stretch, disregarding aspect ratio.") },
-
- { "2", 2 },
- { "aspect", 2, gettext_noop("Aspect Preserve"), gettext_noop("Full-screen stretch as far as the aspect ratio(in this sense, the equivalent xscalefs == yscalefs) can be maintained.") },
-
- { "aspect_int", 3, gettext_noop("Aspect Preserve + Integer Scale"), gettext_noop("Full-screen stretch, same as \"aspect\" except that the equivalent xscalefs and yscalefs are rounded down to the nearest integer.") },
- { "aspect_mult2", 4, gettext_noop("Aspect Preserve + Integer Multiple-of-2 Scale"), gettext_noop("Full-screen stretch, same as \"aspect_int\", but rounds down to the nearest multiple of 2.") },
-
- { NULL, 0 },
-};
-
-static const MDFNSetting_EnumList VideoIP_List[] =
-{
- { "0", VIDEOIP_OFF, gettext_noop("Disabled") },
-
- { "1", VIDEOIP_BILINEAR, gettext_noop("Bilinear") },
-
- // Disabled until a fix can be made for rotation.
- { "x", VIDEOIP_LINEAR_X, gettext_noop("Linear (X)"), gettext_noop("Interpolation only on the X axis.") },
- { "y", VIDEOIP_LINEAR_Y, gettext_noop("Linear (Y)"), gettext_noop("Interpolation only on the Y axis.") },
-
- { NULL, 0 },
-};
 
 void MakeDebugSettings(std::vector <MDFNSetting> &settings)
 {
@@ -307,84 +191,6 @@ void MakeDebugSettings(std::vector <MDFNSetting> &settings)
  #endif
 }
 
-void MakeVideoSettings(std::vector <MDFNSetting> &settings)
-{
- for(unsigned int i = 0; i < MDFNSystems.size() + 1; i++)
- {
-  int nominal_width;
-  int nominal_height;
-  bool multires;
-  const char *sysname;
-  char default_value[256];
-  MDFNSetting setting;
-  const int default_xres = 0, default_yres = 0;
-  const double default_scalefs = 1.0;
-  double default_scale;
-
-  if(i == MDFNSystems.size())
-  {
-   nominal_width = 384;
-   nominal_height = 240;
-   multires = FALSE;
-   sysname = "player";
-  }
-  else
-  {
-   nominal_width = MDFNSystems[i]->nominal_width;
-   nominal_height = MDFNSystems[i]->nominal_height;
-   multires = MDFNSystems[i]->multires;
-   sysname = (const char *)MDFNSystems[i]->shortname;
-  }
-
-  if(multires)
-   default_scale = ceil(1024 / nominal_width);
-  else
-   default_scale = ceil(768 / nominal_width);
-
-  if(default_scale * nominal_width > 1024)
-   default_scale--;
-
-  if(!default_scale)
-   default_scale = 1;
-
-  trio_snprintf(default_value, 256, "%d", default_xres);
-  BuildSystemSetting(&setting, sysname, "xres", CSD_xres, CSDE_xres, MDFNST_UINT, strdup(default_value), "0", "65536");
-  settings.push_back(setting);
-
-  trio_snprintf(default_value, 256, "%d", default_yres);
-  BuildSystemSetting(&setting, sysname, "yres", CSD_yres, CSDE_yres, MDFNST_UINT, strdup(default_value), "0", "65536");
-  settings.push_back(setting);
-
-  trio_snprintf(default_value, 256, "%f", default_scale);
-  BuildSystemSetting(&setting, sysname, "xscale", CSD_xscale, NULL, MDFNST_FLOAT, strdup(default_value), "0.01", "256");
-  settings.push_back(setting);
-  BuildSystemSetting(&setting, sysname, "yscale", CSD_yscale, NULL, MDFNST_FLOAT, strdup(default_value), "0.01", "256");
-  settings.push_back(setting);
-
-  trio_snprintf(default_value, 256, "%f", default_scalefs);
-  BuildSystemSetting(&setting, sysname, "xscalefs", CSD_xscalefs, CSDE_xyscalefs, MDFNST_FLOAT, strdup(default_value), "0.01", "256");
-  settings.push_back(setting);
-  BuildSystemSetting(&setting, sysname, "yscalefs", CSD_yscalefs, CSDE_xyscalefs, MDFNST_FLOAT, strdup(default_value), "0.01", "256");
-  settings.push_back(setting);
-
-  BuildSystemSetting(&setting, sysname, "scanlines", CSD_scanlines, CSDE_scanlines, MDFNST_INT, "0", "-100", "100");
-  settings.push_back(setting);
-
-  BuildSystemSetting(&setting, sysname, "stretch", CSD_stretch, NULL, MDFNST_ENUM, "aspect_mult2", NULL, NULL, NULL, NULL, StretchMode_List);
-  settings.push_back(setting);
-
-  BuildSystemSetting(&setting, sysname, "videoip", CSD_videoip, NULL, MDFNST_ENUM, multires ? "1" : "0", NULL, NULL, NULL, NULL, VideoIP_List);
-  settings.push_back(setting);
-
-  BuildSystemSetting(&setting, sysname, "special", CSD_special, CSDE_special, MDFNST_ENUM, "none", NULL, NULL, NULL, NULL, Special_List);
-  settings.push_back(setting);
-
-  BuildSystemSetting(&setting, sysname, "pixshader", CSD_pixshader, CSDE_pixshader, MDFNST_ENUM, "none", NULL, NULL, NULL, NULL, Pixshader_List);
-  settings.push_back(setting);
- }
-
-}
-
 static MDFN_Thread* GameThread;
 static MDFN_Surface *VTBuffer[2] = { NULL, NULL };
 static int32 *VTLineWidths[2] = { NULL, NULL };
@@ -450,29 +256,34 @@ void MDFND_Message(const char *s)
  }
 }
 
-// CreateDirs should make sure errno is intact after calling mkdir() if it fails.
-static bool CreateDirs(void)
+static void CreateDirs(void)
 {
- const char *subs[7] = { "mcs", "mcm", "snaps", "palettes", "sav", "cheats", "firmware" };
- char *tdir;
+ static const char *subs[] = { "mcs", "mcm", "snaps", "palettes", "sav", "cheats", "firmware", "pgconfig" };
 
- if(MDFN_mkdir(DrBaseDirectory, S_IRWXU) == -1 && errno != EEXIST)
+ try
  {
-  return(FALSE);
+  MDFN_mkdir_T(DrBaseDirectory);
+ }
+ catch(MDFN_Error &e)
+ {
+  if(e.GetErrno() != EEXIST)
+   throw;
  }
 
- for(unsigned int x = 0; x < sizeof(subs) / sizeof(const char *); x++)
+ for(auto const& s : subs)
  {
-  tdir = trio_aprintf("%s" PSS "%s",DrBaseDirectory,subs[x]);
-  if(MDFN_mkdir(tdir, S_IRWXU) == -1 && errno != EEXIST)
+  std::string tdir = std::string(DrBaseDirectory) + std::string(PSS) + std::string(s);
+
+  try
   {
-   free(tdir);
-   return(FALSE);
+   MDFN_mkdir_T(tdir.c_str());
+  }
+  catch(MDFN_Error &e)
+  {
+   if(e.GetErrno() != EEXIST)
+    throw;
   }
-  free(tdir);
  }
-
- return(TRUE);
 }
 
 #if defined(HAVE_SIGNAL) || defined(HAVE_SIGACTION)
@@ -606,6 +417,141 @@ static void CloseStuff(int signum)
 }
 #endif
 
+//
+//
+//
+#include <mednafen/FileStream.h>
+#include <mednafen/compress/GZFileStream.h>
+static void Stream64Test(const char* path)
+{
+ try
+ {
+ {
+  FileStream fp(path, FileStream::MODE_WRITE_SAFE);
+
+  assert(fp.tell() == 0);
+  assert(fp.size() == 0);
+  fp.put_BE<uint32>(0xDEADBEEF);
+  assert(fp.tell() == 4);
+  assert(fp.size() == 4);
+
+  fp.seek((uint64)8192 * 1024 * 1024, SEEK_SET);
+  fp.put_BE<uint32>(0xCAFEBABE);
+  assert(fp.tell() == (uint64)8192 * 1024 * 1024 + 4);
+  assert(fp.size() == (uint64)8192 * 1024 * 1024 + 4);
+
+  fp.put_BE<uint32>(0xAAAAAAAA);
+  assert(fp.tell() == (uint64)8192 * 1024 * 1024 + 8);
+  assert(fp.size() == (uint64)8192 * 1024 * 1024 + 8);
+
+  fp.truncate((uint64)8192 * 1024 * 1024 + 4);
+  assert(fp.size() == (uint64)8192 * 1024 * 1024 + 4);
+
+  fp.seek(-((uint64)8192 * 1024 * 1024 + 8), SEEK_CUR);
+  assert(fp.tell() == 0);
+  fp.seek((uint64)-4, SEEK_END);
+  assert(fp.tell() == (uint64)8192 * 1024 * 1024);
+ }
+
+ {
+  FileStream fp(path, FileStream::MODE_READ);
+  uint32 tmp;
+
+  assert(fp.size() == (uint64)8192 * 1024 * 1024 + 4);
+  tmp = fp.get_LE<uint32>();
+  assert(tmp == 0xEFBEADDE);
+  fp.seek((uint64)8192 * 1024 * 1024 - 4, SEEK_CUR);
+  tmp = fp.get_LE<uint32>(); 
+  assert(tmp == 0xBEBAFECA);
+ }
+
+ {
+  GZFileStream fp(path, GZFileStream::MODE::READ);
+  uint32 tmp;
+
+  tmp = fp.get_LE<uint32>();
+  assert(tmp == 0xEFBEADDE);
+  fp.seek((uint64)8192 * 1024 * 1024 - 4, SEEK_CUR);
+  tmp = fp.get_LE<uint32>(); 
+  assert(tmp == 0xBEBAFECA);  
+  assert(fp.tell() == (uint64)8192 * 1024 * 1024 + 4);
+ }
+ }
+ catch(std::exception& e)
+ {
+  printf("%s\n", e.what());
+  abort();
+ }
+}
+//
+//
+//
+#include <mednafen/cdrom/cdromif.h>
+static void CDTest(const char* path)
+{
+ try
+ {
+  CDIF* cds[2];
+  CDUtility::TOC toc[2];
+
+  cds[0] = CDIF_Open(path, false);
+  cds[1] = CDIF_Open(path, true);
+
+  for(unsigned i = 0; i < 2; i++)
+   cds[0]->ReadTOC(&toc[i]);
+
+  assert(!memcmp(&toc[0], &toc[1], sizeof(CDUtility::TOC)));
+
+  srand(0xDEADBEEF);
+
+  for(int32 lba = -150; lba < (int32)toc[0].tracks[100].lba + 5*60*75; lba++)
+  {
+   uint8 secbuf[2][2352 + 96];
+   uint8 pwobuf[2][96];
+
+   for(unsigned i = 0; i < 2; i++)
+   {
+    for(unsigned sbj = 0; sbj < 2352 + 96; sbj++)
+     secbuf[i][sbj] = rand() >> 8;
+
+    for(unsigned sbj = 0; sbj < 96; sbj++)
+     pwobuf[i][sbj] = rand() >> 8;
+
+    cds[i]->ReadRawSector(secbuf[i], lba);
+    cds[i]->ReadRawSectorPWOnly(pwobuf[i], lba, true);
+
+    for(unsigned p = 0; p < 96; p++)
+     assert(secbuf[i][2352 + p] == pwobuf[i][p]);
+   }
+   assert(!memcmp(secbuf[0], secbuf[1], 2352 + 96));
+   assert(!memcmp(pwobuf[0], pwobuf[1], 96));
+
+   uint8 subq[12];
+   CDUtility::subq_deinterleave(pwobuf[0], subq);
+   if(CDUtility::subq_check_checksum(subq))
+   {
+    /*if(lba == 0 || lba == 1)
+    {
+     for(unsigned i = 0; i < 12; i++)
+      printf("0x%02x ", subq[i]);
+     printf("\n");
+    }*/
+   }
+   else
+    printf("SubQ checksum error at lba=%d\n", lba);
+  }
+ }
+ catch(std::exception& e)
+ {
+  printf("%s\n", e.what());
+  abort();
+ }
+
+ printf("CDTest Done.\n");
+}
+//
+//
+//
 static ARGPSTRUCT *MDFN_Internal_Args = NULL;
 
 static int HokeyPokeyFallDown(const char *name, const char *value)
@@ -643,7 +589,7 @@ static void MakeMednafenArgsStruct(void)
  for(sit = settings->begin(); sit != settings->end(); sit++)
  {
   MDFN_Internal_Args[x].name = strdup(sit->second.name);
-  MDFN_Internal_Args[x].description = _(sit->second.desc->description);
+  MDFN_Internal_Args[x].description = sit->second.desc->description ? _(sit->second.desc->description) : NULL;
   MDFN_Internal_Args[x].var = NULL;
   MDFN_Internal_Args[x].subs = (void *)HokeyPokeyFallDown;
   MDFN_Internal_Args[x].substype = SUBSTYPE_FUNCTION;
@@ -656,7 +602,6 @@ static void MakeMednafenArgsStruct(void)
 
 static int netconnect = 0;
 static char* loadcd = NULL;	// Deprecated
-static int physcd = 0;
 
 static char * force_module_arg = NULL;
 static int DoArgs(int argc, char *argv[], char **filename)
@@ -666,6 +611,8 @@ static int DoArgs(int argc, char *argv[], char **filename)
 	char *dsfn = NULL;
 	char *dmfn = NULL;
 	char *dummy_remote = NULL;
+	char *stream64testpath = NULL;
+	char *cdtestpath = NULL;
 
         ARGPSTRUCT MDFNArgs[] = 
 	{
@@ -675,8 +622,6 @@ static int DoArgs(int argc, char *argv[], char **filename)
 	 // -loadcd is deprecated and only still supported because it's been around for yeaaaars.
 	 { "loadcd", NULL/*_("Load and boot a CD for the specified system.")*/, 0, &loadcd, SUBSTYPE_STRING_ALLOC },
 
-	 { "physcd", _("Load and boot from a physical CD, treating [FILE] as the optional device name."), &physcd, 0, 0 },
-
 	 { "force_module", _("Force usage of specified emulation module."), 0, &force_module_arg, SUBSTYPE_STRING_ALLOC },
 
 	 { "soundrecord", _("Record sound output to the specified filename in the MS WAV format."), 0,&soundrecfn, SUBSTYPE_STRING_ALLOC },
@@ -689,6 +634,11 @@ static int DoArgs(int argc, char *argv[], char **filename)
 
 	 { "connect", _("Connect to the remote server and start network play."), &netconnect, 0, 0 },
 
+	 // Testing functionality for FileStream and GZFileStream largefile support(mostly intended for testing the Windows builds)
+	 { "stream64test", NULL, 0, &stream64testpath, SUBSTYPE_STRING_ALLOC },
+
+	 { "cdtest", NULL, 0, &cdtestpath, SUBSTYPE_STRING_ALLOC },
+
 	 { 0, 0, 0, 0 }
         };
 
@@ -723,6 +673,20 @@ static int DoArgs(int argc, char *argv[], char **filename)
 	  return(0);
 	 }
 
+	 if(stream64testpath)
+	 {
+	  Stream64Test(stream64testpath);
+	  free(stream64testpath);
+	  stream64testpath = NULL;
+	 }
+
+	 if(cdtestpath)
+	 {
+	  CDTest(cdtestpath);
+	  free(cdtestpath);
+	  cdtestpath = NULL;
+	 }
+
 	 if(dsfn)
 	  MDFNI_DumpSettingsDef(dsfn);
 
@@ -732,7 +696,7 @@ static int DoArgs(int argc, char *argv[], char **filename)
 	 if(dsfn || dmfn)
 	  return(0);
 
-	 if(*filename == NULL && loadcd == NULL && physcd == 0)
+	 if(*filename == NULL)
 	 {
 	  MDFN_PrintError(_("No game filename specified!"));
 	  return(0);
@@ -762,46 +726,21 @@ static int LoadGame(const char *force_module, const char *path)
 	pending_snapshot = 0;
 	pending_ssnapshot = 0;
 
-	if(physcd)
+	if(loadcd)	// Deprecated
 	{
-	 if(!(tmp = MDFNI_LoadCD(force_module, path, true)))
-		return(0);
-	}
-	else if(loadcd)	// Deprecated
-	{
-	 const char *system = loadcd;
-         bool is_physical = false;
-
-	 if(!system)
-	  system = force_module;
-
-	 if(path == NULL)
-	  is_physical = true;
-	 else
-	 {
-	  int sr;
-	  //int se;
-	  struct stat sb;
-
-	  sr = stat(path, &sb);
-	  //se = errno;	  
-
-	  if(!sr && !S_ISREG(sb.st_mode))
-	  //if(sr || !S_ISREG(sb.st_mode))
-	   is_physical = true;
-	 }
-
-	 if(!(tmp = MDFNI_LoadCD(system, path, is_physical)))
-		return(0);
+	 if(!(tmp = MDFNI_LoadCD(loadcd ? loadcd : force_module, path)))
+	  return(0);
 	}
 	else
 	{
          if(!(tmp=MDFNI_LoadGame(force_module, path)))
 	  return 0;
 	}
+
 	CurGame = tmp;
 	InitGameInput(tmp);
 	InitCommandInput(tmp);
+	RMDUI_Init(tmp);
 
         RefreshThrottleFPS(1);
 
@@ -858,6 +797,12 @@ static int LoadGame(const char *force_module, const char *path)
         }
 
 	ffnosound = MDFN_GetSettingB("ffnosound");
+	RewindState = MDFN_GetSettingB("srwautoenable");
+	if(RewindState)
+	{
+	 MDFN_DispMessage(_("State rewinding functionality enabled."));
+	 MDFNI_EnableStateRewind(RewindState);
+	}
 
 	//
 	// Game thread creation should come lastish.
@@ -892,6 +837,7 @@ int CloseGame(void)
 
 	MDFNI_CloseGame();
 
+	RMDUI_Kill();
 	KillCommandInput();
         KillGameInput();
 	Sound_Kill();
@@ -902,7 +848,7 @@ int CloseGame(void)
 }
 
 static void GameThread_HandleEvents(void);
-static int volatile NeedExitNow = 0;
+static int volatile NeedExitNow = 0;	// Set 'true' in various places, including signal handler.
 double CurGameSpeed = 1;
 
 void MainRequestExit(void)
@@ -910,6 +856,11 @@ void MainRequestExit(void)
  NeedExitNow = 1;
 }
 
+bool MainExitPending(void)	// Called from netplay code, so we can break out of blocking loops after receiving a signal.
+{
+ return (bool)NeedExitNow;
+}
+
 static bool InFrameAdvance = 0;
 static bool NeedFrameAdvance = 0;
 
@@ -1342,11 +1293,6 @@ void PumpWrap(void)
 			{
 			 SendCEvent_to_GT(CEVT_SET_INPUT_FOCUS, (char*)0 + (bool)event.active.gain, NULL);
 			}
-
-			if(event.active.state & SDL_APPACTIVE)
-			{
-			 VideoAppActive((bool)(event.active.gain));
-			}
 			break;
 
    case SDL_SYSWMEVENT: break;
@@ -1370,13 +1316,13 @@ void PumpWrap(void)
 		     }
 		     break;
 	         case CEVT_SET_GRAB_INPUT:
-                         SDL_WM_GrabInput(*(int *)event.user.data1 ? SDL_GRAB_ON : SDL_GRAB_OFF);
+                         SDL_WM_GrabInput(*(uint8 *)event.user.data1 ? SDL_GRAB_ON : SDL_GRAB_OFF);
                          free(event.user.data1);
                          break;
 		 //case CEVT_TOGGLEFS: NeedVideoChange = 1; break;
 		 //case CEVT_VIDEOSYNC: NeedVideoChange = -1; break;
 		 case CEVT_SHOWCURSOR: SDL_ShowCursor(*(int *)event.user.data1); free(event.user.data1); break;
-	  	 case CEVT_DISP_MESSAGE: VideoShowMessage((UTF8*)event.user.data1); break;
+	  	 case CEVT_DISP_MESSAGE: VideoShowMessage((char*)event.user.data1); break;
 		 default: 
 			if(numevents < gtevents_size)
 			{
@@ -1421,8 +1367,32 @@ void PrintCompilerVersion(void)
  #if defined(__GNUC__)
   MDFN_printf(_("Compiled with gcc %s\n"), __VERSION__);
  #endif
+
+ #ifdef HAVE___MINGW_GET_CRT_INFO
+  MDFN_printf(_("Running with %s\n"), __mingw_get_crt_info());
+ #endif
 }
 
+#if 0
+#include <vector>	// To make sure we pick up c++config.h if it's there
+void PrintGLIBCXXInfo(void)
+{
+ #if defined(__GLIBCXX__)
+  MDFN_printf(_("Compiled with GNU libstdc++ %lu\n"), (unsigned long)__GLIBCXX__);
+  {
+   MDFN_AutoIndent aind(1);
+   const char* sjljresp;
+   #if defined(_GLIBCXX_SJLJ_EXCEPTIONS)
+    sjljresp = _("Yes");
+   #else
+    sjljresp = _("No");
+   #endif
+   MDFN_printf(_("Using SJLJ Exceptions: %s\n"), sjljresp);
+  }
+ #endif
+}
+#endif
+
 void PrintSDLVersion(void)
 {
  const SDL_version *sver = SDL_Linked_Version();
@@ -1441,26 +1411,22 @@ void PrintLIBSNDFILEVersion(void)
  #endif
 }
 
+#include <zlib.h>
 void PrintZLIBVersion(void)
 {
  #ifdef ZLIB_VERSION
-  MDFN_printf(_("Compiled against zlib %s, running with zlib %s\n"), ZLIB_VERSION, zlibVersion());
+  MDFN_printf(_("Compiled against zlib %s, running with zlib %s(flags=0x%08lx)\n"), ZLIB_VERSION, zlibVersion(), (unsigned long)zlibCompileFlags());
  #endif
 }
 
-void PrintLIBCDIOVersion(void)
+void PrintLIBICONVVersion(void)
 {
- #ifdef HAVE_LIBCDIO
-  #if LIBCDIO_VERSION_NUM < 83
-  const char *cdio_version_string = "(unknown)";
-  #endif
-
-  MDFN_printf(_("Compiled against libcdio %s, running with libcdio %s\n"), CDIO_VERSION, cdio_version_string);
+ #ifdef _LIBICONV_VERSION
+  MDFN_printf(_("Compiled against libiconv %u.%u, running with libiconv %u.%u\n"), _LIBICONV_VERSION & 0xFF, _LIBICONV_VERSION >> 8,
+										   _libiconv_version & 0xFF, _libiconv_version >> 8);
  #endif
 }
 
-int sdlhaveogl = 0;
-
 //#include <sched.h>
 
 #if 0//#ifdef WIN32
@@ -1609,6 +1575,49 @@ static void ThreadTest(void)
 
 #endif
 
+static bool HandleVideoChange(void)
+{
+ if(NeedVideoChange == -1)
+ {
+  try
+  {
+   Video_Init(CurGame);
+  }
+  catch(std::exception &e)
+  {
+   MDFND_PrintError(e.what());
+   return(false);
+  }
+ }
+ else
+ {
+  bool original_fs_setting = MDFN_GetSettingB("video.fs");
+
+  try
+  {
+   MDFNI_SetSettingB("video.fs", !original_fs_setting);
+   Video_Init(CurGame);
+  }
+  catch(std::exception &e)
+  {
+   MDFND_PrintError(e.what());
+
+   try
+   {
+    MDFNI_SetSettingB("video.fs", original_fs_setting);
+    Video_Init(CurGame);	    
+   }
+   catch(std::exception &ne)
+   {
+    MDFND_PrintError(ne.what());
+    return(false);
+   }
+  }
+ }
+
+ return(true);
+}
+
 
 int main(int argc, char *argv[])
 {
@@ -1687,10 +1696,11 @@ int main(int argc, char *argv[])
         MDFN_printf(_("Build information:\n"));
         MDFN_indent(2);
         PrintCompilerVersion();
+	//PrintGLIBCXXInfo();
         PrintZLIBVersion();
+	PrintLIBICONVVersion();
         PrintSDLVersion();
         PrintLIBSNDFILEVersion();
-	PrintLIBCDIOVersion();
         MDFN_indent(-2);
 
         MDFN_printf(_("Base directory: %s\n"), DrBaseDirectory);
@@ -1721,7 +1731,7 @@ int main(int argc, char *argv[])
 	 NeoDriverSettings.push_back(DriverSettings[x]);
 
 	MakeDebugSettings(NeoDriverSettings);
-	MakeVideoSettings(NeoDriverSettings);
+	Video_MakeSettings(NeoDriverSettings);
 	MakeInputSettings(NeoDriverSettings);
 
         if(!MDFNI_Initialize(DrBaseDirectory, NeoDriverSettings))
@@ -1733,11 +1743,13 @@ int main(int argc, char *argv[])
         SetSignals(CloseStuff);
         #endif
 
-	if(!CreateDirs())
+	try
 	{
-	 ErrnoHolder ene(errno);	// TODO: Maybe we should have CreateDirs() return this instead?
-
-	 MDFN_PrintError(_("Error creating directories: %s\n"), ene.StrError());
+	 CreateDirs();
+	}
+	catch(std::exception &e)
+	{
+	 MDFN_PrintError(_("Error creating directories: %s\n"), e.what());
 	 MDFNI_Kill();
 	 return(-1);
 	}
@@ -1762,7 +1774,7 @@ int main(int argc, char *argv[])
 	*/
 	int ret = 0;
 
-	//InitVideo(NULL);
+	//Video_Init(NULL);
 
 	VTMutex = MDFND_CreateMutex();
         EVMutex = MDFND_CreateMutex();
@@ -1829,36 +1841,24 @@ for(int zgi = 1; zgi < argc; zgi++)// start game load test loop
 	 NeedExitNow = 1;
 	}
 
-	while(!NeedExitNow)
+	while(MDFN_LIKELY(!NeedExitNow))
 	{
 	 bool DidVideoChange = false;
 
 	 MDFND_LockMutex(VTMutex);	/* Lock mutex */
 
-         if(NeedVideoChange)
+         if(MDFN_UNLIKELY(NeedVideoChange))
          {
-          KillVideo();
+          Video_Kill();
 
-          if(NeedVideoChange == -1)
-          {
-           if(!InitVideo(CurGame))
-           {
-	    ret = -1;
-            NeedExitNow = 1;
-	    MDFND_UnlockMutex(VTMutex);   /* Unlock mutex */
-            break;
-           }
-          }
-          else
-          {
-           MDFNI_SetSettingB("video.fs", !MDFN_GetSettingB("video.fs"));
-
-           if(!InitVideo(CurGame))
-           {
-            MDFNI_SetSettingB("video.fs", !MDFN_GetSettingB("video.fs"));
-            InitVideo(CurGame);
-           }
-          }
+	  if(!HandleVideoChange())
+	  {
+	   ret = -1;
+           NeedExitNow = 1;
+           NeedVideoChange = 0;
+	   MDFND_UnlockMutex(VTMutex);   /* Unlock mutex */
+	   break;
+	  }
 
 	  DidVideoChange = true;
           NeedVideoChange = 0;
@@ -1921,7 +1921,7 @@ for(int zgi = 1; zgi < argc; zgi++)// start game load test loop
 	delete joy_manager;
 	joy_manager = NULL;
 
-	KillVideo();
+	Video_Kill();
 
 	SDL_Quit();
 
@@ -2090,17 +2090,17 @@ bool MDFND_Update(MDFN_Surface *surface, MDFN_Rect *rect, int32 *lw, int Interla
  return(ret);
 }
 
-void MDFND_DispMessage(UTF8 *text)
+void MDFND_DispMessage(char* text)
 {
  SendCEvent(CEVT_DISP_MESSAGE, text, NULL);
 }
 
-void MDFND_SetStateStatus(StateStatusStruct *status)
+void MDFND_SetStateStatus(StateStatusStruct *status) noexcept
 {
  SendCEvent(CEVT_SET_STATE_STATUS, status, NULL);
 }
 
-void MDFND_SetMovieStatus(StateStatusStruct *status)
+void MDFND_SetMovieStatus(StateStatusStruct *status) noexcept
 {
  SendCEvent(CEVT_SET_MOVIE_STATUS, status, NULL);
 }
