@@ -12,18 +12,18 @@ class MDFN_Error : public std::exception
 {
  public:
 
- MDFN_Error() throw();
+ MDFN_Error() noexcept;
 
- MDFN_Error(int errno_code_new, const char *format, ...) throw() MDFN_FORMATSTR(gnu_printf, 3, 4);
+ MDFN_Error(int errno_code_new, const char *format, ...) noexcept MDFN_FORMATSTR(gnu_printf, 3, 4);
  MDFN_Error(const ErrnoHolder &enh);
 
- ~MDFN_Error() throw();
+ ~MDFN_Error() noexcept;
 
- MDFN_Error(const MDFN_Error &ze_error) throw();
- MDFN_Error & operator=(const MDFN_Error &ze_error) throw();
+ MDFN_Error(const MDFN_Error &ze_error) noexcept;
+ MDFN_Error & operator=(const MDFN_Error &ze_error) noexcept;
 
- virtual const char *what(void) const throw();
- int GetErrno(void) const throw();
+ virtual const char *what(void) const noexcept;
+ int GetErrno(void) const noexcept;
 
  private:
 
