@@ -16,13 +16,12 @@
 // along with this program; if not, write to the Free Software Foundation,
 // Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
-#include "../mednafen.h"
+#include <mednafen/mednafen.h>
 #include "GBA.h"
 #include "Globals.h"
 #include "Port.h"
 
 #include <time.h>
-#include <memory.h>
 
 namespace MDFN_IEN_GBA
 {
