@@ -41,8 +41,8 @@ found freely through public domain sources.
 //////////////////////////////////////////////////////////
 // CPU routines
 
-#include "mednafen/mednafen.h"
-#include <mednafen/masmem.h>
+#include <mednafen/mednafen.h>
+#include <trio/trio.h>
 
 //#include "pcfx.h"
 //#include "debug.h"
@@ -662,7 +662,7 @@ void V810::Run_Accurate_Debug(int32 MDFN_FASTCALL (*event_handler)(const v810_ti
 //
 #define RB_GETPC()      	((uint32)(PC_ptr - PC_base))
 
-#define RB_RDOP(PC_offset, ...) LoadU16_LE((uint16 *)&PC_ptr[PC_offset])
+#define RB_RDOP(PC_offset, ...) MDFN_de16lsb<true>(&PC_ptr[PC_offset])
 
 void V810::Run_Fast(int32 MDFN_FASTCALL (*event_handler)(const v810_timestamp_t timestamp))
 {
@@ -736,6 +736,59 @@ void V810::SetCPUHook(void (*newhook)(const v810_timestamp_t timestamp, uint32 P
 }
 #endif
 
+uint32 V810::GetRegister(unsigned int which, char *special, const uint32 special_len)
+{
+ if(which >= GSREG_PR && which <= GSREG_PR + 31)
+ {
+  return GetPR(which - GSREG_PR);
+ }
+ else if(which >= GSREG_SR && which <= GSREG_SR + 31)
+ {
+  uint32 val = GetSREG(which - GSREG_SR);
+
+  if(special && which == GSREG_SR + PSW)
+  {
+   trio_snprintf(special, special_len, "Z: %d, S: %d, OV: %d, CY: %d, ID: %d, AE: %d, EP: %d, NP: %d, IA: %2d",
+	(int)(bool)(val & PSW_Z), (int)(bool)(val & PSW_S), (int)(bool)(val & PSW_OV), (int)(bool)(val & PSW_CY),
+	(int)(bool)(val & PSW_ID), (int)(bool)(val & PSW_AE), (int)(bool)(val & PSW_EP), (int)(bool)(val & PSW_NP),
+	(val & PSW_IA) >> 16);
+  }
+
+  return val;
+ }
+ else if(which == GSREG_PC)
+ {
+  return GetPC();
+ }
+ else if(which == GSREG_TIMESTAMP)
+ {
+  return v810_timestamp;
+ }
+
+ return 0xDEADBEEF;
+}
+
+void V810::SetRegister(unsigned int which, uint32 value)
+{
+ if(which >= GSREG_PR && which <= GSREG_PR + 31)
+ {
+  if(which)
+   P_REG[which - GSREG_PR] = value;
+ }
+ else if(which >= GSREG_SR && which <= GSREG_SR + 31)
+ {
+  // SetSREG(timestamp, which - GSREG_SR, value);
+ }
+ else if(which == GSREG_PC)
+ {
+  SetPC(value & ~1);
+ }
+ else if(which == GSREG_TIMESTAMP)
+ {
+  //v810_timestamp = value;
+ }
+}
+
 uint32 V810::GetPC(void)
 {
  if(EmuMode == V810_EMU_MODE_ACCURATE)
@@ -757,37 +810,6 @@ void V810::SetPC(uint32 new_pc)
  }
 }
 
-uint32 V810::GetPR(const unsigned int which)
-{
- assert(which <= 0x1F);
-
-
- return(which ? P_REG[which] : 0);
-}
-
-void V810::SetPR(const unsigned int which, uint32 value)
-{
- assert(which <= 0x1F);
-
- if(which)
-  P_REG[which] = value;
-}
-
-uint32 V810::GetSR(const unsigned int which)
-{
- assert(which <= 0x1F);
-
- return(GetSREG(which));
-}
-
-void V810::SetSR(const unsigned int which, uint32 value)
-{
- assert(which <= 0x1F);
-
-// SetSREG(timestamp, which, value);
-}
-
-
 #define BSTR_OP_MOV dst_cache &= ~(1 << dstoff); dst_cache |= ((src_cache >> srcoff) & 1) << dstoff;
 #define BSTR_OP_NOT dst_cache &= ~(1 << dstoff); dst_cache |= (((src_cache >> srcoff) & 1) ^ 1) << dstoff;
 
@@ -1426,32 +1448,20 @@ void V810::Exception(uint32 handler, uint16 eCode)
     }
 }
 
-int V810::StateAction(StateMem *sm, int load, int data_only)
+void V810::StateAction(StateMem *sm, const unsigned load, const bool data_only)
 {
- uint32 *cache_tag_temp = NULL;
- uint32 *cache_data_temp = NULL;
- bool *cache_data_valid_temp = NULL;
+ PODFastVector<uint32> cache_tag_temp;
+ PODFastVector<uint32> cache_data_temp;
+ PODFastVector<bool> cache_data_valid_temp;
+
  uint32 PC_tmp = GetPC();
 
  if(EmuMode == V810_EMU_MODE_ACCURATE)
  {
-  cache_tag_temp = (uint32 *)malloc(sizeof(uint32 *) * 128);
-  cache_data_temp = (uint32 *)malloc(sizeof(uint32 *) * 128 * 2);
-  cache_data_valid_temp = (bool *)malloc(sizeof(bool *) * 128 * 2);
+  cache_tag_temp.resize(128);
+  cache_data_temp.resize(128 * 2);
+  cache_data_valid_temp.resize(128 * 2);
 
-  if(!cache_tag_temp || !cache_data_temp || !cache_data_valid_temp)
-  {
-   if(cache_tag_temp)
-    free(cache_tag_temp);
-
-   if(cache_data_temp)
-    free(cache_data_temp);
-
-   if(cache_data_valid_temp)
-    free(cache_data_valid_temp);
-
-   return(0);
-  }
   if(!load)
   {
    for(int i = 0; i < 128; i++)
@@ -1469,9 +1479,9 @@ int V810::StateAction(StateMem *sm, int load, int data_only)
        // in case the save state was saved while in fast mode
        // and the cache data isn't present and thus won't be loaded.
   {
-   memset(cache_tag_temp, 0, sizeof(uint32) * 128);
-   memset(cache_data_temp, 0, sizeof(uint32) * 128 * 2);
-   memset(cache_data_valid_temp, 0, sizeof(bool) * 128 * 2);
+   cache_tag_temp.fill(0);
+   cache_data_temp.fill(0);
+   cache_data_valid_temp.fill(false);
   }
  }
 
@@ -1486,9 +1496,9 @@ int V810::StateAction(StateMem *sm, int load, int data_only)
 
   SFVAR(lastop),
 
-  SFARRAY32(cache_tag_temp, 128),
-  SFARRAY32(cache_data_temp, 128 * 2),
-  SFARRAYB(cache_data_valid_temp, 128 * 2),
+  SFARRAY32N(&cache_tag_temp[0], cache_tag_temp.size(), "cache_tag_temp"),
+  SFARRAY32N(&cache_data_temp[0], cache_data_temp.size(), "cache_data_temp"),
+  SFARRAYBN(&cache_data_valid_temp[0], cache_data_valid_temp.size(), "cache_data_valid_temp"),
 
   SFVAR(ilevel),		// Perhaps remove in future?
   SFVAR(next_event_ts_delta),
@@ -1504,7 +1514,7 @@ int V810::StateAction(StateMem *sm, int load, int data_only)
   SFEND
  };
 
- int ret = MDFNSS_StateAction(sm, load, data_only, StateRegs, "V810");
+ MDFNSS_StateAction(sm, load, data_only, StateRegs, "V810");
 
  if(load)
  {
@@ -1533,13 +1543,4 @@ int V810::StateAction(StateMem *sm, int load, int data_only)
    }
   }
  }
-
- if(EmuMode == V810_EMU_MODE_ACCURATE)
- {
-  free(cache_tag_temp);
-  free(cache_data_temp);
-  free(cache_data_valid_temp);
- }
-
- return(ret);
 }
