@@ -5,6 +5,7 @@
 
 #include <vector>
 #include <stdexcept>
+#include <stdlib.h>
 
 /* Little-endian byte packer(and unpacker). */
 
@@ -22,15 +23,16 @@ class LEPacker : public std::vector<uint8>
 {
  public:
 
- LEPacker() : read_mode(0), read_pos(0)
+ LEPacker() : read_mode(0), read_pos(0), randomize_read_mode(0)
  {
 
  }
 
 
- inline void set_read_mode(bool new_read_mode)
+ inline void set_read_mode(bool new_read_mode, bool new_randomize_read_mode = false)
  {
   read_mode = new_read_mode;
+  randomize_read_mode = new_randomize_read_mode;
  }
 
  inline void reset_read_pos(void)
@@ -43,7 +45,7 @@ class LEPacker : public std::vector<uint8>
   o.pack(*this);
  }
 
- template<typename T> void operator^(T &val)
+ template<typename T> INLINE void operator^(T &val)
  {
   size_type csize = size();
 
@@ -55,8 +57,16 @@ class LEPacker : public std::vector<uint8>
    uint8 *ptr = &(*this)[read_pos];
    val = 0;
 
-   for(unsigned int n = 0; n < sizeof(T); n++)
-    val |= ((T)ptr[n]) << (n << 3);
+   if(randomize_read_mode)
+   {
+    for(unsigned int n = 0; n < sizeof(T); n++)
+     val |= ((T)((rand() >> 4) & 0xFF)) << (n << 3);
+   }
+   else
+   {
+    for(unsigned int n = 0; n < sizeof(T); n++)
+     val |= ((T)ptr[n]) << (n << 3);
+   }
 
    read_pos += sizeof(T);
   }
@@ -71,32 +81,21 @@ class LEPacker : public std::vector<uint8>
   }
  }
 
- void operator^(bool &val)
+ INLINE void operator^(bool &val)
  {
-  size_type csize = size();
+  uint8 tmp = val;
+
+  (*this) ^ tmp;
 
   if(read_mode)
-  {
-   if((read_pos + sizeof(bool)) > csize)
-    throw(std::out_of_range("LEPacker::operator^ for bool"));
-
-   uint8 *ptr = &(*this)[read_pos];
-   val = (bool)*ptr;
-
-   read_pos += sizeof(bool);
-  }
-  else
-  {
-   resize(csize + 1);
-
-   (*this)[csize] = (unsigned int)val;
-  }
+   val = tmp;
  }
 
  private:
 
  bool read_mode;
  uint64 read_pos;
+ bool randomize_read_mode;
 };
 
 }
