@@ -14,11 +14,11 @@ int MDSound_ReadFM(int address);
 void MDSound_SetSoundMultiplier(double multiplier);
 bool MDSound_SetSoundRate(uint32 rate);
 int32 MDSound_Flush(int16 *SoundBuf, const int32 MaxSoundFrames);
-int MDSound_Init(void);
+void MDSound_Init(void);
 void MDSound_Kill(void);
 void MDSound_Power(void);
 
-int MDSound_StateAction(StateMem *sm, int load, int data_only);
+void MDSound_StateAction(StateMem *sm, const unsigned load, const bool data_only);
 
 };
 
