@@ -51,7 +51,7 @@ static void Update(void *data)
  bss|=bss<<8;
 }
 
-static int StateActionFC(StateMem *sm, int load, int data_only)
+static void StateActionFC(StateMem *sm, const unsigned load, const bool data_only)
 {
  SFORMAT StateRegs[] =
  {
@@ -60,12 +60,13 @@ static int StateActionFC(StateMem *sm, int load, int data_only)
    SFVAR(boop),
    SFEND
  };
- int ret = MDFNSS_StateAction(sm, load, data_only, StateRegs, "INPF");
+
+ MDFNSS_StateAction(sm, load, data_only, StateRegs, "INPF", true);
+
  if(load)
  {
 
  }
- return(ret);
 }
 
 static INPUTCFC TopRider={Read,Write,0,Update,0,0, StateActionFC };
