@@ -1,30 +1,30 @@
-typedef struct {
-	char *GameName, *Artist, *Copyright, *Ripper;
+#ifndef __MDFN_NES_NSF_H
+#define __MDFN_NES_NSF_H
 
-        char **SongNames;
-        int32 *SongLengths;
-        int32 *SongFades;
+class MDFNFILE;
 
-        int TotalSongs;
-        int StartingSong;
-        int CurrentSong;
-        int TotalChannels;
-        int VideoSystem;
+namespace MDFN_IEN_NES
+{
 
-        uint16 PlayAddr,InitAddr,LoadAddr;
-        uint8 BankSwitch[8];
-        int SoundChip;
+struct NSFINFO
+{
+	std::string GameName, Artist, Copyright, Ripper;
+	std::vector<std::string> SongNames;
 
-        /* NOTE:  NSFRawData does not necessarily point to the value returned by malloc() or
-           calloc(), so it should not be passed to free()!
-        */
-        uint8 *NSFRawData;
-        uint32 NSFRawDataSize;
+        unsigned TotalSongs = 0;
+        unsigned StartingSong = 0;
+        unsigned CurrentSong = 0;
+        unsigned TotalChannels = 0 ;
+        unsigned VideoSystem = 0;
 
-        uint8 *NSFDATA;
-        int NSFMaxBank;
-        int NSFSize;
-} NSFINFO;
+        uint16 PlayAddr = 0,InitAddr = 0, LoadAddr = 0;
+        uint8 BankSwitch[8] = { 0 };
+        unsigned SoundChip = 0;
+
+        uint8 *NSFDATA = NULL;
+        unsigned NSFMaxBank = 0;
+        unsigned NSFSize = 0;
+};
 
 typedef struct {
                 char ID[5]; /*NESM^Z*/
@@ -37,9 +37,9 @@ typedef struct {
                 uint8 InitAddressHigh;
                 uint8 PlayAddressLow;
                 uint8 PlayAddressHigh;
-                UTF8 GameName[32];
-                UTF8 Artist[32];
-                UTF8 Copyright[32];
+                uint8 GameName[32];
+                uint8 Artist[32];
+                uint8 Copyright[32];
                 uint8 NTSCspeed[2];              // Unused
                 uint8 BankSwitch[8];
                 uint8 PALspeed[2];               // Unused
@@ -47,13 +47,17 @@ typedef struct {
                 uint8 SoundChip;
                 uint8 Expansion[4];
                 uint8 reserve[8];
-        } NSF_HEADER;
+} NSF_HEADER;
+
+void NSFLoad(Stream *fp, NESGameType *gt);
+bool NSF_TestMagic(MDFNFILE *fp);
 
-void NSF_init(void);
-void NSFDealloc(void);
-void NSFDodo(void);
 void DoNSFFrame(void);
 void MDFNNES_DrawNSF(MDFN_Surface *surface, MDFN_Rect *DisplayRect, int16 *samples, int32 scount);
 
 // NSF Expansion Chip Set Write Handler
 void NSFECSetWriteHandler(int32 start, int32 end, writefunc func);
+
+}
+
+#endif
