@@ -24,6 +24,9 @@
 #include	"input.h"
 #include	"vsuni.h"
 
+namespace MDFN_IEN_NES
+{
+
 struct UNIF_HEADER
 {
            char ID[4];
@@ -41,7 +44,7 @@ struct BMAPPING
 struct BFMAPPING
 {
            const char *name;
-           int (*init)(MDFNFILE *fp);
+           void (*init)(Stream *fp);
 	   const uint32 info_ms;
 };
 
@@ -59,7 +62,6 @@ static uint8 *exntar = NULL;
 static UNIF_HEADER unhead;
 static UNIF_HEADER uchead;
 
-
 static uint8 *malloced[32] = { NULL };
 static uint32 mallocedsizes[32] = { 0 };
 
@@ -76,7 +78,7 @@ static uint32 FixRomSize(uint32 size, uint32 minimum)
   return x;
 }
 
-static int UNIF_StateAction(StateMem *sm, int load, int data_only)
+static void UNIF_StateAction(StateMem *sm, const unsigned load, const bool data_only)
 {
  SFORMAT StateRegs[] =
  {
@@ -84,34 +86,38 @@ static int UNIF_StateAction(StateMem *sm, int load, int data_only)
   SFARRAY(UNIFchrrama, CHRRAMSize),
   SFEND
  };
+
  if(NESIsVSUni)
-  if(!MDFNNES_VSUNIStateAction(sm,load, data_only))
-   return(0);
+ {
+  MDFNNES_VSUNIStateAction(sm,load, data_only);
+ }
+
  if(exntar || UNIFchrrama)
-  if(!MDFNSS_StateAction(sm, load, data_only, StateRegs, "UNIF"))
-   return(0);
+ {
+  MDFNSS_StateAction(sm, load, data_only, StateRegs, "UNIF");
+ }
+
  if(UNIFCart.StateAction)
-  return(UNIFCart.StateAction(sm, load, data_only));
- return(1);
+  UNIFCart.StateAction(sm, load, data_only);
 }
 
 static void FreeUNIF(void)
 {
  if(UNIFchrrama)
  {
-  free(UNIFchrrama);
+  MDFN_free(UNIFchrrama);
   UNIFchrrama = NULL;
  }
 
  if(exntar)
  {
-  free(exntar);
+  MDFN_free(exntar);
   exntar = NULL;
  }
 
  if(boardname)
  {
-  free(boardname);
+  MDFN_free(boardname);
   boardname = NULL;
  }
 
@@ -119,7 +125,7 @@ static void FreeUNIF(void)
  {
   if(malloced[x])
   {
-   free(malloced[x]);
+   MDFN_free(malloced[x]);
    malloced[x] = NULL;
   }
  }
@@ -143,17 +149,19 @@ static void InitBoardMirroring(void)
   SetupCartMirroring(mirrortodo,1,0);
  else if(mirrortodo==0x4)
  {
-  exntar = (uint8 *)malloc(2048);
+  exntar = (uint8 *)MDFN_malloc_T(2048, _("Nametable RAM"));
   SetupCartMirroring(4,1,exntar);
  }
  else
   SetupCartMirroring(0,0,0);
 }
 
-static int DoMirroring(MDFNFILE *fp)
+static void DoMirroring(Stream *fp)
 {
  uint8 t;
- t = fp->fgetc();
+
+ fp->read(&t, 1);
+
  mirrortodo=t; 
 
  {
@@ -161,23 +169,16 @@ static int DoMirroring(MDFNFILE *fp)
   if(t<6)
    MDFN_printf(_("Name/Attribute Table Mirroring: %s\n"),stuffo[t]);
  }
- return(1);
 }
 
-static int NAME(MDFNFILE *fp)
+static void NAME(Stream *fp)
 {
  char* namebuf = NULL;
 
  assert(uchead.info <= (SIZE_MAX - 1));
- namebuf = (char*)malloc((size_t)uchead.info + 1);
+ namebuf = (char*)MDFN_malloc_T((size_t)uchead.info + 1, "Name");
 
- if(fp->fread(namebuf, 1, uchead.info) != uchead.info)
- {
-  free(namebuf);
-  namebuf = NULL;
-
-  return(0);
- }
+ fp->read(namebuf, uchead.info);
 
  namebuf[uchead.info] = 0;
  MDFN_RemoveControlChars(namebuf);
@@ -185,57 +186,57 @@ static int NAME(MDFNFILE *fp)
  MDFN_printf(_("Name: %s\n"), namebuf);
 
  if(!MDFNGameInfo->name)
-  MDFNGameInfo->name = (uint8*)namebuf;
+  MDFNGameInfo->name = namebuf;
  else
  {
   free(namebuf);
   namebuf = NULL;
  }
-
- return(1);
 }
-static int DINF(MDFNFILE *fp)
+
+static void DINF(Stream *fp)
 {
- char name[100], method[100];
+ union
+ {
+  struct
+  {
+   char name[100];
+   uint8 raw_date[4];
+   char method[100];
+  };
+  uint8 raw[100 + 4 + 100];
+ } dinf;
  uint8 d, m;
  uint16 y;
- int t;
 
- if(fp->fread(name, 1, 100) != 100)
-  return(0);
- if((t=fp->fgetc())==EOF) return(0);
- d=t;
- if((t=fp->fgetc())==EOF) return(0);
- m=t;
- if((t=fp->fgetc())==EOF) return(0);
- y=t;
- if((t=fp->fgetc())==EOF) return(0);
- y|=t<<8;
- if(fp->fread(method, 1, 100)!=100)
-  return(0);
- name[99]=method[99]=0;
+ fp->read(dinf.raw, sizeof(dinf.raw));
 
- MDFN_RemoveControlChars(name);
- MDFN_RemoveControlChars(method);
+ d = dinf.raw_date[0];
+ m = dinf.raw_date[1];
+ y = MDFN_de16lsb(&dinf.raw_date[2]);
 
- MDFN_printf(_("Dumped by: %s\n"),name);
- MDFN_printf(_("Dumped with: %s\n"),method);
+ dinf.name[99] = dinf.method[99] = 0;
+
+ MDFN_RemoveControlChars(dinf.name);
+ MDFN_RemoveControlChars(dinf.method);
+
+ MDFN_printf(_("Dumped by: %s\n"), dinf.name);
+ MDFN_printf(_("Dumped with: %s\n"), dinf.method);
  {
-  char *months[12]={_("January"),_("February"),_("March"),_("April"),_("May"),_("June"),_("July"),
+  const char* months[12]={_("January"),_("February"),_("March"),_("April"),_("May"),_("June"),_("July"),
 		    _("August"),_("September"),_("October"),_("November"),_("December")};
   MDFN_printf(_("Dumped on: %s %d, %d\n"),months[(m-1)%12],d,y);
  }
- return(1);
 }
 
 static const char *WantInput[3];
 
-static int CTRL(MDFNFILE *fp)
+static void CTRL(Stream *fp)
 {
- int t;
+ uint8 t;
+
+ fp->read(&t, 1);
 
- if((t=fp->fgetc())==EOF)
-  return(0);
  /* The information stored in this byte isn't very helpful, but it's
     better than nothing...maybe.
  */
@@ -253,15 +254,14 @@ static int CTRL(MDFNFILE *fp)
  {
   WantInput[1] = "zapper";
  }
-
- return(1);
 }
 
-static int TVCI(MDFNFILE *fp)
+static void TVCI(Stream *fp)
 {
- int t;
- if( (t=fp->fgetc()) ==EOF)
-  return(0);
+ uint8 t;
+
+ fp->read(&t, 1);
+
  if(t<=2)
  {
   const char *stuffo[3]={"NTSC","PAL","NTSC and PAL"};
@@ -273,57 +273,51 @@ static int TVCI(MDFNFILE *fp)
 
   MDFN_printf(_("TV Standard Compatibility: %s\n"),stuffo[t]);
  }
- return(1);
 }
 
-static int EnableBattery(MDFNFILE *fp)
+static void EnableBattery(Stream *fp)
 {
+ uint8 t;
+
+ fp->read(&t, 1);
+
  MDFN_printf(_("Battery-backed.\n"));
- if(fp->fgetc()==EOF)
-  return(0);
- UNIFCart.battery=1;
- return(1);
+
+ UNIFCart.battery = 1;
 }
 
-static int LoadPRG(MDFNFILE *fp)
+static void LoadPRG(Stream *fp)
 {
  uint32 t;
- int z;
+ unsigned z;
 
- z = uchead.ID[3] - '0';
+ z = uchead.ID[3] - '0';	// FIXME hex
 
- if(z < 0 || z > 15)
-  return(0);
+ if(z > 15)
+  throw MDFN_Error(0, "Invalid PRG ROM index '%c'.\n", uchead.ID[3]);
 
- MDFN_printf(_("PRG ROM %d size: %u"), z, uchead.info);
+ MDFN_printf(_("PRG ROM %u size: %u\n"), z, uchead.info);
 
  if(malloced[z])
   free(malloced[z]);
- t=FixRomSize(uchead.info,2048);
- if(!(malloced[z]=(uint8 *)malloc(t)))
-  return(0);
- mallocedsizes[z]=t;
- memset(malloced[z]+uchead.info,0xFF,t-uchead.info);
- if(fp->fread(malloced[z],1,uchead.info)!=uchead.info)
- {
-  MDFN_PrintError(_("Error reading PRG chunk %d"), z);
-  return(0);
- }
- else
-  MDFN_printf("\n");
+
+ t = FixRomSize(uchead.info, 2048);
+
+ malloced[z] = (uint8 *)MDFN_malloc_T(t, _("PRG ROM"));
+ mallocedsizes[z] = t;
+ memset(malloced[z] + uchead.info, 0xFF, t - uchead.info);
+
+ fp->read(malloced[z], uchead.info);
 
  SetupCartPRGMapping(z,malloced[z],t,0); 
- return(1);
 }
 
-static int SetBoardName(MDFNFILE *fp)
+static void SetBoardName(Stream *fp)
 {
  assert(uchead.info <= (SIZE_MAX - 1));
+ boardname = (uint8*)MDFN_malloc_T((size_t)uchead.info + 1, _("Board Name"));
 
- if(!(boardname=(uint8 *)malloc((size_t)uchead.info + 1)))
-  return(0);
-
- fp->fread(boardname, 1, uchead.info);
+ fp->read(boardname, uchead.info);
  boardname[uchead.info] = 0;
  MDFN_RemoveControlChars((char*)boardname);
 
@@ -331,39 +325,31 @@ static int SetBoardName(MDFNFILE *fp)
  sboardname=boardname;
  if(!memcmp(boardname,"NES-",4) || !memcmp(boardname,"UNL-",4) || !memcmp(boardname,"HVC-",4) || !memcmp(boardname,"BTL-",4) || !memcmp(boardname,"BMC-",4))
   sboardname+=4;
- return(1);
 }
 
-static int LoadCHR(MDFNFILE *fp)
+static void LoadCHR(Stream *fp)
 {
  uint32 t;
- int z;
+ unsigned z;
 
  z = uchead.ID[3] - '0';
 
- if(z < 0 || z > 15)
-  return(0);
+ if(z > 15)
+  throw MDFN_Error(0, "Invalid CHR ROM index '%c'.\n", uchead.ID[3]);
 
- MDFN_printf(_("CHR ROM %d size: %u"), z, uchead.info);
+ MDFN_printf(_("CHR ROM %u size: %u\n"), z, uchead.info);
 
- if(malloced[16+z])
-  free(malloced[16+z]);
+ if(malloced[16 + z])
+  free(malloced[16 + z]);
 
- t=FixRomSize(uchead.info, 8192);
- if(!(malloced[16+z]=(uint8 *)malloc(t)))
-  return(0);
- mallocedsizes[16+z]=t;
- memset(malloced[16+z]+uchead.info,0xFF,t-uchead.info);
- if(fp->fread(malloced[16+z],1,uchead.info)!=uchead.info)
- {
-  MDFN_PrintError(_("Error reading CHR chunk %d"), z);
-  return(0);
- }
- else
-  MDFN_printf("\n");
+ t = FixRomSize(uchead.info, 8192);
+ malloced[16 + z] = (uint8 *)MDFN_malloc_T(t, _("CHR ROM"));
+ mallocedsizes[16 + z]=t;
+ memset(malloced[16 + z] + uchead.info, 0xFF, t - uchead.info);
 
- SetupCartCHRMapping(z,malloced[16+z],t,0);
- return(1);
+ fp->read(malloced[16 + z], uchead.info);
+
+ SetupCartCHRMapping(z, malloced[16 + z], t, 0);
 }
 
 
@@ -449,11 +435,10 @@ static const BMAPPING bmap[] =
  { "BioMiracleA", 0x0001U, BioMiracleA_Init, 0},
 
  { "603-5052", 0x0001U, UNL6035052_Init, 0},
-
- {0,0,0,0}
 };
 
-static const BFMAPPING bfunc[] = {
+static const BFMAPPING bfunc[] =
+{
  { "CTRL", CTRL,		~0U },
  { "TVCI", TVCI,		~0U },
  { "BATR", EnableBattery,	~0U },
@@ -464,94 +449,76 @@ static const BFMAPPING bfunc[] = {
  { "CHR",  LoadCHR,		512 * 1024 * 1024 },
  { "MAPR", SetBoardName,	512 * 1024 * 1024 },
  { "NAME", NAME,		512 * 1024 * 1024 },
-
- { 0, 0, 0 }
 };
 
-int LoadUNIFChunks(MDFNFILE *fp)
+static void LoadUNIFChunks(Stream *fp)
 {
-   int x;
-   int t;
-   for(;;)
-   {
-    t = fp->fread(&uchead,1,4);
-    if(t<4) 
-    {
-     if(t>0)
-      return 0; 
-     return 1;
-    }
-    if(!(fp->read32le(&uchead.info))) 
-     return 0;
-    t=0;
-    x=0;
-	//printf("Funky: %s\n",((uint8 *)&uchead));
-    while(bfunc[x].name)
-    {
-     if(!memcmp(&uchead,bfunc[x].name,strlen(bfunc[x].name)))
-     {
-      if(uchead.info > bfunc[x].info_ms)
-      {
-       MDFN_PrintError("Value(%u) in length field of chunk \"%.4s\" is too large.", uchead.info, &uchead.ID[0]);
-       return(0);
-      }
+   uint64 uhrc;
 
-      if(!bfunc[x].init(fp))
-       return 0;
-      t=1;
+   while((uhrc = fp->read(&uchead, 4, false)) == 4)
+   {
+    bool found_handler = false;
+
+    uchead.info = fp->get_LE<uint32>();
+
+    for(auto const& bf : bfunc)
+    {
+     if(!memcmp(&uchead, bf.name, strlen(bf.name)))
+     {
+      if(uchead.info > bf.info_ms)
+       throw MDFN_Error(0, _("Value(%u) in length field of chunk \"%.4s\" is too large."), uchead.info, &uchead.ID[0]);
+
+      bf.init(fp);
+
+      found_handler = true;
       break;     
      }
-     x++;
     }
-    if(!t)
-     if(fp->fseek(uchead.info, SEEK_CUR))
-      return(0);
+
+    //printf("%.4s\n", &uchead);
+
+    if(!found_handler)
+     fp->seek(uchead.info, SEEK_CUR);
    }
+
+   if(uhrc != 0)
+    throw MDFN_Error(0, _("Unexpected EOF"));
 }
 
-static int InitializeBoard(void)
+static void InitializeBoard(void)
 {
-   int x=0;
+   if(!sboardname)
+    throw MDFN_Error(0, _("Missing UNIF board name."));
 
-   if(!sboardname) return(0);
-
-   while(bmap[x].name)
+   for(auto const& bm : bmap)
    {
-    if(!strcmp((char *)sboardname,(char *)bmap[x].name))
+    if(!strcmp((char *)sboardname, (char *)bm.name))
     {
      for(unsigned i = 0; i < 16; i++)
      {
-      if((bmap[x].prg_rm & (1U << i)) && !malloced[i])
-      {
-       MDFN_PrintError(_("Missing PRG ROM %u."), i);
-       return(0);
-      }
+      if((bm.prg_rm & (1U << i)) && !malloced[i])
+       throw MDFN_Error(0, _("Missing PRG ROM %u."), i);
      }
 
      if(!malloced[16])
      {
-      if(bmap[x].flags & BMCFLAG_32KCHRR)
+      if(bm.flags & BMCFLAG_32KCHRR)
 	CHRRAMSize = 32768;
       else
 	CHRRAMSize = 8192;
-      if((UNIFchrrama=(uint8 *)malloc(CHRRAMSize)))
-      {
-       SetupCartCHRMapping(0,UNIFchrrama,CHRRAMSize,1);
-      }
-      else
-       return(-1);
+
+      UNIFchrrama = (uint8 *)MDFN_malloc_T(CHRRAMSize, _("CHR RAM"));
+      SetupCartCHRMapping(0,UNIFchrrama,CHRRAMSize,1);
      }
 
-     if(bmap[x].flags&BMCFLAG_FORCE4)
+     if(bm.flags & BMCFLAG_FORCE4)
       mirrortodo=4;
      InitBoardMirroring();
-     bmap[x].init(&UNIFCart);
-     return(1);
+     bm.init(&UNIFCart);
+     return;
     }
-    x++;
    }
-   MDFN_PrintError(_("Board type not supported."));
-   return(0);
+   throw MDFN_Error(0, _("Board type not supported."));
 }
 
 static void UNIF_Reset(void)
@@ -567,9 +534,13 @@ static void UNIF_Power(void)
  if(UNIFchrrama) memset(UNIFchrrama, 0xFF, CHRRAMSize);
 }
 
-static void UNIF_Close(void)
+static void UNIF_SaveNV(void)
 {
  MDFN_SaveGameSave(&UNIFCart);
+}
+
+static void UNIF_Kill(void)
+{
  if(UNIFCart.Close)
   UNIFCart.Close();
  FreeUNIF();
@@ -577,60 +548,60 @@ static void UNIF_Close(void)
 
 bool UNIF_TestMagic(MDFNFILE *fp)
 {
- if(fp->size < 4)
-  return(FALSE);
+ uint8 magic[4];
 
- if(memcmp(fp->data, "UNIF", 4))
-  return(FALSE);
+ if(fp->read(magic, 4, false) != 4 || memcmp(magic, "UNIF", 4))
+  return false;
 
- return(TRUE);
+ return true;
 }
 
 
-bool UNIFLoad(MDFNFILE *fp, NESGameType *gt)
+void UNIFLoad(Stream *fp, NESGameType *gt)
 {
-	if(!UNIF_TestMagic(fp))
-	 return(FALSE);
+  try
+  {
+	uint8 magic[4];
 
-	fp->fseek(4, SEEK_SET);
+	if(fp->read(magic, 4, false) != 4 || memcmp(magic, "UNIF", 4))
+	 throw MDFN_Error(0, _("Not a valid UNIF file."));
 
 	ResetCartMapping();
 
         ResetUNIF();
 	memset(WantInput, 0, sizeof(WantInput));
 
-        if(!fp->read32le(&unhead.info))
-	 goto aborto;
-        if(fp->fseek(0x20,SEEK_SET)<0)
-	 goto aborto;
-        if(!LoadUNIFChunks(fp))
-	 goto aborto;
+        unhead.info = fp->get_LE<uint32>();
+        fp->seek(0x20, SEEK_SET);
+
+        LoadUNIFChunks(fp);
+
 	{
-	 int x;
 	 md5_context md5;
 	 
 	 md5.starts();
 
-	 for(x=0;x<32;x++)
+	 for(int x = 0; x < 32; x++)
+	 {
 	  if(malloced[x])
 	  {
 	   md5.update(malloced[x],mallocedsizes[x]);
 	  }
-	  md5.finish(UNIFCart.MD5);
-          MDFN_printf(_("ROM MD5:  0x%s\n"), md5_context::asciistr(UNIFCart.MD5, 0).c_str());
-	  memcpy(MDFNGameInfo->MD5,UNIFCart.MD5,sizeof(UNIFCart.MD5));
-	  MDFN_printf("\n");
+	 }
+	 md5.finish(UNIFCart.MD5);
+         MDFN_printf(_("ROM MD5:  0x%s\n"), md5_context::asciistr(UNIFCart.MD5, 0).c_str());
+	 memcpy(MDFNGameInfo->MD5,UNIFCart.MD5,sizeof(UNIFCart.MD5));
+	 MDFN_printf("\n");
 	}
 
-        if(!InitializeBoard())
-	 goto aborto;
+        InitializeBoard();
 
-	if(!MDFN_LoadGameSave(&UNIFCart))
-	 goto aborto;
+	MDFN_LoadGameSave(&UNIFCart);
 
 	gt->Power = UNIF_Power;
 	gt->Reset = UNIF_Reset;
-	gt->Close = UNIF_Close;
+	gt->SaveNV = UNIF_SaveNV;
+	gt->Kill = UNIF_Kill;
 	gt->StateAction = UNIF_StateAction;
 
         if(UNIFCart.CartExpSound.HiFill)
@@ -642,13 +613,16 @@ bool UNIFLoad(MDFNFILE *fp, NESGameType *gt)
         MDFNGameInfo->DesiredInput.push_back("gamepad");
 
         MDFNGameInfo->DesiredInput.push_back(WantInput[2]);
+  }
+  catch(...)
+  {
+   if(UNIFCart.Close)
+    UNIFCart.Close();
+
+   FreeUNIF();
+   ResetUNIF();
+   throw;
+  }
+}
 
-
-        return 1;
-	
-	aborto:
-
-	FreeUNIF();
-	ResetUNIF();
-	return 0;
 }
