@@ -65,10 +65,10 @@
 //---------------------------------------------------------------------------
 */
 
-#include "neopop.h"
+#include "../neopop.h"
 #include "TLCS900h_interpret.h"
 #include "TLCS900h_registers.h"
-#include "mem.h"
+#include "../mem.h"
 
 namespace TLCS900H
 {
