@@ -50,7 +50,7 @@ void NGPGFX_CLASS::drawMonoPattern(uint8 screenx, uint16 tile, uint8 tiley, uint
 				 uint8* palette_ptr, uint16 pal, uint8 depth)
 {
 	//Get the data for th e "tiley'th" line of "tile".
-	uint16 data = LoadU16_LE((uint16*)(CharacterRAM + (tile * 16) + (tiley * 2)));
+	uint16 data = MDFN_de16lsb<true>(CharacterRAM + (tile * 16) + (tiley * 2));
 
 	//Horizontal Flip
 	if (mirror)
@@ -89,7 +89,7 @@ void NGPGFX_CLASS::draw_mono_scroll1(uint8 depth, int ngpc_scanline)
 	//Draw Foreground scroll plane (Scroll 1)
 	for (tx = 0; tx < 32; tx++)
 	{
-		data16 = LoadU16_LE((uint16*)(ScrollVRAM + ((tx + ((line >> 3) << 5)) << 1)));
+		data16 = MDFN_de16lsb<true>(ScrollVRAM + ((tx + ((line >> 3) << 5)) << 1));
 		
 		//Draw the line of the tile
 		drawMonoPattern((tx << 3) - scroll1x, data16 & 0x01FF, 
@@ -109,7 +109,7 @@ void NGPGFX_CLASS::draw_mono_scroll2(uint8 depth, int ngpc_scanline)
 	//Draw Background scroll plane (Scroll 2)
 	for (tx = 0; tx < 32; tx++)
 	{
-		data16 = LoadU16_LE((uint16*)(ScrollVRAM + 0x0800 + ((tx + ((line >> 3) << 5)) << 1)));
+		data16 = MDFN_de16lsb<true>(ScrollVRAM + 0x0800 + ((tx + ((line >> 3) << 5)) << 1));
 		
 		//Draw the line of the tile
 		drawMonoPattern((tx << 3) - scroll2x, data16 & 0x01FF, 
@@ -209,7 +209,7 @@ void NGPGFX_CLASS::draw_scanline_mono(int layer_enable, int ngpc_scanline)
 			int16 x = sx;
 			int16 y = sy;
 			
-			data16 = LoadU16_LE((uint16*)(SpriteVRAM + (spr * 4)));
+			data16 = MDFN_de16lsb<true>(SpriteVRAM + (spr * 4));
 			priority = (data16 & 0x1800) >> 11;
 
 			if (data16 & 0x0400) x = lastSpriteX + sx;	//Horizontal chain?
