@@ -13,14 +13,14 @@ typedef enum
  SYSCARD_ARCADE		// 3.0 + extras
 } SysCardType;
 
-void HuC_Load(const uint8 *data, uint32 len, uint32 crc32, bool DisableBRAM = false, SysCardType syscard = SYSCARD_NONE);
-void HuC_Kill(void);
+uint32 HuC_Load(MDFNFILE* fp, bool DisableBRAM = false, SysCardType syscard = SYSCARD_NONE);
 void HuC_SaveNV(void);
+void HuC_Kill(void);
 
 void HuC_Update(int32 timestamp);
 void HuC_ResetTS(int32 ts_base);
 
-int HuC_StateAction(StateMem *sm, int load, int data_only);
+void HuC_StateAction(StateMem *sm, const unsigned load, const bool data_only);
 
 void HuC_Power(void);
 
