@@ -40,8 +40,8 @@ void PCEFast_PSG::UpdateOutput_Norm(const int32 timestamp, psg_channel *ch)
  samp[0] = dbtable[ch->vl[0]][sv];
  samp[1] = dbtable[ch->vl[1]][sv];
 
- Synth.offset_inline(timestamp, samp[0] - ch->blip_prev_samp[0], sbuf[0]);
- Synth.offset_inline(timestamp, samp[1] - ch->blip_prev_samp[1], sbuf[1]);
+ Synth.offset_inline(timestamp, samp[0] - ch->blip_prev_samp[0], &sbuf[0]);
+ Synth.offset_inline(timestamp, samp[1] - ch->blip_prev_samp[1], &sbuf[1]);
 
  ch->blip_prev_samp[0] = samp[0];
  ch->blip_prev_samp[1] = samp[1];
@@ -55,8 +55,8 @@ void PCEFast_PSG::UpdateOutput_Noise(const int32 timestamp, psg_channel *ch)
  samp[0] = dbtable[ch->vl[0]][sv];
  samp[1] = dbtable[ch->vl[1]][sv];
 
- Synth.offset_inline(timestamp, samp[0] - ch->blip_prev_samp[0], sbuf[0]);
- Synth.offset_inline(timestamp, samp[1] - ch->blip_prev_samp[1], sbuf[1]);
+ Synth.offset_inline(timestamp, samp[0] - ch->blip_prev_samp[0], &sbuf[0]);
+ Synth.offset_inline(timestamp, samp[1] - ch->blip_prev_samp[1], &sbuf[1]);
 
  ch->blip_prev_samp[0] = samp[0];
  ch->blip_prev_samp[1] = samp[1];
@@ -68,8 +68,8 @@ void PCEFast_PSG::UpdateOutput_Off(const int32 timestamp, psg_channel *ch)
 
  samp[0] = samp[1] = 0;
 
- Synth.offset_inline(timestamp, samp[0] - ch->blip_prev_samp[0], sbuf[0]);
- Synth.offset_inline(timestamp, samp[1] - ch->blip_prev_samp[1], sbuf[1]);
+ Synth.offset_inline(timestamp, samp[0] - ch->blip_prev_samp[0], &sbuf[0]);
+ Synth.offset_inline(timestamp, samp[1] - ch->blip_prev_samp[1], &sbuf[1]);
 
  ch->blip_prev_samp[0] = samp[0];
  ch->blip_prev_samp[1] = samp[1];
@@ -83,8 +83,8 @@ void PCEFast_PSG::UpdateOutput_Accum(const int32 timestamp, psg_channel *ch)
  samp[0] = ((int32)dbtable_volonly[ch->vl[0]] * ((int32)ch->samp_accum - 496)) >> (8 + 5);
  samp[1] = ((int32)dbtable_volonly[ch->vl[1]] * ((int32)ch->samp_accum - 496)) >> (8 + 5);
 
- Synth.offset_inline(timestamp, samp[0] - ch->blip_prev_samp[0], sbuf[0]);
- Synth.offset_inline(timestamp, samp[1] - ch->blip_prev_samp[1], sbuf[1]);
+ Synth.offset_inline(timestamp, samp[0] - ch->blip_prev_samp[0], &sbuf[0]);
+ Synth.offset_inline(timestamp, samp[1] - ch->blip_prev_samp[1], &sbuf[1]);
 
  ch->blip_prev_samp[0] = samp[0];
  ch->blip_prev_samp[1] = samp[1];
@@ -146,14 +146,10 @@ void PCEFast_PSG::RecalcNoiseFreqCache(int chnum)
  ch->noise_freq_cache = freq;
 }
 
-PCEFast_PSG::PCEFast_PSG(Blip_Buffer *bb_l, Blip_Buffer *bb_r)
+PCEFast_PSG::PCEFast_PSG(Blip_Buffer* bbs) : sbuf(bbs)
 {
-	//printf("Test: %u, %u\n", sizeof(psg_channel), (uint8*)&channel[0].balance - (uint8*)&channel[0].waveform[0]);
 	Synth.treble_eq(-2.0);
 
-	sbuf[0] = bb_l;
-	sbuf[1] = bb_r;
-
 	lastts = 0;
 	for(int ch = 0; ch < 6; ch++)
 	{
