@@ -1,7 +1,7 @@
 #ifndef __PCFX_INPUT_MOUSE_H
 #define __PCFX_INPUT_MOUSE_H
 
-extern const InputDeviceInputInfoStruct PCFX_MouseIDII[4];
+extern const IDIISG PCFX_MouseIDII;
 
 PCFX_Input_Device *PCFXINPUT_MakeMouse(int which);
 
