@@ -28,21 +28,21 @@
 #include <mednafen/cdrom/scsicd.h>
 #include <mednafen/mempatcher.h>
 #include <mednafen/cdrom/cdromif.h>
-#include <mednafen/md5.h>
+#include <mednafen/hash/md5.h>
 #include <mednafen/FileStream.h>
+#include <mednafen/compress/GZFileStream.h>
 
 #include <trio/trio.h>
 #include <errno.h>
 #include <string.h>
 #include <math.h>
 
+extern MDFNGI EmulatedPCFX;
+
 /* FIXME:  soundbox, vce, vdc, rainbow, and king store wait states should be 4, not 2, but V810 has write buffers which can mask wait state penalties.
   This is a hack to somewhat address the issue, but to really fix it, we need to handle write buffer emulation in the V810 emulation core itself.
 */
-
 static std::vector<CDIF*> *cdifs = NULL;
-static bool CD_TrayOpen;
-static int CD_SelectedDisc;	// -1 for no disc
 
 V810 PCFX_V810;
 
@@ -270,7 +270,8 @@ static void Emulate(EmulateSpecStruct *espec)
  // new_base_ts is guaranteed to be <= v810_timestamp
  //
  v810_timestamp_t new_base_ts;
- espec->SoundBufSize = SoundBox_Flush(v810_timestamp, &new_base_ts, espec->SoundBuf, espec->SoundBufMaxSize);
+ espec->SoundBufSize = SoundBox_Flush(v810_timestamp, &new_base_ts, espec->SoundBuf, espec->SoundBufMaxSize, espec->NeedSoundReverse);
+ espec->NeedSoundReverse = false;
 
  KING_ResetTS(new_base_ts);
  FXTIMER_ResetTS(new_base_ts);
@@ -541,8 +542,6 @@ static void VDCB_IRQHook(bool asserted)
  PCFXIRQ_Assert(PCFXIRQ_SOURCE_VDCB, asserted);
 }
 
-static void SetRegGroups(void);
-
 static void Cleanup(void)
 {
  for(int i = 0; i < 2; i++)
@@ -569,7 +568,7 @@ static void LoadCommon(std::vector<CDIF *> *CDInterfaces)
  V810_Emu_Mode cpu_mode;
 
  #ifdef WANT_DEBUGGER
- SetRegGroups();
+ PCFXDBG_Init();
  #endif
 
  cpu_mode = (V810_Emu_Mode)MDFN_GetSettingI("pcfx.cpu_emulation");
@@ -593,14 +592,13 @@ static void LoadCommon(std::vector<CDIF *> *CDInterfaces)
  BIOSROM = PCFX_V810.SetFastMap(BIOSROM_Map_Addresses, 0x00100000, 1, _("BIOS ROM"));
 
  {
-  std::string biospath = MDFN_MakeFName(MDFNMKF_FIRMWARE, 0, MDFN_GetSettingS("pcfx.bios").c_str());
+  std::string biospath = MDFN_MakeFName(MDFNMKF_FIRMWARE, 0, MDFN_GetSettingS("pcfx.bios"));
   MDFNFILE BIOSFile(biospath.c_str(), NULL, "BIOS");
 
-  if(BIOSFile.Size() != 1024 * 1024)
+  if(BIOSFile.size() != 1024 * 1024)
    throw MDFN_Error(0, _("BIOS ROM file is incorrect size.\n"));
 
-  memcpy(BIOSROM, BIOSFile.Data(), 1024 * 1024);
-
+  BIOSFile.read(BIOSROM, 1024 * 1024);
   BIOSFile.Close();
  }
 
@@ -609,7 +607,7 @@ static void LoadCommon(std::vector<CDIF *> *CDInterfaces)
 
   if(fxscsi_path != "0" && fxscsi_path != "" && fxscsi_path != "none")
   {
-   FileStream FXSCSIFile(fxscsi_path.c_str(), FileStream::MODE_READ);
+   FileStream FXSCSIFile(fxscsi_path, FileStream::MODE_READ);
    uint32 FXSCSI_Map_Addresses[1] = { 0x80780000 };
 
    FXSCSIROM = PCFX_V810.SetFastMap(FXSCSI_Map_Addresses, 0x0080000, 1, _("FX-SCSI ROM"));
@@ -646,12 +644,7 @@ static void LoadCommon(std::vector<CDIF *> *CDInterfaces)
 
  KING_Init();
 
- CD_TrayOpen = false;
- CD_SelectedDisc = 0;
-
  SCSICD_SetDisc(true, NULL, true);
- SCSICD_SetDisc(false, (*CDInterfaces)[0], true);
-
 
  #ifdef WANT_DEBUGGER
  for(unsigned disc = 0; disc < CDInterfaces->size(); disc++)
@@ -670,7 +663,7 @@ static void LoadCommon(std::vector<CDIF *> *CDInterfaces)
     trio_snprintf(tmpn, 256, "track%d-%d-%d", disc, track, toc.tracks[track].lba);
     trio_snprintf(tmpln, 256, "CD - Disc %d/%d - Track %d/%d", disc + 1, (int)CDInterfaces->size(), track, toc.last_track - toc.first_track + 1);
 
-    sectors = toc.tracks[track + 1].lba - toc.tracks[track].lba;
+    sectors = toc.tracks[(track == toc.last_track) ? 100 : track + 1].lba - toc.tracks[track].lba;
     ASpace_Add(PCFXDBG_GetAddressSpaceBytes, PCFXDBG_PutAddressSpaceBytes, tmpn, tmpln, 0, sectors * 2048);
    }
   }
@@ -725,12 +718,22 @@ static void LoadCommon(std::vector<CDIF *> *CDInterfaces)
   memcpy(ExBackupRAM + 0x00, ExBRInit00, sizeof(ExBRInit00));
   memcpy(ExBackupRAM + 0x80, ExBRInit80, sizeof(ExBRInit80));
 
-  gzFile savefp;
-  if((savefp = gzopen(MDFN_MakeFName(MDFNMKF_SAV, 0, "sav").c_str(), "rb")))
+  try
   {
-   gzread(savefp, BackupRAM, 0x8000);
-   gzread(savefp, ExBackupRAM, 0x8000);
-   gzclose(savefp);
+   std::string save_path = MDFN_MakeFName(MDFNMKF_SAV, 0, "sav");
+   GZFileStream savefp(save_path, GZFileStream::MODE::READ);
+   const uint64 fp_size_tmp = savefp.size();
+
+   if(fp_size_tmp != 65536)
+    throw MDFN_Error(0, _("Save game memory file \"%s\" is an incorrect size(%llu bytes).  The correct size is %llu bytes."), save_path.c_str(), (unsigned long long)fp_size_tmp, (unsigned long long)65536);
+
+   savefp.read(BackupRAM, 0x8000);
+   savefp.read(ExBackupRAM, 0x8000);
+  }
+  catch(MDFN_Error &e)
+  {
+   if(e.GetErrno() != ENOENT)
+    throw;
   }
  }
 
@@ -884,7 +887,7 @@ static void DoMD5CDVoodoo(std::vector<CDIF *> *CDInterfaces)
     MDFNGameInfo->GameSetMD5Valid = TRUE;
    }
    //printf("%s\n", found_entry->name);
-   MDFNGameInfo->name = (UTF8*)strdup(found_entry->name);
+   MDFNGameInfo->name = strdup(found_entry->name);
    break;
   }
  } // end: for(unsigned if_disc = 0; if_disc < CDInterfaces->size(); if_disc++)
@@ -938,8 +941,6 @@ static void LoadCD(std::vector<CDIF *> *CDInterfaces)
 
   MDFN_printf(_("Emulated CD-ROM drive speed: %ux\n"), (unsigned int)MDFN_GetSettingUI("pcfx.cdspeed"));
 
-  MDFNGameInfo->GameType = GMT_CDROM;
-
   PCFX_Power();
  }
  catch(...)
@@ -949,58 +950,34 @@ static void LoadCD(std::vector<CDIF *> *CDInterfaces)
  }
 }
 
-static void PCFX_CDInsertEject(void)
+static bool SetMedia(uint32 drive_idx, uint32 state_idx, uint32 media_idx, uint32 orientation_idx)
 {
- CD_TrayOpen = !CD_TrayOpen;
+ const RMD_Layout* rmd = EmulatedPCFX.RMD;
+ const RMD_Drive* rd = &rmd->Drives[drive_idx];
+ const RMD_State* rs = &rd->PossibleStates[state_idx];
 
- for(unsigned disc = 0; disc < cdifs->size(); disc++)
+ if(rs->MediaPresent && rs->MediaUsable)
  {
-  if(!(*cdifs)[disc]->Eject(CD_TrayOpen))
-  {
-   MDFN_DispMessage(_("Eject error."));
-   CD_TrayOpen = !CD_TrayOpen;
-  }
+  SCSICD_SetDisc(false, (*cdifs)[media_idx]);
  }
-
- if(CD_TrayOpen)
-  MDFN_DispMessage(_("Virtual CD Drive Tray Open"));
  else
-  MDFN_DispMessage(_("Virtual CD Drive Tray Closed"));
-
- SCSICD_SetDisc(CD_TrayOpen, (CD_SelectedDisc >= 0 && !CD_TrayOpen) ? (*cdifs)[CD_SelectedDisc] : NULL);
-}
-
-static void PCFX_CDEject(void)
-{
- if(!CD_TrayOpen)
-  PCFX_CDInsertEject();
-}
-
-static void PCFX_CDSelect(void)
-{
- if(cdifs && CD_TrayOpen)
  {
-  CD_SelectedDisc = (CD_SelectedDisc + 1) % (cdifs->size() + 1);
-
-  if((unsigned)CD_SelectedDisc == cdifs->size())
-   CD_SelectedDisc = -1;
-
-  if(CD_SelectedDisc == -1)
-   MDFN_DispMessage(_("Disc absence selected."));
-  else
-   MDFN_DispMessage(_("Disc %d of %d selected."), CD_SelectedDisc + 1, (int)cdifs->size());
+  SCSICD_SetDisc(rs->MediaCanChange, NULL);
  }
+
+ return(true);
 }
 
 static void SaveBackupMemory(void)
 {
  if(!BRAMDisabled)
  {
-  FileStream fp(MDFN_MakeFName(MDFNMKF_SAV, 0, "sav").c_str(), FileStream::MODE_WRITE);
+  FileStream fp(MDFN_MakeFName(MDFNMKF_SAV, 0, "sav"), FileStream::MODE_WRITE_INPLACE);
 
   fp.write(BackupRAM, 0x8000);
   fp.write(ExBackupRAM, 0x8000);
 
+  fp.truncate(fp.tell());
   fp.close();
  }
 }
@@ -1023,24 +1000,12 @@ static void DoSimpleCommand(int cmd)
 {
  switch(cmd)
  {
-   case MDFN_MSC_INSERT_DISK:
-		PCFX_CDInsertEject();
-                break;
-
-   case MDFN_MSC_SELECT_DISK:
-		PCFX_CDSelect();
-                break;
-
-   case MDFN_MSC_EJECT_DISK:
-		PCFX_CDEject();
-                break;
-
   case MDFN_MSC_RESET: PCFX_Reset(); break;
   case MDFN_MSC_POWER: PCFX_Power(); break;
  }
 }
 
-static int StateAction(StateMem *sm, int load, int data_only)
+static void StateAction(StateMem *sm, const unsigned load, const bool data_only)
 {
  const v810_timestamp_t timestamp = PCFX_V810.v810_timestamp;
 
@@ -1053,25 +1018,22 @@ static int StateAction(StateMem *sm, int load, int data_only)
   SFARRAY(BackupRAM, BRAMDisabled ? 0 : 0x8000),
   SFARRAY(ExBackupRAM, BRAMDisabled ? 0 : 0x8000),
 
-  SFVAR(CD_TrayOpen),
-  SFVAR(CD_SelectedDisc),
-
   SFEND
  };
 
- int ret = MDFNSS_StateAction(sm, load, data_only, StateRegs, "MAIN");
+ MDFNSS_StateAction(sm, load, data_only, StateRegs, "MAIN");
 
  for(int i = 0; i < 2; i++)
-  ret &= fx_vdc_chips[i]->StateAction(sm, load, data_only, i ? "VDC1" : "VDC0");
+  fx_vdc_chips[i]->StateAction(sm, load, data_only, i ? "VDC1" : "VDC0");
 
- ret &= FXINPUT_StateAction(sm, load, data_only);
- ret &= PCFXIRQ_StateAction(sm, load, data_only);
- ret &= KING_StateAction(sm, load, data_only);
- ret &= PCFX_V810.StateAction(sm, load, data_only);
- ret &= FXTIMER_StateAction(sm, load, data_only);
- ret &= SoundBox_StateAction(sm, load, data_only);
- ret &= SCSICD_StateAction(sm, load, data_only, "CDRM");
- ret &= RAINBOW_StateAction(sm, load, data_only);
+ FXINPUT_StateAction(sm, load, data_only);
+ PCFXIRQ_StateAction(sm, load, data_only);
+ KING_StateAction(sm, load, data_only);
+ PCFX_V810.StateAction(sm, load, data_only);
+ FXTIMER_StateAction(sm, load, data_only);
+ SoundBox_StateAction(sm, load, data_only);
+ SCSICD_StateAction(sm, load, data_only, "CDRM");
+ RAINBOW_StateAction(sm, load, data_only);
 
  if(load)
  {
@@ -1081,22 +1043,11 @@ static int StateAction(StateMem *sm, int load, int data_only)
   //
   ForceEventUpdates(timestamp);
 
-  if(cdifs)
-  {
-   // Sanity check.
-   if(CD_SelectedDisc >= (int)cdifs->size())
-    CD_SelectedDisc = (int)cdifs->size() - 1;
-
-   SCSICD_SetDisc(CD_TrayOpen, (CD_SelectedDisc >= 0 && !CD_TrayOpen) ? (*cdifs)[CD_SelectedDisc] : NULL, true);
-  }
-
   if(!BRAMDisabled)
    BackupSignalDirty = true;
  }
 
  //printf("0x%08x, %d %d %d %d\n", load, next_pad_ts, next_timer_ts, next_adpcm_ts, next_king_ts);
-
- return(ret);
 }
 
 static const MDFNSetting_EnumList V810Mode_List[] =
@@ -1151,284 +1102,6 @@ static MDFNSetting PCFXSettings[] =
   { NULL }
 };
 
-#ifdef WANT_DEBUGGER
-static RegType PCFXRegs0[] =
-{
-        { 0, "PC", "Program Counter", 4 },
-	{ 0, "PR1", "Program Register 1", 4 },
-	{ 0, "HSP", "Program Register 2(Handler Stack Pointer)", 4 },
-	{ 0, "SP", "Program Register 3(Stack Pointer)", 4 },
-	{ 0, "GP", "Program Register 4(Global Pointer)", 4 },
-	{ 0, "TP", "Program Register 5(Text Pointer)", 4 },
-	{ 0, "PR6", "Program Register 6", 4 },
-	{ 0, "PR7", "Program Register 7", 4 },
-	{ 0, "PR8", "Program Register 8", 4 },
-	{ 0, "PR9", "Program Register 9", 4 },
-	{ 0, "PR10", "Program Register 10", 4 },
-	{ 0, "PR11", "Program Register 11", 4 },
-	{ 0, "PR12", "Program Register 12", 4 },
-	{ 0, "PR13", "Program Register 13", 4 },
-	{ 0, "PR14", "Program Register 14", 4 },
-	{ 0, "PR15", "Program Register 15", 4 },
-        { 0, "PR16", "Program Register 16", 4 },
-        { 0, "PR17", "Program Register 17", 4 },
-        { 0, "PR18", "Program Register 18", 4 },
-        { 0, "PR19", "Program Register 19", 4 },
-        { 0, "PR20", "Program Register 20", 4 },
-        { 0, "PR21", "Program Register 21", 4 },
-        { 0, "PR22", "Program Register 22", 4 },
-        { 0, "PR23", "Program Register 23", 4 },
-        { 0, "PR24", "Program Register 24", 4 },
-        { 0, "PR25", "Program Register 25", 4 },
-        { 0, "PR26", "Program Register 26(String Dest Bit Offset)", 4 },
-        { 0, "PR27", "Program Register 27(String Source Bit Offset)", 4 },
-        { 0, "PR28", "Program Register 28(String Length)", 4 },
-        { 0, "PR29", "Program Register 29(String Dest)", 4 },
-        { 0, "PR30", "Program Register 30(String Source)", 4 },
-        { 0, "LP", "Program Register 31(Link Pointer)", 4 },
-        { 0, "SR0", "Exception/Interrupt PC", 4 },
-        { 0, "SR1", "Exception/Interrupt PSW", 4 },
-        { 0, "SR2", "Fatal Error PC", 4 },
-        { 0, "SR3", "Fatal Error PSW", 4 },
-        { 0, "SR4", "Exception Cause Register", 4 },
-        { 0, "SR5", "Program Status Word", 4 },
-        { 0, "SR6", "Processor ID Register", 4 },
-        { 0, "SR7", "Task Control Word", 4 },
-        { 0, "SR24", "Cache Control Word", 4 },
-        { 0, "SR25", "Address Trap Register", 4 },
-	{ 0, "IPEND", "Interrupts Pending", 2 },
-        { 0, "IMASK", "Interrupt Mask", 2 },
-        { 0, "IPRIO0", "Interrupt Priority Register 0", 2 },
-        { 0, "IPRIO1", "Interrupt Priority Register 1", 2 },
-        { 0, "TCTRL", "Timer Control", 2 },
-        { 0, "TPRD", "Timer Period", 2 },
-        { 0, "TCNTR", "Timer Counter", 3 },
-
-	{ 0, "KPCTRL0", "Keyport 0 Control", 1 },
-	{ 0, "KPCTRL1", "Keyport 1 Control", 1 },
-
-	{ 0, "TStamp", "Timestamp", 3 },
-        { 0, "", "", 0 },
-};
-
-static RegType PCFXRegs1[] =
-{
-	{ 0, "AR", "Active Register", 1 },
-	{ 0, "MPROGADDR", "Micro-program Address", 2 },
-        { 0, "MPROGCTRL", "Micro-program Control", 2 },
-
-	{ 0, "PAGESET", "KRAM Page Settings", 2 },
-        { 0, "RTCTRL", "Rainbow Transfer Control", 2 },
-        { 0, "RKRAMA", "Rainbow Transfer K-RAM Address", 3},
-        { 0, "RSTART", "Rainbow Transfer Start Position", 2},
-        { 0, "RCOUNT", "Rainbow Transfer Block Count", 2 },
-        { 0, "RIRQLINE", "Raster IRQ Line", 2 },
-	{ 0, "KRAMWA", "K-RAM Write Address", 4 },
-	{ 0, "KRAMRA", "K-RAM Read Address", 4 },
-        { 0, "DMATA", "DMA Transfer Address", 3 },
-        { 0, "DMATS", "DMA Transfer Size", 4 },
-        { 0, "DMASTT", "DMA Status", 2 },
-        { 0, "ADPCMCTRL", "ADPCM Control", 2 },
-        { 0, "ADPCMBM0", "ADPCM Buffer Mode Ch0", 2 },
-        { 0, "ADPCMBM1", "ADPCM Buffer Mode Ch1", 2 },
-        { 0, "ADPCMPA0", "ADPCM PlayAddress Ch0", 0x100 | 18 },
-        { 0, "ADPCMPA1", "ADPCM PlayAddress Ch1", 0x100 | 18 },
-        { 0, "ADPCMSA0", "ADPCM Start Address Ch0", 2 },
-        { 0, "ADPCMSA1", "ADPCM Start Address Ch1", 2 },
-        { 0, "ADPCMIA0", "ADPCM Intermediate Address Ch0", 2 },
-        { 0, "ADPCMIA1", "ADPCM Intermediate Address Ch1", 2 },
-        { 0, "ADPCMEA0", "ADPCM End Address Ch0", 0x100 | 18 },
-        { 0, "ADPCMEA1", "ADPCM End Address Ch1", 0x100 | 18 },
-	{ 0, "ADPCMStat", "ADPCM Status Register", 1 },
-	{ 0, "Reg01", "KING Register 0x01", 1 },
-	{ 0, "Reg02", "KING Register 0x02", 1 },
-	{ 0, "Reg03", "KING Register 0x03", 1 },
-	{ 0, "SUBCC", "Sub-channel Control", 1 },
-	{ 0, "------", "", 0xFFFF },
-	{ 0, "DB", "SCSI Data Bus", 0x100 | 8 },
-	{ 0, "BSY", "SCSI BSY", 0x100 | 1 },
-	{ 0, "REQ", "SCSI REQ", 0x100 | 1 },
-	{ 0, "ACK", "SCSI ACK", 0x100 | 1 },
-	{ 0, "MSG", "SCSI MSG", 0x100 | 1 },
-	{ 0, "IO", "SCSI IO", 0x100 | 1 },
-	{ 0, "CD", "SCSI CD", 0x100 | 1 },
-	{ 0, "SEL", "SCSI SEL", 0x100 | 1 },
-
-        { 0, "", "", 0 },
-};
-
-static RegType PCFXRegs4[] =
-{
-	{ 0, "--VCE--", "", 0xFFFF },
-        { 0, "Line", "VCE Frame Counter", 0x100 | 9 },
-        { 0, "PRIO0", "VCE Priority 0", 0x100 | 12 },
-        { 0, "PRIO1", "VCE Priority 1", 2 },
-        { 0, "PICMODE", "VCE Picture Mode", 2},
-        { 0, "PALRWOF", "VCE Palette R/W Offset", 2 },
-        { 0, "PALRWLA", "VCE Palette R/W Latch", 2 },
-        { 0, "PALOFS0", "VCE Palette Offset 0", 2 } ,
-        { 0, "PALOFS1", "VCE Palette Offset 1", 2 },
-        { 0, "PALOFS2", "VCE Palette Offset 2", 2 },
-        { 0, "PALOFS3", "VCE Palette Offset 3", 1 },
-        { 0, "CCR", "VCE Fixed Color Register", 2 },
-        { 0, "BLE", "VCE Cellophane Setting Register", 2 },
-        { 0, "SPBL", "VCE Sprite Cellophane Setting Register", 2 },
-        { 0, "COEFF0", "VCE Cellophane Coefficient 0(1A)", 0x100 | 12 },
-        { 0, "COEFF1", "VCE Cellophane Coefficient 1(1B)", 0x100 | 12 },
-        { 0, "COEFF2", "VCE Cellophane Coefficient 2(2A)", 0x100 | 12 },
-        { 0, "COEFF3", "VCE Cellophane Coefficient 3(2B)", 0x100 | 12 },
-        { 0, "COEFF4", "VCE Cellophane Coefficient 4(3A)", 0x100 | 12 },
-        { 0, "COEFF5", "VCE Cellophane Coefficient 5(3B)", 0x100 | 12 },
-        { 0, "CKeyY", "VCE Chroma Key Y", 2 },
-        { 0, "CKeyU", "VCE Chroma Key U", 2 },
-        { 0, "CKeyV", "VCE Chroma Key V", 2 },
-        { 0, "", "", 0 },
-};
-
-static uint32 GetRegister_VDC(const unsigned int id, char *special, const uint32 special_len)
-{
- return(fx_vdc_chips[(id >> 15) & 1]->GetRegister(id &~0x8000, special, special_len));
-}
-
-static void SetRegister_VDC(const unsigned int id, uint32 value)
-{
- fx_vdc_chips[(id >> 15) & 1]->SetRegister(id &~0x8000, value);
-} 
-
-
-static RegType VDCRegs[] =
-{
-        { 0, "--VDC-A--", "", 0xFFFF },
-	
-        { VDC::GSREG_SELECT, "Select", "Register Select, VDC-A", 1 },
-        { VDC::GSREG_STATUS, "Status", "Status, VDC-A", 1 },
-
-        { VDC::GSREG_MAWR, "MAWR", "Memory Write Address, VDC-A", 2 },
-        { VDC::GSREG_MARR, "MARR", "Memory Read Address, VDC-A", 2 },
-        { VDC::GSREG_CR, "CR", "Control, VDC-A", 2 },
-        { VDC::GSREG_RCR, "RCR", "Raster Counter, VDC-A", 2 },
-        { VDC::GSREG_BXR, "BXR", "X Scroll, VDC-A", 2 },
-        { VDC::GSREG_BYR, "BYR", "Y Scroll, VDC-A", 2 },
-        { VDC::GSREG_MWR, "MWR", "Memory Width, VDC-A", 2 },
-
-        { VDC::GSREG_HSR, "HSR", "HSR, VDC-A", 2 },
-        { VDC::GSREG_HDR, "HDR", "HDR, VDC-A", 2 },
-        { VDC::GSREG_VSR, "VSR", "VSR, VDC-A", 2 },
-        { VDC::GSREG_VDR, "VDR", "VDR, VDC-A", 2 },
-
-        { VDC::GSREG_VCR, "VCR", "VCR, VDC-A", 2 },
-        { VDC::GSREG_DCR, "DCR", "DMA Control, VDC-A", 2 },
-        { VDC::GSREG_SOUR, "SOUR", "VRAM DMA Source Address, VDC-A", 2 },
-        { VDC::GSREG_DESR, "DESR", "VRAM DMA Dest Address, VDC-A", 2 },
-        { VDC::GSREG_LENR, "LENR", "VRAM DMA Length, VDC-A", 2 },
-        { VDC::GSREG_DVSSR, "DVSSR", "DVSSR Update Address, VDC-A", 2 },
-        { 0, "------", "", 0xFFFF },
-
-        { 0, "--VDC-B--", "", 0xFFFF },
-
-        { 0x8000 | VDC::GSREG_SELECT, "Select", "Register Select, VDC-B", 1 },
-        { 0x8000 | VDC::GSREG_STATUS, "Status", "Status, VDC-B", 1 },
-
-        { 0x8000 | VDC::GSREG_MAWR, "MAWR", "Memory Write Address, VDC-B", 2 },
-        { 0x8000 | VDC::GSREG_MARR, "MARR", "Memory Read Address, VDC-B", 2 },
-        { 0x8000 | VDC::GSREG_CR, "CR", "Control, VDC-B", 2 },
-        { 0x8000 | VDC::GSREG_RCR, "RCR", "Raster Counter, VDC-B", 2 },
-        { 0x8000 | VDC::GSREG_BXR, "BXR", "X Scroll, VDC-B", 2 },
-        { 0x8000 | VDC::GSREG_BYR, "BYR", "Y Scroll, VDC-B", 2 },
-        { 0x8000 | VDC::GSREG_MWR, "MWR", "Memory Width, VDC-B", 2 },
-
-        { 0x8000 | VDC::GSREG_HSR, "HSR", "HSR, VDC-B", 2 },
-        { 0x8000 | VDC::GSREG_HDR, "HDR", "HDR, VDC-B", 2 },
-        { 0x8000 | VDC::GSREG_VSR, "VSR", "VSR, VDC-B", 2 },
-        { 0x8000 | VDC::GSREG_VDR, "VDR", "VDR, VDC-B", 2 },
-
-        { 0x8000 | VDC::GSREG_VCR, "VCR", "VCR, VDC-B", 2 },
-        { 0x8000 | VDC::GSREG_DCR, "DCR", "DMA Control, VDC-B", 2 },
-        { 0x8000 | VDC::GSREG_SOUR, "SOUR", "VRAM DMA Source Address, VDC-B", 2 },
-        { 0x8000 | VDC::GSREG_DESR, "DESR", "VRAM DMA Dest Address, VDC-B", 2 },
-        { 0x8000 | VDC::GSREG_LENR, "LENR", "VRAM DMA Length, VDC-B", 2 },
-        { 0x8000 | VDC::GSREG_DVSSR, "DVSSR", "DVSSR Update Address, VDC-B", 2 },
-
-        { 0, "", "", 0 },
-};
-
-static RegGroupType PCFXRegs0Group =
-{
- NULL,
- PCFXRegs0,
- NULL,
- NULL,
- PCFXDBG_GetRegister,
- PCFXDBG_SetRegister,
-};
-
-static RegGroupType PCFXRegs1Group =
-{
- NULL,
- PCFXRegs1,
- NULL,
- NULL,
- KING_GetRegister,
- KING_SetRegister
-};
-
-
-static RegGroupType PCFXRegs4Group =
-{
- NULL,
- PCFXRegs4,
- NULL,
- NULL,
- FXVDCVCE_GetRegister,
- FXVDCVCE_SetRegister
-};
-
-static RegGroupType VDCRegsGroup =
-{
- NULL,
- VDCRegs,
- GetRegister_VDC,
- SetRegister_VDC
-};
-
-
-static void SetRegGroups(void)
-{
- MDFNDBG_AddRegGroup(&PCFXRegs0Group);
- MDFNDBG_AddRegGroup(&PCFXRegs4Group);
- MDFNDBG_AddRegGroup(&VDCRegsGroup);
- MDFNDBG_AddRegGroup(&PCFXRegs1Group);
-}
-
-static void ForceIRQ(int level)
-{
- //v810_int(level);
-}
-
-static DebuggerInfoStruct DBGInfo =
-{
- "shift_jis",
- 4,
- 2,             // Instruction alignment(bytes)
- 32,
- 32,
- 0x00000000,
- ~0U,
-
- PCFXDBG_MemPeek,
- PCFXDBG_Disassemble,
- NULL,
- ForceIRQ,
- NULL, //NESDBG_GetVector,
- PCFXDBG_FlushBreakPoints,
- PCFXDBG_AddBreakPoint,
- PCFXDBG_SetCPUCallback,
- PCFXDBG_EnableBranchTrace,
- PCFXDBG_GetBranchTrace,
- KING_SetGraphicsDecode,
- PCFXDBG_SetLogFunc,
-};
-#endif
 
 static const FileExtensionSpecStruct KnownExtensions[] =
 {
@@ -1443,20 +1116,26 @@ MDFNGI EmulatedPCFX =
  KnownExtensions,
  MODPRIO_INTERNAL_HIGH,
  #ifdef WANT_DEBUGGER
- &DBGInfo,
+ &PCFXDBGInfo,
  #else
  NULL,
  #endif
- &PCFXInputInfo,
+ PCFXPortInfo,
  NULL,
  NULL,
  LoadCD,
  TestMagicCD,
  CloseGame,
+
  KING_SetLayerEnableMask,
  "BG0\0BG1\0BG2\0BG3\0VDC-A BG\0VDC-A SPR\0VDC-B BG\0VDC-B SPR\0RAINBOW\0",
+
  NULL,
  NULL,
+
+ NULL,
+ 0,
+
  NULL,
  NULL,
  NULL,
@@ -1464,7 +1143,9 @@ MDFNGI EmulatedPCFX =
  false,
  StateAction,
  Emulate,
+ FXINPUT_TransformInput,
  FXINPUT_SetInput,
+ SetMedia,
  DoSimpleCommand,
  PCFXSettings,
  MDFN_MASTERCLOCK_FIXED(PCFX_MASTER_CLOCK),
