@@ -248,16 +248,13 @@ static void KillHuffmanLUT(HuffmanQuickLUT *qlut)
  qlut->lut_bits = NULL;
 }
 
-static bool BuildHuffmanLUT(const HuffmanTable *table, HuffmanQuickLUT *qlut, const int bitmax)
+static void BuildHuffmanLUT(const HuffmanTable *table, HuffmanQuickLUT *qlut, const int bitmax)
 {
  // TODO: Allocate only (1 << bitmax) entries.
  // TODO: What should we set invalid bitsequences/entries to? 0? ~0?  Something else?
 
- if(!(qlut->lut = (uint8 *)MDFN_calloc(1 << 12, 1, _("Huffman LUT"))))
-  return(FALSE);
-
- if(!(qlut->lut_bits = (uint8 *)MDFN_calloc(1 << 12, 1, _("Huffman LUT"))))
-  return(FALSE);
+ qlut->lut = (uint8 *)MDFN_calloc_T(1 << 12, 1, _("Huffman LUT"));
+ qlut->lut_bits = (uint8 *)MDFN_calloc_T(1 << 12, 1, _("Huffman LUT"));
 
  for(int numbits = 2; numbits <= 12; numbits++)
  {
@@ -284,8 +281,6 @@ static bool BuildHuffmanLUT(const HuffmanTable *table, HuffmanQuickLUT *qlut, co
   //if(!qlut->lut_bits[i])
   // printf("%d %d\n", i, qlut->lut_bits[i]);
  }
-
- return(TRUE);
 }
 
 
@@ -525,117 +520,39 @@ static void decode(int32 *dct, const uint32 *QuantTable, const int32 dc, const H
 }
 
 #ifdef WANT_DEBUGGER
-static RegType RainbowRegs[] =
-{
- { 0, "RSCRLL", "Rainbow Horizontal Scroll", 2 },
- { 0, "RCTRL", "Rainbow Control", 2 },
- { 0, "RHSYNC", "Rainbow HSync?", 1 },
- { 0, "RNRY", "Rainbow Null Run Y", 2 },
- { 0, "RNRU", "Rainbow Null Run U", 2 },
- { 0, "RNRV", "Rainbow Null Run V", 2 },
- { 0, "------", "", 0xFFFF },
-        { 0, "BGMODE", "Background Mode", 2 },
-        { 0, "BGPRIO", "Background Priority", 2 },
-        { 0, "BGSCRM", "Background Scroll Mode", 2 },
-        { 0, "BGSIZ0", "Background 0 Size", 2 },
-        { 0, "BGSIZ1", "Background 1 Size", 1 },
-        { 0, "BGSIZ2", "Background 2 Size", 1 },
-        { 0, "BGSIZ3", "Background 3 Size", 1 },
-        { 0, "BGXSC0", "Background 0 X Scroll", 0x100 | 11 },
-        { 0, "BGXSC1", "Background 1 X Scroll", 0x100 | 10 },
-        { 0, "BGXSC2", "Background 2 X Scroll", 0x100 | 10 },
-        { 0, "BGXSC3", "Background 3 X Scroll", 0x100 | 10 },
-        { 0, "BGYSC0", "Background 0 Y Scroll", 0x100 | 11 },
-        { 0, "BGYSC1", "Background 1 Y Scroll", 0x100 | 10 },
-        { 0, "BGYSC2", "Background 2 Y Scroll", 0x100 | 10 },
-        { 0, "BGYSC3", "Background 3 Y Scroll", 0x100 | 10 },
-
-        { 0, "BGBAT0", "Background 0 BAT Address", 1 },
-        { 0, "BGBATS", "Background SUB BAT Address", 1 },
-        { 0, "BGBAT1", "Background 1 BAT Address", 1 },
-        { 0, "BGBAT2", "Background 2 BAT Address", 1 },
-        { 0, "BGBAT3", "Background 3 BAT Address", 1 },
-        { 0, "BGCG0", "Background 0 CG Address", 1 },
-        { 0, "BGCGS", "Background SUB CG Address", 1 },
-        { 0, "BGCG1", "Background 1 CG Address", 1 },
-        { 0, "BGCG2", "Background 2 CG Address", 1 },
-        { 0, "BGCG3", "Background 3 CG Address", 1 },
-
- { 0, "AFFINA", "Background Affin Coefficient A", 2 },
- { 0, "AFFINB", "Background Affin Coefficient B", 2 },
- { 0, "AFFINC", "Background Affin Coefficient C", 2 },
- { 0, "AFFIND", "Background Affin Coefficient D", 2 },
- { 0, "AFFINX", "Background Affin Center X", 2 },
- { 0, "AFFINY", "Background Affin Center Y", 2 },
- { 0, "MPROG0", "Micro-program", 2},
- { 0, "MPROG1", "Micro-program", 2},
- { 0, "MPROG2", "Micro-program", 2},
- { 0, "MPROG3", "Micro-program", 2},
- { 0, "MPROG4", "Micro-program", 2},
- { 0, "MPROG5", "Micro-program", 2},
- { 0, "MPROG6", "Micro-program", 2},
- { 0, "MPROG7", "Micro-program", 2},
- { 0, "MPROG8", "Micro-program", 2},
- { 0, "MPROG9", "Micro-program", 2},
- { 0, "MPROGA", "Micro-program", 2},
- { 0, "MPROGB", "Micro-program", 2},
- { 0, "MPROGC", "Micro-program", 2},
- { 0, "MPROGD", "Micro-program", 2},
- { 0, "MPROGE", "Micro-program", 2},
- { 0, "MPROGF", "Micro-program", 2},
-
- // EVIL EVIL:
-
- { 0, "", "", 0 },
-};
-
-static uint32 RainbowDBG_GetRegister(const std::string &name, std::string *special)
+uint32 RAINBOW_GetRegister(const unsigned int id, char* special, const uint32 special_len)
 {
  uint32 value = 0xDEADBEEF;
 
- if(name ==  "RSCRLL")
+ if(id == RAINBOW_GSREG_RSCRLL)
   value = HScroll;
- else if(name == "RCTRL")
+ else if(id == RAINBOW_GSREG_RCTRL)
   value = Control;
- else if(name == "RNRY")
+ else if(id == RAINBOW_GSREG_RNRY)
   value = NullRunY;
- else if(name == "RNRU")
+ else if(id == RAINBOW_GSREG_RNRU)
   value = NullRunU;
- else if(name == "RNRV")
+ else if(id == RAINBOW_GSREG_RNRV)
   value = NullRunV;
- else if(name == "RHSYNC")
+ else if(id == RAINBOW_GSREG_RHSYNC)
   value = HSync;
- else
-  value = KING_GetRegister(name, special); // EVIL EVIL
+
  return(value);
 }
 
-static void RainbowDBG_SetRegister(const std::string &name, uint32 value)
+void RAINBOW_SetRegister(const unsigned int id, uint32 value)
 {
- if(name == "RSCRLL")
+ if(id == RAINBOW_GSREG_RSCRLL)
   HScroll = value & 0x1FF;
- else if(name == "RCTRL")
+ else if(id == RAINBOW_GSREG_RCTRL)
   Control = value;
- else if(name == "RNRY")
+ else if(id == RAINBOW_GSREG_RNRY)
   NullRunY = value;
- else if(name == "RNRU")
+ else if(id == RAINBOW_GSREG_RNRU)
   NullRunU = value;
- else if(name == "RNRV")
+ else if(id == RAINBOW_GSREG_RNRV)
   NullRunV = value;
- else
-  KING_SetRegister(name, value);
 }
-
-static RegGroupType RainbowRegsGroup =
-{
- NULL,
- RainbowRegs,
- NULL,
- NULL,
- RainbowDBG_GetRegister,
- RainbowDBG_SetRegister
-};
-
 #endif
 
 static uint32 LastLine[256];
@@ -661,10 +578,6 @@ void RAINBOW_Init(bool arg_ChromaIP)
 {
  try
  {
-  #ifdef WANT_DEBUGGER
-  MDFNDBG_AddRegGroup(&RainbowRegsGroup);
-  #endif
-
   ChromaIP = arg_ChromaIP;
 
   for(int i = 0; i < 2; i++)
@@ -1131,7 +1044,7 @@ void RAINBOW_Reset(void)
 }
 
 
-int RAINBOW_StateAction(StateMem *sm, int load, int data_only)
+void RAINBOW_StateAction(StateMem *sm, const unsigned load, const bool data_only)
 {
  SFORMAT StateRegs[] =
  {
@@ -1150,13 +1063,12 @@ int RAINBOW_StateAction(StateMem *sm, int load, int data_only)
    SFEND
  };
 
- int ret = MDFNSS_StateAction(sm, load, data_only, StateRegs, "RBOW");
+ MDFNSS_StateAction(sm, load, data_only, StateRegs, "RBOW");
 
  if(load)
  {
   RasterReadPos &= 0xF;
   CalcHappyColor();
  }
- return(ret);
 }
 
