@@ -259,12 +259,18 @@ static void PutAddressSpaceBytes(const char *name, uint32 Address, uint32 Length
  {
   while(Length--)
   {
-   pscpu_timestamp_t dummy = 0;
    Address &= 0xFFFFFFFF;
-
-   // TODO
-   PSX_MemWrite8(dummy, Address, *Buffer);
-
+   CPU->PokeMem8(Address, *Buffer);
+   Address++;
+   Buffer++;
+  }
+ }
+ else if(!strcmp(name, "ram"))
+ {
+  while(Length--)
+  {
+   Address &= 0x1FFFFF;
+   CPU->PokeMem8(Address, *Buffer);
    Address++;
    Buffer++;
   }
@@ -462,14 +468,14 @@ static RegType Regs_SPU[] =
  { PS_SPU::GSREG_CDVOL_L, "CDVolL", "CD Volume Left", 2 },
  { PS_SPU::GSREG_CDVOL_R, "CDVolR", "CD Volume Right", 2 },
 
- { PS_SPU::GSREG_DRYVOL_CTRL_L, "DryVolCL", "Dry Volume Control Left", 2 },
- { PS_SPU::GSREG_DRYVOL_CTRL_R, "DryVolCR", "Dry Volume Control Right", 2 },
+ { PS_SPU::GSREG_RVBVOL_L, "RvbVolL", "Reverb Volume Left", 2 },
+ { PS_SPU::GSREG_RVBVOL_R, "RvbVolR", "Reverb Volume Right", 2 },
 
- { PS_SPU::GSREG_DRYVOL_L, "DryVolL", "Dry Volume Left", 2 },
- { PS_SPU::GSREG_DRYVOL_R, "DryVolR", "Dry Volume Right", 2 },
+ { PS_SPU::GSREG_MAINVOL_CTRL_L, "MainVolCL", "Main Volume Control Left", 2 },
+ { PS_SPU::GSREG_MAINVOL_CTRL_R, "MainVolCR", "Main Volume Control Right", 2 },
 
- { PS_SPU::GSREG_WETVOL_L, "WetVolL", "Wet Volume Left", 2 },
- { PS_SPU::GSREG_WETVOL_R, "WetVolR", "Wet Volume Right", 2 },
+ { PS_SPU::GSREG_MAINVOL_L, "MainVolL", "Dry Volume Left", 2 },
+ { PS_SPU::GSREG_MAINVOL_R, "MainVolR", "Dry Volume Right", 2 },
 
  { PS_SPU::GSREG_RWADDR, "RWAddr", "SPURAM Read/Write Address", 3 },
 
@@ -662,7 +668,7 @@ bool DBG_Init(void)
  MDFNDBG_AddRegGroup(&SPURegsGroup);
  MDFNDBG_AddRegGroup(&SPUVoicesRegsGroup);
  ASpace_Add(GetAddressSpaceBytes, PutAddressSpaceBytes, "cpu", "CPU Physical", 32);
- ASpace_Add(GetAddressSpaceBytes, PutAddressSpaceBytes, "ram", "CPU Main Ram", 21);
+ ASpace_Add(GetAddressSpaceBytes, PutAddressSpaceBytes, "ram", "CPU Main RAM", 21);
  ASpace_Add(GetAddressSpaceBytes, PutAddressSpaceBytes, "spu", "SPU RAM", 19);
  ASpace_Add(GetAddressSpaceBytes, PutAddressSpaceBytes, "gpu", "GPU RAM", 20);
  return(true);
