@@ -20,10 +20,6 @@
 #include "cdc.h"
 #include "spu.h"
 
-//#include <map>
-
-// Notes: DMA tested to abort when 
-
 /* Notes:
 
  Channel 4(SPU):
@@ -100,11 +96,8 @@ static INLINE void RecalcIRQOut(void)
 {
  bool irqo;
 
- irqo = (bool)(DMAIntStatus & ((DMAIntControl >> 16) & 0x7F));
+ irqo = (bool)DMAIntStatus;
  irqo &= (DMAIntControl >> 23) & 1;
-
- // I think it's logical OR, not XOR/invert.  Still kind of weird, maybe it actually does something more complicated?
- //irqo ^= (DMAIntControl >> 15) & 1;
  irqo |= (DMAIntControl >> 15) & 1;
 
  IRQOut = irqo;
@@ -130,8 +123,6 @@ void DMA_Power(void)
  RecalcIRQOut();
 }
 
-void PSX_SetDMASuckSuck(unsigned);
-
 static INLINE bool ChCan(const unsigned ch, const uint32 CRModeCache)
 {
  switch(ch)
@@ -229,19 +220,16 @@ static void RecalcHalt(void)
   if(tmp > 0)
    tmp--;
 
-  if(tmp > 200)	// Due to 8-bit limitations in the CPU core.
-   tmp = 200;
-
-  PSX_SetDMASuckSuck(tmp);
+  PSX_SetDMACycleSteal(tmp);
  }
  else
-  PSX_SetDMASuckSuck(0);
+  PSX_SetDMACycleSteal(0);
 
  CPU->SetHalt(Halt);
 }
 
 
-static INLINE void ChRW(const unsigned ch, const uint32 CRModeCache, uint32 *V, int32 *offset)
+static INLINE void ChRW(const unsigned ch, const uint32 CRModeCache, uint32 *V, uint32 *offset)
 {
  unsigned extra_cyc_overhead = 0;
 
@@ -427,7 +415,7 @@ static INLINE void RunChannelI(const unsigned ch, const uint32 CRModeCache, int3
   //
   {
    uint32 vtmp;
-   int32 voffs = 0;
+   uint32 voffs = 0;
 
    if(MDFN_UNLIKELY(DMACH[ch].CurAddr & 0x800000))
    {
@@ -651,19 +639,8 @@ void DMA_Write(const pscpu_timestamp_t timestamp, uint32 A, uint32 V)
 	     break;
 
    case 0x4: 
-	     //for(int x = 0; x < 7; x++)
-	     //{
-             // if(DMACH[x].WordCounter || (DMACH[x].ChanControl & (1 << 24)))
-	     // {
-	     //  fprintf(stderr, "Write DMAIntControl while channel %d active: 0x%08x\n", x, V);
-	     // }
-	     //}
 	     DMAIntControl = V & 0x00ff803f;
 	     DMAIntStatus &= ~(V >> 24);
-
-	     //if(DMAIntStatus ^ (DMAIntStatus & (V >> 16)))
-	     // fprintf(stderr, "DMAINT Fudge: %02x\n", DMAIntStatus ^ (DMAIntStatus & (V >> 16)));
-	     DMAIntStatus &= (V >> 16);	// THIS IS ALMOST CERTAINLY WRONG AND A HACK.  Remove when CDC emulation is better.
      	     RecalcIRQOut();
 	     break;
 
@@ -778,7 +755,7 @@ uint32 DMA_Read(const pscpu_timestamp_t timestamp, uint32 A)
 }
 
 
-int DMA_StateAction(StateMem *sm, int load, int data_only)
+void DMA_StateAction(StateMem *sm, const unsigned load, const bool data_only)
 {
  SFORMAT StateRegs[] =
  {
@@ -807,14 +784,12 @@ int DMA_StateAction(StateMem *sm, int load, int data_only)
 
   SFEND
  };
- int ret = MDFNSS_StateAction(sm, load, data_only, StateRegs, "DMA");
+ MDFNSS_StateAction(sm, load, data_only, StateRegs, "DMA");
 
  if(load)
  {
 
  }
-
- return(ret);
 }
 
 
