@@ -47,9 +47,9 @@ void InputDevice::Power(void)
 {
 }
 
-int InputDevice::StateAction(StateMem* sm, int load, int data_only, const char* section_name)
+void InputDevice::StateAction(StateMem* sm, const unsigned load, const bool data_only, const char* sname_prefix)
 {
- return(1);
+
 }
 
 
@@ -77,7 +77,7 @@ void InputDevice::SetCrosshairsColor(uint32 color)
  draw_chair = (color != (1 << 24));
 }
 
-INLINE void InputDevice::DrawCrosshairs(uint32 *pixels, const MDFN_PixelFormat* const format, const unsigned width, const unsigned pix_clock)
+void InputDevice::DrawCrosshairs(uint32 *pixels, const MDFN_PixelFormat* const format, const unsigned width, const unsigned pix_clock)
 {
  if(draw_chair && chair_y >= -8 && chair_y <= 8)
  {
@@ -157,14 +157,14 @@ bool InputDevice::Clock(bool TxD, int32 &dsr_pulse_delay)
  return(1);
 }
 
-uint32 InputDevice::GetNVSize(void)
+uint32 InputDevice::GetNVSize(void) const
 {
  return(0);
 }
 
-void InputDevice::ReadNV(uint8 *buffer, uint32 offset, uint32 count)
+const uint8* InputDevice::ReadNV(void) const
 {
-
+ return NULL;
 }
 
 void InputDevice::WriteNV(const uint8 *buffer, uint32 offset, uint32 count)
@@ -172,7 +172,7 @@ void InputDevice::WriteNV(const uint8 *buffer, uint32 offset, uint32 count)
 
 }
 
-uint64 InputDevice::GetNVDirtyCount(void)
+uint64 InputDevice::GetNVDirtyCount(void) const
 {
  return(0);
 }
@@ -216,32 +216,32 @@ void FrontIO::MapDevicesToPorts(void)
  {
   for(unsigned i = 0; i < 2; i++)
   {
-   Ports[i] = DevicesTap[i];
-   MCPorts[i] = DummyDevice;
+   Ports[i] = PossibleMultitaps[i].get();
+   MCPorts[i] = PossibleNone.get();
   }
  }
  else if(!emulate_multitap[0] && emulate_multitap[1])
  {
   Ports[0] = Devices[0];
-  MCPorts[0] = emulate_memcards[0] ? DevicesMC[0] : DummyDevice;
+  MCPorts[0] = MCDevices[0];
 
-  Ports[1] = DevicesTap[1];
-  MCPorts[1] = DummyDevice;
+  Ports[1] = PossibleMultitaps[1].get();
+  MCPorts[1] = PossibleNone.get();
  }
  else if(emulate_multitap[0] && !emulate_multitap[1])
  {
-  Ports[0] = DevicesTap[0];
-  MCPorts[0] = DummyDevice;
+  Ports[0] = PossibleMultitaps[0].get();
+  MCPorts[0] = PossibleNone.get();
 
   Ports[1] = Devices[4];
-  MCPorts[1] = emulate_memcards[4] ? DevicesMC[4] : DummyDevice;
+  MCPorts[1] = MCDevices[4];
  }
  else
  {
   for(unsigned i = 0; i < 2; i++)
   {
    Ports[i] = Devices[i];
-   MCPorts[i] = emulate_memcards[i] ? DevicesMC[i] : DummyDevice;
+   MCPorts[i] = MCDevices[i];
   }
  }
 
@@ -251,38 +251,60 @@ void FrontIO::MapDevicesToPorts(void)
   unsigned mp = EP_to_MP(emulate_multitap, i);
   
   if(emulate_multitap[mp])
-   DevicesTap[mp]->SetSubDevice(EP_to_SP(emulate_multitap, i), Devices[i], emulate_memcards[i] ? DevicesMC[i] : DummyDevice);
+   PossibleMultitaps[mp]->SetSubDevice(EP_to_SP(emulate_multitap, i), Devices[i], MCDevices[i]);
   else
-   DevicesTap[mp]->SetSubDevice(EP_to_SP(emulate_multitap, i), DummyDevice, DummyDevice);
+   PossibleMultitaps[mp]->SetSubDevice(EP_to_SP(emulate_multitap, i), PossibleNone.get(), PossibleNone.get());
 
   //printf("%d-> multitap: %d, sub-port: %d\n", i, mp, EP_to_SP(emulate_multitap, i));
  }
 }
 
-FrontIO::FrontIO(bool emulate_memcards_[8], bool emulate_multitap_[2])
+FrontIO::FrontIO()
 {
- memcpy(emulate_memcards, emulate_memcards_, sizeof(emulate_memcards));
- memcpy(emulate_multitap, emulate_multitap_, sizeof(emulate_multitap));
-
- DummyDevice = new InputDevice();
+ PossibleNone.reset(new InputDevice());
 
  for(unsigned i = 0; i < 8; i++)
  {
+  PossibleDevices[i].Memcard.reset(Device_Memcard_Create());
+  PossibleDevices[i].Gamepad.reset(Device_Gamepad_Create());
+  PossibleDevices[i].DualAnalog.reset(Device_DualAnalog_Create(false));
+  PossibleDevices[i].AnalogJoy.reset(Device_DualAnalog_Create(true));
+  PossibleDevices[i].DualShock.reset(Device_DualShock_Create());
+  PossibleDevices[i].Mouse.reset(Device_Mouse_Create());
+  PossibleDevices[i].neGcon.reset(Device_neGcon_Create());
+  PossibleDevices[i].GunCon.reset(Device_GunCon_Create());
+  PossibleDevices[i].Justifier.reset(Device_Justifier_Create());
+ }
+
+ for(unsigned i = 0; i < 8; i++)
+ {
+  Devices[i] = PossibleNone.get();
   DeviceData[i] = NULL;
-  Devices[i] = new InputDevice();
-  DevicesMC[i] = Device_Memcard_Create();
+  MCDevices[i] = PossibleNone.get();
   chair_colors[i] = 1 << 24;
   Devices[i]->SetCrosshairsColor(chair_colors[i]);
  }
 
  for(unsigned i = 0; i < 2; i++)
  {
-  DevicesTap[i] = new InputDevice_Multitap();
+  PossibleMultitaps[i].reset(new InputDevice_Multitap());
  }
 
  MapDevicesToPorts();
 }
 
+void FrontIO::SetMultitap(unsigned int pport, bool enabled)
+{
+ assert(pport < 2);
+
+ if(emulate_multitap[pport] != enabled)
+ {
+  emulate_multitap[pport] = enabled;
+  MapDevicesToPorts();  
+  PossibleMultitaps[pport]->Power();
+ }
+}
+
 void FrontIO::SetAMCT(bool enabled)
 {
  for(unsigned i = 0; i < 8; i++)
@@ -294,7 +316,7 @@ void FrontIO::SetAMCT(bool enabled)
 
 void FrontIO::SetCrosshairsColor(unsigned port, uint32 color)
 {
- assert(port >= 0 && port < 8);
+ assert(port < 8);
 
  chair_colors[port] = color;
  Devices[port]->SetCrosshairsColor(color);
@@ -302,34 +324,7 @@ void FrontIO::SetCrosshairsColor(unsigned port, uint32 color)
 
 FrontIO::~FrontIO()
 {
- for(int i = 0; i < 8; i++)
- {
-  if(Devices[i])
-  {
-   delete Devices[i];
-   Devices[i] = NULL;
-  }
-  if(DevicesMC[i])
-  {
-   delete DevicesMC[i];
-   DevicesMC[i] = NULL;
-  }
- }
 
- for(unsigned i = 0; i < 2; i++)
- {
-  if(DevicesTap[i])
-  {
-   delete DevicesTap[i];
-   DevicesTap[i] = NULL;
-  }
- }
-
- if(DummyDevice)
- {
-  delete DummyDevice;
-  DummyDevice = NULL;
- }
 }
 
 pscpu_timestamp_t FrontIO::CalcNextEventTS(pscpu_timestamp_t timestamp, int32 next_event)
@@ -413,15 +408,17 @@ INLINE void FrontIO::DoDSRIRQ(void)
 
 void FrontIO::Write(pscpu_timestamp_t timestamp, uint32 A, uint32 V)
 {
- assert(!(A & 0x1));
-
  PSX_FIODBGINFO("[FIO] Write: %08x %08x", A, V);
 
+ V <<= (A & 1) * 8;
+
  Update(timestamp);
 
- switch(A & 0xF)
+ switch(A & 0xE)
  {
   case 0x0:
+  case 0x2:
+        V <<= (A & 2) * 8;
 	TransmitBuffer = V;
 	TransmitPending = true;
 	TransmitInProgress = false;
@@ -515,19 +512,19 @@ uint32 FrontIO::Read(pscpu_timestamp_t timestamp, uint32 A)
 {
  uint32 ret = 0;
 
- assert(!(A & 0x1));
-
  Update(timestamp);
 
- switch(A & 0xF)
+ switch(A & 0xE)
  {
   case 0x0:
+  case 0x2:
 	//printf("FIO Read: 0x%02x\n", ReceiveBuffer);
 	ret = ReceiveBuffer | (ReceiveBuffer << 8) | (ReceiveBuffer << 16) | (ReceiveBuffer << 24);
 	ReceiveBufferAvail = false;
 	ReceivePending = true;
 	ReceiveInProgress = false;
 	CheckStartStopPending(timestamp, false);
+	ret >>= (A & 2) * 8;
 	break;
 
   case 0x4:
@@ -560,6 +557,8 @@ uint32 FrontIO::Read(pscpu_timestamp_t timestamp, uint32 A)
 	break;
  }
 
+ ret >>= (A & 1) * 8;
+
  if((A & 0xF) != 0x4)
   PSX_FIODBGINFO("[FIO] Read: %08x %08x", A, ret);
 
@@ -668,19 +667,13 @@ pscpu_timestamp_t FrontIO::Update(pscpu_timestamp_t timestamp)
 
 void FrontIO::ResetTS(void)
 {
- for(int i = 0; i < 8; i++)
+ for(unsigned i = 0; i < 2; i++)
  {
-  Devices[i]->Update(lastts);	// Maybe eventually call Update() from FrontIO::Update() and remove this(but would hurt speed)?
-  Devices[i]->ResetTS();
+  Ports[i]->Update(lastts);	// Maybe eventually call Update() from FrontIO::Update() and remove this(but would hurt speed)?
+  Ports[i]->ResetTS();
 
-  DevicesMC[i]->Update(lastts);	// Maybe eventually call Update() from FrontIO::Update() and remove this(but would hurt speed)?
-  DevicesMC[i]->ResetTS();
- }
-
- for(int i = 0; i < 2; i++)
- {
-  DevicesTap[i]->Update(lastts);
-  DevicesTap[i]->ResetTS();
+  MCPorts[i]->Update(lastts);	// Maybe eventually call Update() from FrontIO::Update() and remove this(but would hurt speed)?
+  MCPorts[i]->ResetTS();
  }
 
  for(int i = 0; i < 2; i++)
@@ -701,7 +694,7 @@ void FrontIO::ResetTS(void)
 }
 
 
-void FrontIO::Power(void)
+void FrontIO::Reset(bool powering_up)
 {
  for(int i = 0; i < 4; i++)
  {
@@ -739,10 +732,13 @@ void FrontIO::Power(void)
  Control = 0;
  Baudrate = 0;
 
- for(int i = 0; i < 8; i++)
+ if(powering_up)
  {
-  Devices[i]->Power();
-  DevicesMC[i]->Power();
+  for(unsigned i = 0; i < 2; i++)
+  {
+   Ports[i]->Power();
+   MCPorts[i]->Power();
+  }
  }
 
  istatus = false;
@@ -750,75 +746,95 @@ void FrontIO::Power(void)
 
 void FrontIO::UpdateInput(void)
 {
- for(int i = 0; i < 8; i++)
+ for(unsigned i = 0; i < 8; i++)
   Devices[i]->UpdateInput(DeviceData[i]);
 }
 
-void FrontIO::SetInput(unsigned int port, const char *type, void *ptr)
+// Take care to call ->Power() only if the device actually changed.
+void FrontIO::SetInput(unsigned int port, const char *type, uint8 *ptr)
 {
- delete Devices[port];
- Devices[port] = NULL;
-
- if(port < 2)
-  irq10_pulse_ts[port] = PSX_EVENT_MAXTS;
+ InputDevice* nd = PossibleNone.get();
 
  if(!strcmp(type, "gamepad") || !strcmp(type, "dancepad"))
-  Devices[port] = Device_Gamepad_Create();
+  nd = PossibleDevices[port].Gamepad.get();
  else if(!strcmp(type, "dualanalog"))
-  Devices[port] = Device_DualAnalog_Create(false);
+  nd = PossibleDevices[port].DualAnalog.get();
  else if(!strcmp(type, "analogjoy"))
-  Devices[port] = Device_DualAnalog_Create(true);
+  nd = PossibleDevices[port].AnalogJoy.get();
  else if(!strcmp(type, "dualshock"))
- {
-  char name[256];
-  trio_snprintf(name, 256, _("DualShock on port %u"), port + 1);
-  Devices[port] = Device_DualShock_Create(std::string(name));
- }
+  nd = PossibleDevices[port].DualShock.get();
  else if(!strcmp(type, "mouse"))
-  Devices[port] = Device_Mouse_Create();
+  nd = PossibleDevices[port].Mouse.get();
  else if(!strcmp(type, "negcon"))
-  Devices[port] = Device_neGcon_Create();
+  nd = PossibleDevices[port].neGcon.get();
  else if(!strcmp(type, "guncon"))
-  Devices[port] = Device_GunCon_Create();
+  nd = PossibleDevices[port].GunCon.get();
  else if(!strcmp(type, "justifier"))
-  Devices[port] = Device_Justifier_Create();
- else
-  Devices[port] = new InputDevice();
+  nd = PossibleDevices[port].Justifier.get();
+ else if(strcmp(type, "none"))
+  abort();
 
- Devices[port]->SetAMCT(amct_enabled);
- Devices[port]->SetCrosshairsColor(chair_colors[port]);
- DeviceData[port] = ptr;
+ if(Devices[port] != nd)
+ {
+  if(port < 2)
+   irq10_pulse_ts[port] = PSX_EVENT_MAXTS;
 
- MapDevicesToPorts();
+  Devices[port] = nd;
+// ResetTS?
+  Devices[port]->Power();
+  Devices[port]->SetAMCT(amct_enabled);
+  Devices[port]->SetCrosshairsColor(chair_colors[port]);
+  DeviceData[port] = ptr;
+
+  MapDevicesToPorts();
+ }
 }
 
+// Take care to call ->Power() only if the device actually changed.
+void FrontIO::SetMemcard(unsigned int port, bool enabled)
+{
+ assert(port < 8);
+ InputDevice* nd = enabled ? PossibleDevices[port].Memcard.get() : PossibleNone.get();
+
+ if(MCDevices[port] != nd)
+ {
+  MCDevices[port] = nd;
+  MCDevices[port]->Power();
+  MapDevicesToPorts();
+ }
+}
+
+//
+// The memcard functions here should access PossibleDevices[which].Memcard, and NOT MCDevices, so that dynamically disabling/enabling
+// memory cards will work properly and not cause data loss(when we add the feature in the future).
+//
 uint64 FrontIO::GetMemcardDirtyCount(unsigned int which)
 {
  assert(which < 8);
 
- return(DevicesMC[which]->GetNVDirtyCount());
+ return(PossibleDevices[which].Memcard->GetNVDirtyCount());
 }
 
-void FrontIO::LoadMemcard(unsigned int which, const char *path)
+void FrontIO::LoadMemcard(unsigned int which, const std::string& path)
 {
  assert(which < 8);
 
  try
  {
-  if(DevicesMC[which]->GetNVSize())
+  if(PossibleDevices[which].Memcard->GetNVSize())
   {
    FileStream mf(path, FileStream::MODE_READ);
    std::vector<uint8> tmpbuf;
 
-   tmpbuf.resize(DevicesMC[which]->GetNVSize());
+   tmpbuf.resize(PossibleDevices[which].Memcard->GetNVSize());
 
-   if(mf.size() != (int64)tmpbuf.size())
-    throw(MDFN_Error(0, _("Memory card file \"%s\" is an incorrect size(%d bytes).  The correct size is %d bytes."), path, (int)mf.size(), (int)tmpbuf.size()));
+   if(mf.size() != tmpbuf.size())
+    throw(MDFN_Error(0, _("Memory card file \"%s\" is an incorrect size(%d bytes).  The correct size is %d bytes."), path.c_str(), (int)mf.size(), (int)tmpbuf.size()));
 
    mf.read(&tmpbuf[0], tmpbuf.size());
 
-   DevicesMC[which]->WriteNV(&tmpbuf[0], 0, tmpbuf.size());
-   DevicesMC[which]->ResetNVDirtyCount();		// There's no need to rewrite the file if it's the same data.
+   PossibleDevices[which].Memcard->WriteNV(&tmpbuf[0], 0, tmpbuf.size());
+   PossibleDevices[which].Memcard->ResetNVDirtyCount();		// There's no need to rewrite the file if it's the same data.
   }
  }
  catch(MDFN_Error &e)
@@ -828,27 +844,22 @@ void FrontIO::LoadMemcard(unsigned int which, const char *path)
  }
 }
 
-void FrontIO::SaveMemcard(unsigned int which, const char *path)
+void FrontIO::SaveMemcard(unsigned int which, const std::string& path)
 {
  assert(which < 8);
 
- if(DevicesMC[which]->GetNVSize() && DevicesMC[which]->GetNVDirtyCount())
+ if(PossibleDevices[which].Memcard->GetNVSize() && PossibleDevices[which].Memcard->GetNVDirtyCount())
  {
-  FileStream mf(path, FileStream::MODE_WRITE);	// TODO: MODE_WRITE_ATOMIC_OVERWRITE
-  std::vector<uint8> tmpbuf;
-
-  tmpbuf.resize(DevicesMC[which]->GetNVSize());
-
-  DevicesMC[which]->ReadNV(&tmpbuf[0], 0, tmpbuf.size());
-  mf.write(&tmpbuf[0], tmpbuf.size());
+  FileStream mf(path, FileStream::MODE_WRITE_INPLACE);
 
+  mf.write(PossibleDevices[which].Memcard->ReadNV(), PossibleDevices[which].Memcard->GetNVSize());
   mf.close();	// Call before resetting the NV dirty count!
 
-  DevicesMC[which]->ResetNVDirtyCount();
+  PossibleDevices[which].Memcard->ResetNVDirtyCount();
  }
 }
 
-int FrontIO::StateAction(StateMem* sm, int load, int data_only)
+void FrontIO::StateAction(StateMem* sm, const unsigned load, const bool data_only)
 {
  SFORMAT StateRegs[] =
  {
@@ -882,67 +893,57 @@ int FrontIO::StateAction(StateMem* sm, int load, int data_only)
   SFEND
  };
 
- int ret = MDFNSS_StateAction(sm, load, data_only, StateRegs, "FIO");
+ MDFNSS_StateAction(sm, load, data_only, StateRegs, "FIO");
 
- for(unsigned i = 0; i < 8; i++)
+ for(unsigned i = 0; i < 2; i++)
  {
   char tmpbuf[32];
-  trio_snprintf(tmpbuf, sizeof(tmpbuf), "FIODEV%u", i);
 
-  ret &= Devices[i]->StateAction(sm, load, data_only, tmpbuf);
- }
-
- for(unsigned i = 0; i < 8; i++)
- {
-  char tmpbuf[32];
-  trio_snprintf(tmpbuf, sizeof(tmpbuf), "FIOMC%u", i);
-
-  ret &= DevicesMC[i]->StateAction(sm, load, data_only, tmpbuf);
+  trio_snprintf(tmpbuf, sizeof(tmpbuf), "FIOP%u", i);
+  Ports[i]->StateAction(sm, load, data_only, tmpbuf);
  }
 
  for(unsigned i = 0; i < 2; i++)
  {
   char tmpbuf[32];
-  trio_snprintf(tmpbuf, sizeof(tmpbuf), "FIOTAP%u", i);
 
-  ret &= DevicesTap[i]->StateAction(sm, load, data_only, tmpbuf);
+  trio_snprintf(tmpbuf, sizeof(tmpbuf), "FIOMC%u", i);
+  MCPorts[i]->StateAction(sm, load, data_only, tmpbuf);
  }
 
  if(load)
  {
   IRQ_Assert(IRQ_SIO, istatus);
  }
-
- return(ret);
 }
 
 bool FrontIO::RequireNoFrameskip(void)
 {
- for(unsigned i = 0; i < 8; i++)
-  if(Devices[i]->RequireNoFrameskip())
-   return(true);
+ bool ret = false;
+
+ for(unsigned i = 0; i < 2; i++)
+ {
+  ret |= Ports[i]->RequireNoFrameskip();
+ }
  
- return(false);
+ return(ret);
 }
 
 void FrontIO::GPULineHook(const pscpu_timestamp_t timestamp, const pscpu_timestamp_t line_timestamp, bool vsync, uint32 *pixels, const MDFN_PixelFormat* const format, const unsigned width, const unsigned pix_clock_offset, const unsigned pix_clock, const unsigned pix_clock_divider)
 {
  Update(timestamp);
 
- for(unsigned i = 0; i < 8; i++)
+ for(unsigned i = 0; i < 2; i++)
  {
-  pscpu_timestamp_t plts = Devices[i]->GPULineHook(line_timestamp, vsync, pixels, format, width, pix_clock_offset, pix_clock, pix_clock_divider);
+  pscpu_timestamp_t plts = Ports[i]->GPULineHook(line_timestamp, vsync, pixels, format, width, pix_clock_offset, pix_clock, pix_clock_divider);
 
-  if(i < 2)
+  irq10_pulse_ts[i] = plts;
+
+  if(irq10_pulse_ts[i] <= timestamp)
   {
-   irq10_pulse_ts[i] = plts;
-
-   if(irq10_pulse_ts[i] <= timestamp)
-   {
-    irq10_pulse_ts[i] = PSX_EVENT_MAXTS;
-    IRQ_Assert(IRQ_PIO, true);
-    IRQ_Assert(IRQ_PIO, false);
-   }
+   irq10_pulse_ts[i] = PSX_EVENT_MAXTS;
+   IRQ_Assert(IRQ_PIO, true);
+   IRQ_Assert(IRQ_PIO, false);
   }
  }
 
@@ -951,25 +952,23 @@ void FrontIO::GPULineHook(const pscpu_timestamp_t timestamp, const pscpu_timesta
  //
  if(pixels && pix_clock)
  {
-  for(unsigned i = 0; i < 8; i++)
+  for(unsigned i = 0; i < 2; i++)
   {
-   Devices[i]->DrawCrosshairs(pixels, format, width, pix_clock);
+   Ports[i]->DrawCrosshairs(pixels, format, width, pix_clock);
   }
  }
 
  PSX_SetEventNT(PSX_EVENT_FIO, CalcNextEventTS(timestamp, 0x10000000));
 }
 
-static InputDeviceInfoStruct InputDeviceInfoPSXPort[] =
+static const std::vector<InputDeviceInfoStruct> InputDeviceInfoPSXPort =
 {
  // None
  {
   "none",
   "none",
   NULL,
-  NULL,
-  0,
-  NULL 
+  IDII_Empty
  },
 
  // Gamepad(SCPH-1080)
@@ -977,8 +976,6 @@ static InputDeviceInfoStruct InputDeviceInfoPSXPort[] =
   "gamepad",
   "Digital Gamepad",
   "PlayStation digital gamepad; SCPH-1080.",
-  NULL,
-  sizeof(Device_Gamepad_IDII) / sizeof(InputDeviceInputInfoStruct),
   Device_Gamepad_IDII,
  },
 
@@ -987,8 +984,6 @@ static InputDeviceInfoStruct InputDeviceInfoPSXPort[] =
   "dualshock",
   "DualShock",
   "DualShock gamepad; SCPH-1200.  Emulation in Mednafen includes the analog mode toggle button.  Rumble is emulated, but currently only supported on Linux, and MS Windows via the XInput API and XInput-compatible gamepads/joysticks.  If you're having trouble getting rumble to work on Linux, see if Mednafen is printing out error messages during startup regarding /dev/input/event*, and resolve the issue(s) as necessary.",
-  NULL,
-  sizeof(Device_DualShock_IDII) / sizeof(InputDeviceInputInfoStruct),
   Device_DualShock_IDII,
  },
 
@@ -997,8 +992,6 @@ static InputDeviceInfoStruct InputDeviceInfoPSXPort[] =
   "dualanalog",
   "Dual Analog",
   "Dual Analog gamepad; SCPH-1180.  It is the predecessor/prototype to the more advanced DualShock.  Emulated in Mednafen as forced to analog mode, and without rumble.",
-  NULL,
-  sizeof(Device_DualAnalog_IDII) / sizeof(InputDeviceInputInfoStruct),
   Device_DualAnalog_IDII,
  },
 
@@ -1008,8 +1001,6 @@ static InputDeviceInfoStruct InputDeviceInfoPSXPort[] =
   "analogjoy",
   "Analog Joystick",
   "Flight-game-oriented dual-joystick controller; SCPH-1110.   Emulated in Mednafen as forced to analog mode.",
-  NULL,
-  sizeof(Device_AnalogJoy_IDII) / sizeof(InputDeviceInputInfoStruct),
   Device_AnalogJoy_IDII,
  },
 
@@ -1017,8 +1008,6 @@ static InputDeviceInfoStruct InputDeviceInfoPSXPort[] =
   "mouse",
   "Mouse",
   NULL,
-  NULL,
-  sizeof(Device_Mouse_IDII) / sizeof(InputDeviceInputInfoStruct),
   Device_Mouse_IDII,
  },
 
@@ -1026,8 +1015,6 @@ static InputDeviceInfoStruct InputDeviceInfoPSXPort[] =
   "negcon",
   "neGcon",
   "Namco's unconventional twisty racing-game-oriented gamepad; NPC-101.",
-  NULL,
-  sizeof(Device_neGcon_IDII) / sizeof(InputDeviceInputInfoStruct),
   Device_neGcon_IDII,
  },
 
@@ -1035,8 +1022,6 @@ static InputDeviceInfoStruct InputDeviceInfoPSXPort[] =
   "guncon",
   "GunCon",
   "Namco's light gun; NPC-103.",
-  NULL,
-  sizeof(Device_GunCon_IDII) / sizeof(InputDeviceInputInfoStruct),
   Device_GunCon_IDII,
  },
 
@@ -1044,8 +1029,6 @@ static InputDeviceInfoStruct InputDeviceInfoPSXPort[] =
   "justifier",
   "Konami Justifier",
   "Konami's light gun; SLUH-00017.  Rumored to be wrought of the coagulated rage of all who tried to shoot The Dog.  If the game you want to play supports the \"GunCon\", you should use that instead. NOTE: Currently does not work properly when on any of ports 1B-1D and 2B-2D.",
-  NULL,
-  sizeof(Device_Justifier_IDII) / sizeof(InputDeviceInputInfoStruct),
   Device_Justifier_IDII,
  },
 
@@ -1053,30 +1036,21 @@ static InputDeviceInfoStruct InputDeviceInfoPSXPort[] =
   "dancepad",
   "Dance Pad",
   "Dingo Dingo Rodeo!",
-  NULL,
-  sizeof(Device_Dancepad_IDII) / sizeof(InputDeviceInputInfoStruct),
   Device_Dancepad_IDII,
  },
 
 };
 
-static const InputPortInfoStruct PortInfo[] =
+const std::vector<InputPortInfoStruct> FIO_PortInfo =
 {
- { "port1", "Virtual Port 1", sizeof(InputDeviceInfoPSXPort) / sizeof(InputDeviceInfoStruct), InputDeviceInfoPSXPort, "gamepad" },
- { "port2", "Virtual Port 2", sizeof(InputDeviceInfoPSXPort) / sizeof(InputDeviceInfoStruct), InputDeviceInfoPSXPort, "gamepad" },
- { "port3", "Virtual Port 3", sizeof(InputDeviceInfoPSXPort) / sizeof(InputDeviceInfoStruct), InputDeviceInfoPSXPort, "gamepad" },
- { "port4", "Virtual Port 4", sizeof(InputDeviceInfoPSXPort) / sizeof(InputDeviceInfoStruct), InputDeviceInfoPSXPort, "gamepad" },
- { "port5", "Virtual Port 5", sizeof(InputDeviceInfoPSXPort) / sizeof(InputDeviceInfoStruct), InputDeviceInfoPSXPort, "gamepad" },
- { "port6", "Virtual Port 6", sizeof(InputDeviceInfoPSXPort) / sizeof(InputDeviceInfoStruct), InputDeviceInfoPSXPort, "gamepad" },
- { "port7", "Virtual Port 7", sizeof(InputDeviceInfoPSXPort) / sizeof(InputDeviceInfoStruct), InputDeviceInfoPSXPort, "gamepad" },
- { "port8", "Virtual Port 8", sizeof(InputDeviceInfoPSXPort) / sizeof(InputDeviceInfoStruct), InputDeviceInfoPSXPort, "gamepad" },
+ { "port1", "Virtual Port 1", InputDeviceInfoPSXPort, "gamepad" },
+ { "port2", "Virtual Port 2", InputDeviceInfoPSXPort, "gamepad" },
+ { "port3", "Virtual Port 3", InputDeviceInfoPSXPort, "gamepad" },
+ { "port4", "Virtual Port 4", InputDeviceInfoPSXPort, "gamepad" },
+ { "port5", "Virtual Port 5", InputDeviceInfoPSXPort, "gamepad" },
+ { "port6", "Virtual Port 6", InputDeviceInfoPSXPort, "gamepad" },
+ { "port7", "Virtual Port 7", InputDeviceInfoPSXPort, "gamepad" },
+ { "port8", "Virtual Port 8", InputDeviceInfoPSXPort, "gamepad" },
 };
 
-InputInfoStruct FIO_InputInfo =
-{
- sizeof(PortInfo) / sizeof(InputPortInfoStruct),
- PortInfo
-};
-
-
 }
