@@ -5,7 +5,7 @@ namespace MDFN_IEN_PSX
 {
 
 InputDevice *Device_DualAnalog_Create(bool joystick_mode);
-extern InputDeviceInputInfoStruct Device_DualAnalog_IDII[24];
-extern InputDeviceInputInfoStruct Device_AnalogJoy_IDII[24];
+extern IDIISG Device_DualAnalog_IDII;
+extern IDIISG Device_AnalogJoy_IDII;
 }
 #endif
