@@ -5,23 +5,23 @@
 namespace MDFN_IEN_PSX
 {
 
-class InputDevice_Gamepad : public InputDevice
+class InputDevice_Gamepad final : public InputDevice
 {
  public:
 
  InputDevice_Gamepad();
- virtual ~InputDevice_Gamepad();
+ virtual ~InputDevice_Gamepad() override;
 
- virtual void Power(void);
- virtual int StateAction(StateMem* sm, int load, int data_only, const char* section_name);
- virtual void UpdateInput(const void *data);
+ virtual void Power(void) override;
+ virtual void StateAction(StateMem* sm, const unsigned load, const bool data_only, const char* sname_prefix) override;
+ virtual void UpdateInput(const void *data) override;
 
  //
  //
  //
- virtual void SetDTR(bool new_dtr);
- virtual bool GetDSR(void);
- virtual bool Clock(bool TxD, int32 &dsr_pulse_delay);
+ virtual void SetDTR(bool new_dtr) override;
+ virtual bool GetDSR(void) override;
+ virtual bool Clock(bool TxD, int32 &dsr_pulse_delay) override;
 
  private:
 
@@ -70,7 +70,7 @@ void InputDevice_Gamepad::Power(void)
  transmit_count = 0;
 }
 
-int InputDevice_Gamepad::StateAction(StateMem* sm, int load, int data_only, const char* section_name)
+void InputDevice_Gamepad::StateAction(StateMem* sm, const unsigned load, const bool data_only, const char* sname_prefix)
 {
  SFORMAT StateRegs[] =
  {
@@ -90,9 +90,12 @@ int InputDevice_Gamepad::StateAction(StateMem* sm, int load, int data_only, cons
 
   SFEND
  };
- int ret = MDFNSS_StateAction(sm, load, data_only, StateRegs, section_name);
+ char section_name[32];
+ trio_snprintf(section_name, sizeof(section_name), "%s_Gamepad", sname_prefix);
 
- if(load)
+ if(!MDFNSS_StateAction(sm, load, data_only, StateRegs, section_name, true) && load)
+  Power();
+ else if(load)
  {
   if((transmit_pos + transmit_count) > sizeof(transmit_buffer))
   {
@@ -100,8 +103,6 @@ int InputDevice_Gamepad::StateAction(StateMem* sm, int load, int data_only, cons
    transmit_count = 0;
   }
  }
-
- return(ret);
 }
 
 
@@ -227,7 +228,7 @@ InputDevice *Device_Gamepad_Create(void)
 }
 
 
-InputDeviceInputInfoStruct Device_Gamepad_IDII[16] =
+IDIISG Device_Gamepad_IDII =
 {
  { "select", "SELECT", 4, IDIT_BUTTON, NULL },
  { NULL, "empty", 0, IDIT_BUTTON },
@@ -249,7 +250,7 @@ InputDeviceInputInfoStruct Device_Gamepad_IDII[16] =
  { "square", "□ (left)", 8, IDIT_BUTTON_CAN_RAPID, NULL },
 };
 
-InputDeviceInputInfoStruct Device_Dancepad_IDII[16] =
+IDIISG Device_Dancepad_IDII =
 {
  { "select", "SELECT", 0, IDIT_BUTTON, NULL },
  { NULL, "empty", 0, IDIT_BUTTON },
