@@ -5,8 +5,8 @@ namespace MDFN_IEN_PSX
 {
 
 InputDevice *Device_Gamepad_Create(void);
-extern InputDeviceInputInfoStruct Device_Gamepad_IDII[16];
-extern InputDeviceInputInfoStruct Device_Dancepad_IDII[16];
+extern IDIISG Device_Gamepad_IDII;
+extern IDIISG Device_Dancepad_IDII;
 
 }
 #endif
