@@ -5,7 +5,7 @@ namespace MDFN_IEN_PSX
 {
 
 InputDevice *Device_Justifier_Create(void);
-extern InputDeviceInputInfoStruct Device_Justifier_IDII[6];
+extern IDIISG Device_Justifier_IDII;
 
 }
 #endif
