@@ -5,7 +5,7 @@ namespace MDFN_IEN_PSX
 {
 
 InputDevice *Device_Mouse_Create(void);
-extern InputDeviceInputInfoStruct Device_Mouse_IDII[4];
+extern IDIISG Device_Mouse_IDII;
 
 }
 #endif
