@@ -22,23 +22,23 @@
 namespace MDFN_IEN_PSX
 {
 
-class InputDevice_neGcon : public InputDevice
+class InputDevice_neGcon final : public InputDevice
 {
  public:
 
  InputDevice_neGcon(void);
- virtual ~InputDevice_neGcon();
+ virtual ~InputDevice_neGcon() override;
 
- virtual void Power(void);
- virtual int StateAction(StateMem* sm, int load, int data_only, const char* section_name);
- virtual void UpdateInput(const void *data);
+ virtual void Power(void) override;
+ virtual void StateAction(StateMem* sm, const unsigned load, const bool data_only, const char* sname_prefix) override;
+ virtual void UpdateInput(const void *data) override;
 
  //
  //
  //
- virtual void SetDTR(bool new_dtr);
- virtual bool GetDSR(void);
- virtual bool Clock(bool TxD, int32 &dsr_pulse_delay);
+ virtual void SetDTR(bool new_dtr) override;
+ virtual bool GetDSR(void) override;
+ virtual bool Clock(bool TxD, int32 &dsr_pulse_delay) override;
 
  private:
 
@@ -93,7 +93,7 @@ void InputDevice_neGcon::Power(void)
  transmit_count = 0;
 }
 
-int InputDevice_neGcon::StateAction(StateMem* sm, int load, int data_only, const char* section_name)
+void InputDevice_neGcon::StateAction(StateMem* sm, const unsigned load, const bool data_only, const char* sname_prefix)
 {
  SFORMAT StateRegs[] =
  {
@@ -115,9 +115,12 @@ int InputDevice_neGcon::StateAction(StateMem* sm, int load, int data_only, const
 
   SFEND
  };
- int ret = MDFNSS_StateAction(sm, load, data_only, StateRegs, section_name);
+ char section_name[32];
+ trio_snprintf(section_name, sizeof(section_name), "%s_neGcon", sname_prefix);
 
- if(load)
+ if(!MDFNSS_StateAction(sm, load, data_only, StateRegs, section_name, true) && load)
+  Power();
+ else if(load)
  {
   if((transmit_pos + transmit_count) > sizeof(transmit_buffer))
   {
@@ -125,8 +128,6 @@ int InputDevice_neGcon::StateAction(StateMem* sm, int load, int data_only, const
    transmit_count = 0;
   }
  }
-
- return(ret);
 }
 
 
@@ -265,7 +266,7 @@ InputDevice *Device_neGcon_Create(void)
 }
 
 
-InputDeviceInputInfoStruct Device_neGcon_IDII[21] =
+IDIISG Device_neGcon_IDII =
 {
  { NULL, "empty", -1, IDIT_BUTTON, NULL },
  { NULL, "empty", -1, IDIT_BUTTON, NULL },
