@@ -4,6 +4,6 @@
 namespace MDFN_IEN_PSX
 {
  InputDevice *Device_neGcon_Create(void);
- extern InputDeviceInputInfoStruct Device_neGcon_IDII[21];
+ extern IDIISG Device_neGcon_IDII;
 }
 #endif
