@@ -26,7 +26,7 @@
 
   if(InFIFO.CanWrite())
   {
-   InFIFO.WriteUnit(V);
+   InFIFO.Write(V);
 
    if(InCommand)
    {
@@ -55,9 +55,8 @@
 
 #include "psx.h"
 #include "mdec.h"
+#include "FastFIFO.h"
 
-#include <mednafen/masmem.h>
-#include <mednafen/cdrom/SimpleFIFO.h>
 #include <math.h>
 
 #if defined(__SSE2__)
@@ -74,8 +73,8 @@ namespace MDFN_IEN_PSX
 
 static int32 ClockCounter;
 static unsigned MDRPhase;
-static SimpleFIFO<uint32> InFIFO(0x20);
-static SimpleFIFO<uint32> OutFIFO(0x20);
+static FastFIFO<uint32, 0x20> InFIFO;
+static FastFIFO<uint32, 0x20> OutFIFO;
 
 static int8 block_y[8][8];
 static int8 block_cb[8][8];	// [y >> 1][x >> 1]
@@ -88,12 +87,12 @@ static bool InCommand;
 static uint8 QMatrix[2][64];
 static uint32 QMIndex;
 
-static int16 IDCTMatrix[64] MDFN_ALIGN(16);
+alignas(16) static int16 IDCTMatrix[64];
 static uint32 IDCTMIndex;
 
 static uint8 QScale;
 
-static int16 Coeff[64] MDFN_ALIGN(16);
+alignas(16) static int16 Coeff[64];
 static uint32 CoeffIndex;
 static uint32 DecodeWB;
 
@@ -163,14 +162,14 @@ void MDEC_Power(void)
  RAMOffsetWWS = 0;
 }
 
-int MDEC_StateAction(StateMem *sm, int load, int data_only)
+void MDEC_StateAction(StateMem *sm, const unsigned load, const bool data_only)
 {
  SFORMAT StateRegs[] =
  {
   SFVAR(ClockCounter),
   SFVAR(MDRPhase),
 
-#define SFFIFO32(fifoobj)  SFARRAY32(&fifoobj.data[0], fifoobj.data.size()),	\
+#define SFFIFO32(fifoobj)  SFARRAY32(&fifoobj.data[0], sizeof(fifoobj.data) / sizeof(fifoobj.data[0])),	\
 			 SFVAR(fifoobj.read_pos),				\
 			 SFVAR(fifoobj.write_pos),				\
 			 SFVAR(fifoobj.in_count)
@@ -212,15 +211,13 @@ int MDEC_StateAction(StateMem *sm, int load, int data_only)
   SFEND
  };
 
- int ret = MDFNSS_StateAction(sm, load, data_only, StateRegs, "MDEC");
+ MDFNSS_StateAction(sm, load, data_only, StateRegs, "MDEC");
 
  if(load)
  {
   InFIFO.SaveStatePostLoad();
   OutFIFO.SaveStatePostLoad();
  }
-
- return(ret);
 }
 
 static INLINE int8 Mask9ClampS8(int32 v)
@@ -249,7 +246,7 @@ static void IDCT_1D_Multi(int16 *in_coeff, T *out_coeff)
   {
    __m128i sum;
    __m128i m;
-   int32 tmp[4] MDFN_ALIGN(16);
+   alignas(16) int32 tmp[4];
 
    m = _mm_load_si128((__m128i *)&IDCTMatrix[(x * 8)]);
    sum = _mm_madd_epi16(m, c);
@@ -290,7 +287,7 @@ static void IDCT_1D_Multi(int16 *in_coeff, T *out_coeff)
 static void IDCT(int16 *in_coeff, int8 *out_coeff) NO_INLINE;
 static void IDCT(int16 *in_coeff, int8 *out_coeff)
 {
- int16 tmpbuf[64] MDFN_ALIGN(16);
+ alignas(16) int16 tmpbuf[64];
 
  IDCT_1D_Multi<int16>(in_coeff, tmpbuf);
  IDCT_1D_Multi<int8>(tmpbuf, out_coeff);
@@ -420,7 +417,7 @@ static void EncodeImage(const unsigned ybn)
 
      YCbCr_to_RGB(by[x], cb[x >> 1], cr[x >> 1], r, g, b);
 
-     StoreU16_LE(pix_out, pixel_xor ^ RGB_to_RGB555(r, g, b));
+     MDFN_en16lsb<true>(pix_out, pixel_xor ^ RGB_to_RGB555(r, g, b));
      pix_out++;
     }
    }
@@ -453,9 +450,9 @@ static INLINE void WriteImageData(uint16 V, int32* eat_cycles)
     int tmp;
 
     if(q != 0)
-     tmp = ((ci * q) << 4) + (ci ? ((ci < 0) ? 8 : -8) : 0);
+     tmp = (int32)((uint32)(ci * q) << 4) + (ci ? ((ci < 0) ? 8 : -8) : 0);
     else
-     tmp = (ci * 2) << 4;
+     tmp = (uint32)(ci * 2) << 4;
 
     // Not sure if it should be 0x3FFF or 0x3FF0 or maybe 0x3FF8?
     Coeff[ZigZag[0]] = std::min<int>(0x3FFF, std::max<int>(-0x4000, tmp));
@@ -486,9 +483,9 @@ static INLINE void WriteImageData(uint16 V, int32* eat_cycles)
      int tmp;
 
      if(q != 0)
-      tmp = (((ci * q) >> 3) << 4) + (ci ? ((ci < 0) ? 8 : -8) : 0);
+      tmp = (int32)((uint32)((ci * q) >> 3) << 4) + (ci ? ((ci < 0) ? 8 : -8) : 0);
      else
-      tmp = (ci * 2) << 4;
+      tmp = (uint32)(ci * 2) << 4;
 
      // Not sure if it should be 0x3FFF or 0x3FF0 or maybe 0x3FF8?
      Coeff[ZigZag[CoeffIndex]] = std::min<int>(0x3FFF, std::max<int>(-0x4000, tmp));
@@ -514,9 +511,11 @@ static INLINE void WriteImageData(uint16 V, int32* eat_cycles)
    }   
 
    //
-   // Approximate, actual timing seems to be a bit complex.
+   // Timing in the actual PS1 MDEC is complex due to (apparent) pipelining, but the average when decoding a large number of blocks is
+   // about 512.  We'll go with a lower value here to be conservative due to timing granularity and other timing deficiencies in Mednafen.  BUT, don't
+   // go lower than 460, or Parasite Eve 2's 3D models will stutter like crazy during FMV-background sequences.
    //
-   *eat_cycles += 341;
+   *eat_cycles += 474;
 
    if(DecodeWB >= 2)
    {
@@ -538,8 +537,8 @@ static INLINE void WriteImageData(uint16 V, int32* eat_cycles)
 //
 #define MDEC_WAIT_COND(n)  { case __COUNTER__: if(!(n)) { MDRPhase = __COUNTER__ - MDRPhaseBias - 1; return; } }
 
-#define MDEC_WRITE_FIFO(n) { MDEC_WAIT_COND(OutFIFO.CanWrite()); OutFIFO.WriteUnit(n);  }
-#define MDEC_READ_FIFO(n)  { MDEC_WAIT_COND(InFIFO.CanRead()); n = InFIFO.ReadUnit(); }
+#define MDEC_WRITE_FIFO(n) { MDEC_WAIT_COND(OutFIFO.CanWrite()); OutFIFO.Write(n);  }
+#define MDEC_READ_FIFO(n)  { MDEC_WAIT_COND(InFIFO.CanRead()); n = InFIFO.Read(); }
 #define MDEC_EAT_CLOCKS(n) { ClockCounter -= (n); MDEC_WAIT_COND(ClockCounter > 0); }
 
 void MDEC_Run(int32 clocks)
@@ -616,7 +615,7 @@ void MDEC_Run(int32 clocks)
      PixelBufferReadOffset = 0;
      while(PixelBufferReadOffset != PixelBufferCount32)
      {
-      MDEC_WRITE_FIFO(LoadU32_LE(&PixelBuffer.pix32[PixelBufferReadOffset++]));
+      MDEC_WRITE_FIFO((MDFN_de32lsb<true>(&PixelBuffer.pix32[PixelBufferReadOffset++])));
      }
     } while(InCounter != 0xFFFF);
    }
@@ -684,16 +683,16 @@ void MDEC_DMAWrite(uint32 V)
 {
  if(InFIFO.CanWrite())
  {
-  InFIFO.WriteUnit(V);
+  InFIFO.Write(V);
   MDEC_Run(0);
  }
  else
  {
-  PSX_DBG(PSX_DBG_WARNING, "Input FIFO DMA write overflow?!\n");
+  PSX_DBG(PSX_DBG_WARNING, "[MDEC] DMA write when input FIFO is full!!\n");
  }
 }
 
-uint32 MDEC_DMARead(int32* offs)
+uint32 MDEC_DMARead(uint32* offs)
 {
  uint32 V = 0;
 
@@ -701,7 +700,7 @@ uint32 MDEC_DMARead(int32* offs)
 
  if(MDFN_LIKELY(OutFIFO.CanRead()))
  {
-  V = OutFIFO.ReadUnit();
+  V = OutFIFO.Read();
 
   *offs = (RAMOffsetY & 0x7) * RAMOffsetWWS;
 
@@ -716,11 +715,12 @@ uint32 MDEC_DMARead(int32* offs)
    RAMOffsetCounter = RAMOffsetWWS;
    RAMOffsetY++;
   }
+
+  MDEC_Run(0);
  }
  else
  {
-  PSX_DBG(PSX_DBG_WARNING, "[MDEC] BONUS GNOMES\n");
-  V = rand();
+  PSX_DBG(PSX_DBG_WARNING, "[MDEC] DMA read when output FIFO is empty!\n");
  }
 
  return(V);
@@ -768,7 +768,7 @@ void MDEC_Write(const pscpu_timestamp_t timestamp, uint32 A, uint32 V)
  {
   if(InFIFO.CanWrite())
   {
-   InFIFO.WriteUnit(V);
+   InFIFO.Write(V);
 
    if(!InCommand)
    {
@@ -808,7 +808,7 @@ uint32 MDEC_Read(const pscpu_timestamp_t timestamp, uint32 A)
  else
  {
   if(OutFIFO.CanRead())
-   ret = OutFIFO.ReadUnit();
+   ret = OutFIFO.Read();
  }
 
  //PSX_WARNING("[MDEC] Read: 0x%08x 0x%08x -- %d %d", A, ret, InputBuffer.CanRead(), InCounter);
