@@ -25,8 +25,7 @@
 	Test reverb upsampler on the real thing.
 
 	Alter reverb algorithm to process in the pattern of L,R,L,R,L,R on each input sample, instead of doing both L and R on every 2 input samples(make
-	sure the real thing does it this way too, I think it at least runs the downsampler this way); and while we're at it, implement the correct buffer
-	offset(probably either -39 or -40, the latter is what we have now).
+	sure the real thing does it this way too, I think it at least runs the downsampler this way).
 
 	Alter reverb algorithm to perform saturation more often, as occurs on the real thing.
 
@@ -47,8 +46,6 @@
 
 /*
  Notes:
-	The last half of the noise freq table was confirmed on a real PSX(more or less, number of changes * 0x8000 / samples), but the first half hasn't been yet with sufficient precision.
-
 	All addresses(for 16-bit access, at least) within the SPU address space appear to be fully read/write as if they were RAM, though
 	values at some addresses(like the envelope current value) will be "overwritten" by the sound processing at certain times.
 
@@ -178,10 +175,8 @@ void PS_SPU::Power(void)
  memset(Regs, 0, sizeof(Regs));
 
  memset(RDSB, 0, sizeof(RDSB));
- RDSB_WP = 0;
-
  memset(RUSB, 0, sizeof(RUSB));
- RUSB_WP = 0;
+ RvbResPos = 0;
 
  ReverbCur = ReverbWA;
 
@@ -198,7 +193,7 @@ static INLINE void CalcVCDelta(const uint8 zs, uint8 speed, bool log_mode, bool
   divinco = 32768;
 
   if(speed < 0x2C)
-   increment <<= (0x2F - speed) >> 2;
+   increment = (unsigned)increment << ((0x2F - speed) >> 2);
 
   if(speed >= 0x30)
    divinco >>= (speed - 0x2C) >> 2;
@@ -337,7 +332,16 @@ void PS_SPU::RunDecoder(SPU_Voice *voice)
 
  if((voice->CurAddr & 0x7) == 0)
  {
+  //
   // Handle delayed flags from the previously-decoded block.
+  //
+  // NOTE: The timing of setting the BlockEnd bit here, and forcing ADSR envelope volume to 0, is a bit late.  (And I'm not sure if it should be done once
+  // per decoded block, or more than once, but that's probably not something games would rely on, but we should test it anyway).
+  //
+  // Correctish timing can be achieved by moving this block of code up above voice->DecodeAvail >= 11, and sticking it inside an: if(voice->DecodeAvail <= 12),
+  // though more tests are needed on the ADPCM decoding process as a whole before we should actually make such a change.  Additionally, we'd probably
+  // have to separate the CurAddr = LoopAddr logic, so we don't generate spurious early SPU IRQs.
+  //
   if(voice->DecodeFlags & 0x1)
   {
    voice->CurAddr = voice->LoopAddr & ~0x7;
@@ -398,13 +402,26 @@ void PS_SPU::RunDecoder(SPU_Voice *voice)
   // at higher rates will fail horribly.
   //
   {
-   const uint16 CV = SPURAM[voice->CurAddr];
-   const unsigned shift = voice->DecodeShift;
    const int32 weight_m1 = Weights[voice->DecodeWeight][0];
    const int32 weight_m2 = Weights[voice->DecodeWeight][1];
-   uint32 coded = (uint32)CV << 12;
+   uint16 CV;
+   unsigned shift;
+   uint32 coded;
    int16 *tb = &voice->DecodeBuffer[voice->DecodeWritePos];
 
+   CV = SPURAM[voice->CurAddr];
+   shift = voice->DecodeShift;
+
+   if(MDFN_UNLIKELY(shift > 12))
+   {
+    //PSX_DBG(PSX_DBG_FLOOD, "[SPU] Buggy/Illegal ADPCM block shift value on voice %u: %u\n", (unsigned)(voice - Voices), shift);
+
+    shift = 8;
+    CV &= 0x8888;
+   }
+
+   coded = (uint32)CV << 12;
+
    for(int i = 0; i < 4; i++)
    {
     int32 sample = (int16)(coded & 0xF000) >> shift;
@@ -601,21 +618,19 @@ int32 PS_SPU::UpdateFromCDC(int32 clocks)
 
  while(sample_clocks > 0)
  {
-  // Accumulated normal sound output.
-  int32 accum_l = 0;
-  int32 accum_r = 0;
+  // xxx[0] = left, xxx[1] = right
+
+  // Accumulated sound output.
+  int32 accum[2] = { 0, 0 };
 
   // Accumulated sound output for reverb input
-  int32 accum_fv_l = 0;
-  int32 accum_fv_r = 0;
+  int32 accum_fv[2] = { 0, 0 };
 
   // Output of reverb processing.
-  int32 reverb_l = 0;
-  int32 reverb_r = 0;
+  int32 reverb[2] = { 0, 0 };
 
   // Final output.
-  int32 output_l = 0;
-  int32 output_r = 0;
+  int32 output[2] = { 0, 0 };
 
   const uint32 PhaseModCache = FM_Mode & ~ 1;
 /*
@@ -697,13 +712,13 @@ int32 PS_SPU::UpdateFromCDC(int32 clocks)
    l = (voice_pvs * voice->Sweep[0].ReadVolume()) >> 15;
    r = (voice_pvs * voice->Sweep[1].ReadVolume()) >> 15;
 
-   accum_l += l;
-   accum_r += r;
+   accum[0] += l;
+   accum[1] += r;
 
    if(Reverb_Mode & (1 << voice_num))
    {
-    accum_fv_l += l;
-    accum_fv_r += r;
+    accum_fv[0] += l;
+    accum_fv[1] += r;
    }
 
    // Run sweep
@@ -789,10 +804,10 @@ int32 PS_SPU::UpdateFromCDC(int32 clocks)
   // TODO: If we add sub-sample timing accuracy, see if it's checked for every channel at different times, or just once.
   if(!(SPUControl & 0x4000))
   {
-   accum_l = 0;
-   accum_r = 0;
-   accum_fv_l = 0;
-   accum_fv_r = 0;
+   accum[0] = 0;
+   accum[1] = 0;
+   accum_fv[0] = 0;
+   accum_fv[1] = 0;
   }
 
   // Get CD-DA
@@ -811,13 +826,13 @@ int32 PS_SPU::UpdateFromCDC(int32 clocks)
 
    if(SPUControl & 0x0001)
    {
-    accum_l += cdav[0];
-    accum_r += cdav[1];
+    accum[0] += cdav[0];
+    accum[1] += cdav[1];
 
     if(SPUControl & 0x0004)	// TODO: Test this bit(and see if it is really dependent on bit0)
     {
-     accum_fv_l += cdav[0];
-     accum_fv_r += cdav[1];
+     accum_fv[0] += cdav[0];
+     accum_fv[1] += cdav[1];
     }
    }
   }
@@ -826,43 +841,32 @@ int32 PS_SPU::UpdateFromCDC(int32 clocks)
 
   RunNoise();
 
-  clamp(&accum_l, -32768, 32767);
-  clamp(&accum_r, -32768, 32767);
-  clamp(&accum_fv_l, -32768, 32767);
-  clamp(&accum_fv_r, -32768, 32767);
+  for(unsigned lr = 0; lr < 2; lr++)
+   clamp(&accum_fv[lr], -32768, 32767);
   
-#if 0
-  accum_l = 0;
-  accum_r = 0;
-  //accum_fv_l = (short)(rand());
-  //accum_fv_r = (short)(rand());
-#endif
+  RunReverb(accum_fv, reverb);
 
-  RunReverb(accum_fv_l, accum_fv_r, reverb_l, reverb_r);
-
-  //MDFN_DispMessage("%d %d\n", MainVol[0], MainVol[1], ReverbVol[0], ReverbVol[1]);
-
-  // FIXME: Dry volume versus everything else
-  output_l = (((accum_l * GlobalSweep[0].ReadVolume()) >> 16) + ((reverb_l * ReverbVol[0]) >> 15));
-  output_r = (((accum_r * GlobalSweep[1].ReadVolume()) >> 16) + ((reverb_r * ReverbVol[1]) >> 15));
-
-  //output_l = reverb_l;
-  //output_r = reverb_r;
-
-  clamp(&output_l, -32768, 32767);
-  clamp(&output_r, -32768, 32767);
+  for(unsigned lr = 0; lr < 2; lr++)
+  {
+   accum[lr] += ((reverb[lr] * ReverbVol[lr]) >> 15);
+   clamp(&accum[lr], -32768, 32767);
+   output[lr] = (accum[lr] * GlobalSweep[lr].ReadVolume()) >> 15;
+   clamp(&output[lr], -32768, 32767);
+  }
 
   if(IntermediateBufferPos < 4096)	// Overflow might occur in some debugger use cases.
   {
-   IntermediateBuffer[IntermediateBufferPos][0] = output_l;
-   IntermediateBuffer[IntermediateBufferPos][1] = output_r;
+   // 75%, for some (resampling) headroom.
+   for(unsigned lr = 0; lr < 2; lr++)
+    IntermediateBuffer[IntermediateBufferPos][lr] = (output[lr] * 3 + 2) >> 2;
+
    IntermediateBufferPos++;
   }
 
   sample_clocks--;
 
   // Clock global sweep
-  for(int lr = 0; lr < 2; lr++)
+  for(unsigned lr = 0; lr < 2; lr++)
    GlobalSweep[lr].Clock();
  }
 
@@ -1177,9 +1181,28 @@ void PS_SPU::StartFrame(double rate, uint32 quality)
 
 }
 
-int32 PS_SPU::EndFrame(int16 *SoundBuf)
+int32 PS_SPU::EndFrame(int16 *SoundBuf, bool reverse)
 {
- //lastts = 0;
+ if(reverse)
+ {
+  for(unsigned lr = 0; lr < 2; lr++)
+  {
+   int16* p0 = &IntermediateBuffer[0][lr];
+   int16* p1 = &IntermediateBuffer[IntermediateBufferPos - 1][lr];
+   unsigned count = IntermediateBufferPos >> 1;
+
+   while(MDFN_LIKELY(count--))
+   {
+    int16 tmp;
+
+    tmp = *p0;
+    *p0 = *p1;
+    *p1 = tmp;
+    p0 += 2;
+    p1 -= 2;
+   }
+  }
+ }
 
  if(last_rate == 44100)
  {
@@ -1216,7 +1239,7 @@ int32 PS_SPU::EndFrame(int16 *SoundBuf)
  }
 }
 
-int PS_SPU::StateAction(StateMem *sm, int load, int data_only)
+void PS_SPU::StateAction(StateMem *sm, const unsigned load, const bool data_only)
 {
  SFORMAT StateRegs[] =
  {
@@ -1325,10 +1348,9 @@ int PS_SPU::StateAction(StateMem *sm, int load, int data_only)
   SFARRAY16(AuxRegs, sizeof(AuxRegs) / sizeof(AuxRegs[0])),
 
   SFARRAY16(&RDSB[0][0], sizeof(RDSB) / sizeof(RDSB[0][0])),
-  SFVAR(RDSB_WP),
+  SFVAR(RvbResPos),
 
   SFARRAY16(&RUSB[0][0], sizeof(RUSB) / sizeof(RUSB[0][0])),
-  SFVAR(RUSB_WP),
 
   SFVAR(ReverbCur),
   SFVAR(IRQAsserted),
@@ -1339,9 +1361,8 @@ int PS_SPU::StateAction(StateMem *sm, int load, int data_only)
   SFEND
  };
 #undef SFSWEEP
- int ret = 1;
 
- ret &= MDFNSS_StateAction(sm, load, data_only, StateRegs, "SPU");
+ MDFNSS_StateAction(sm, load, data_only, StateRegs, "SPU");
 
  if(load)
  {
@@ -1351,13 +1372,10 @@ int PS_SPU::StateAction(StateMem *sm, int load, int data_only)
    Voices[i].DecodeWritePos &= 0x1F;
   }
 
-  RDSB_WP &= 0x3F;
-  RUSB_WP &= 0x3F;
+  RvbResPos &= 0x3F;
 
   IRQ_Assert(IRQ_SPU, IRQAsserted);
  }
-
- return(ret);
 }
 
 uint16 PS_SPU::PeekSPURAM(uint32 address)
@@ -1421,6 +1439,10 @@ uint32 PS_SPU::GetRegister(unsigned int which, char *special, const uint32 speci
 	break;
   }
  }
+ else if(which >= GSREG_FB_SRC_A && which <= GSREG_IN_COEF_R)
+ {
+	ret = ReverbRegs[which - GSREG_FB_SRC_A];
+ }
  else switch(which)
  {
   case GSREG_SPUCONTROL:
@@ -1447,27 +1469,27 @@ uint32 PS_SPU::GetRegister(unsigned int which, char *special, const uint32 speci
 	ret = (uint16)CDVol[1];
 	break;
 
-  case GSREG_DRYVOL_CTRL_L:
+  case GSREG_MAINVOL_CTRL_L:
 	ret = Regs[0xC0];
 	break;
 
-  case GSREG_DRYVOL_CTRL_R:
+  case GSREG_MAINVOL_CTRL_R:
 	ret = Regs[0xC1];
 	break;
 
-  case GSREG_DRYVOL_L:
+  case GSREG_MAINVOL_L:
 	ret = GlobalSweep[0].ReadVolume() & 0xFFFF;
 	break;
 
-  case GSREG_DRYVOL_R:
+  case GSREG_MAINVOL_R:
 	ret = GlobalSweep[1].ReadVolume() & 0xFFFF;
 	break;
 
-  case GSREG_WETVOL_L:
+  case GSREG_RVBVOL_L:
 	ret = (uint16)ReverbVol[0];
 	break;
 
-  case GSREG_WETVOL_R:
+  case GSREG_RVBVOL_R:
 	ret = (uint16)ReverbVol[1];
 	break;
 
@@ -1494,11 +1516,6 @@ uint32 PS_SPU::GetRegister(unsigned int which, char *special, const uint32 speci
   case GSREG_BLOCKEND:
 	ret = BlockEnd;
 	break;
-
-
-  case GSREG_FB_SRC_A ... GSREG_IN_COEF_R:
-	ret = ReverbRegs[which - GSREG_FB_SRC_A] & 0xFFFF;
-	break;
  }
 
  return(ret);
@@ -1506,8 +1523,11 @@ uint32 PS_SPU::GetRegister(unsigned int which, char *special, const uint32 speci
 
 void PS_SPU::SetRegister(unsigned int which, uint32 value)
 {
-
- switch(which)
+ if(which >= GSREG_FB_SRC_A && which <= GSREG_IN_COEF_R)
+ {
+	ReverbRegs[which - GSREG_FB_SRC_A] = value;
+ }
+ else switch(which)
  {
   case GSREG_SPUCONTROL:
 	SPUControl = value;
@@ -1533,31 +1553,31 @@ void PS_SPU::SetRegister(unsigned int which, uint32 value)
 	CDVol[1] = (int16)value;
 	break;
 
-  case GSREG_DRYVOL_CTRL_L:
+  case GSREG_MAINVOL_CTRL_L:
 	Regs[0xC0] = value;
 	GlobalSweep[0].WriteControl(value);
 	//GlobalSweep[0].Control = value;
 	break;
 
-  case GSREG_DRYVOL_CTRL_R:
+  case GSREG_MAINVOL_CTRL_R:
 	Regs[0xC1] = value;
 	GlobalSweep[1].WriteControl(value);
 	//GlobalSweep[1].Control = value;
 	break;
 
-  case GSREG_DRYVOL_L:
+  case GSREG_MAINVOL_L:
 	GlobalSweep[0].WriteVolume(value);
 	break;
 
-  case GSREG_DRYVOL_R:
+  case GSREG_MAINVOL_R:
 	GlobalSweep[1].WriteVolume(value);
 	break;
 
-  case GSREG_WETVOL_L:
+  case GSREG_RVBVOL_L:
 	ReverbVol[0] = (int16)value;
 	break;
 
-  case GSREG_WETVOL_R:
+  case GSREG_RVBVOL_R:
 	ReverbVol[1] = (int16)value;
 	break;
 
@@ -1584,8 +1604,6 @@ void PS_SPU::SetRegister(unsigned int which, uint32 value)
   case GSREG_BLOCKEND:
         BlockEnd = value & 0xFFFFFF;
         break;
-
-
  }
 }
 
