@@ -1,26 +1,19 @@
-/* SexyAL - Simple audio abstraction library.
-
-Copyright (c) 2014 Mednafen Team
-
-Permission is hereby granted, free of charge, to any person obtaining
-a copy of this software and associated documentation files (the
-"Software"), to deal in the Software without restriction, including
-without limitation the rights to use, copy, modify, merge, publish,
-distribute, sublicense, and/or sell copies of the Software, and to
-permit persons to whom the Software is furnished to do so, subject to
-the following conditions:
-
-The above copyright notice and this permission notice shall be included
-in all copies or substantial portions of the Software.
-
-THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
-EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
-MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
-IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
-CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
-TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
-SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
-*/
+/* Mednafen - Multi-system Emulator
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ */
 
 /*
  DSP versus output capabilities(from Creative's manual):
@@ -664,6 +657,14 @@ static uint64_t SaveFPAndResetMixer(SB_Driver_t* ds)
  return(fip);
 }
 
+bool SexyALI_DOS_SB_Avail(void)
+{
+ if(getenv("BLASTER") != NULL)
+  return(true);
+
+ return(false);
+}
+
 #define SB_INIT_CLEANUP		\
 	if(device) free(device);	\
 	if(ds) { Cleanup(ds); free(ds); }
@@ -830,7 +831,8 @@ SexyAL_device *SexyALI_DOS_SB_Open(const char *id, SexyAL_format *format, SexyAL
  // Get DSP version.
  //
  dsp_command(ds, 0xE1);
- ds->dsp_version = (dsp_read(ds) << 8) | (dsp_read(ds) << 0);
+ ds->dsp_version = dsp_read(ds) << 8;
+ ds->dsp_version |= dsp_read(ds) << 0;
 
  if(ds->dsp_version < 0x200)
  {
