@@ -1,26 +1,19 @@
-/* SexyAL - Simple audio abstraction library.
-
-Copyright (c) 2013 Mednafen Team
-
-Permission is hereby granted, free of charge, to any person obtaining
-a copy of this software and associated documentation files (the
-"Software"), to deal in the Software without restriction, including
-without limitation the rights to use, copy, modify, merge, publish,
-distribute, sublicense, and/or sell copies of the Software, and to
-permit persons to whom the Software is furnished to do so, subject to
-the following conditions:
-
-The above copyright notice and this permission notice shall be included
-in all copies or substantial portions of the Software.
-
-THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
-EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
-MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
-IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
-CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
-TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
-SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
-*/
+/* Mednafen - Multi-system Emulator
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ */
 
 // TODO: Memory fences and/or atomic types if we want this code to work properly on non-x86 Windows.
 
@@ -201,6 +194,24 @@ static DWORD WINAPI AThreadMain(LPVOID param)
  return(0);
 }
 
+bool SexyALI_WASAPI_Avail(void)
+{
+ IMMDeviceEnumerator *imd = NULL;
+ HRESULT hr;
+
+ hr = CoInitializeEx(NULL, COINIT_MULTITHREADED);
+ if(hr != S_OK && hr != S_FALSE)
+  return(false);
+
+ hr = CoCreateInstance(LV_CLSID_MMDeviceEnumerator, NULL, CLSCTX_ALL, LV_IID_IMMDeviceEnumerator, (void**)&imd);
+ if(FAILED(hr))
+  return(false);
+
+ imd->Release();
+
+ return(true);
+}
+
 SexyAL_device *SexyALI_WASAPI_Open(const char *id, SexyAL_format *format, SexyAL_buffering *buffering)
 {
  SexyAL_device *dev;
@@ -445,7 +456,7 @@ SexyAL_device *SexyALI_WASAPI_Open(const char *id, SexyAL_format *format, SexyAL
   //printf("MOOMOOOOO: %u\n", (unsigned)raw_ac_latency);
 
   int32_t dtbfs = (int64_t)(buffering->ms ? buffering->ms : 32) * wfe.Format.nSamplesPerSec / 1000;
-  int32_t des_local_bufsize = std::max<int32_t>((int32_t)dtbfs - (w->bfc * 2), w->bfc);
+  int32_t des_local_bufsize = std::max<int32_t>(dtbfs, w->bfc);
   int32_t des_local_bufsize_super = des_local_bufsize + ((30 * wfe.Format.nSamplesPerSec + 999) / 1000);
 
   w->BufferRBC = 0;
@@ -462,7 +473,7 @@ SexyAL_device *SexyALI_WASAPI_Open(const char *id, SexyAL_format *format, SexyAL
 
   buffering->buffer_size = des_local_bufsize;
   buffering->period_size = w->bfc;
-  buffering->latency = des_local_bufsize + w->bfc * 2;
+  buffering->latency = des_local_bufsize + w->bfc;
   buffering->bt_gran = 0;
  }
 
@@ -487,6 +498,7 @@ SexyAL_device *SexyALI_WASAPI_Open(const char *id, SexyAL_format *format, SexyAL
 
  TRYHR(w->ac->GetService(LV_IID_IAudioRenderClient, (void**)&w->arc));
 
+ memcpy(&dev->buffering, buffering, sizeof(SexyAL_buffering));
  memcpy(&dev->format, format, sizeof(SexyAL_format));
  dev->RawWrite = RawWrite;
  dev->RawCanWrite = RawCanWrite;
