@@ -1,7 +1,7 @@
-#include <../base.hpp>
+//#include <../base.hpp>
 
 #define SMPCORE_CPP
-namespace SNES {
+namespace bSNES_v059 {
 
 #include "serialization.cpp"
 #include "algorithms.cpp"
@@ -24,8 +24,4 @@ namespace SNES {
 #undef Y
 #undef SP
 
-SMPcore::SMPcore() {
-  initialize_opcode_table();
-}
-
 };
