@@ -207,7 +207,7 @@ static void nec_interrupt(unsigned int_num)
 {
     uint32 dest_seg, dest_off;
 
-	if (int_num == -1)
+	if ((int)int_num == -1)
 		return;
 
 	 i_real_pushf();
@@ -339,7 +339,7 @@ static void DoOP(uint8 opcode)
 {
  //#define OP(num,func_name) static void func_name(void)
  #define OP(num, func_name) case num: 
- #define OP_RANGE(num1, num2, func_name) case num1 ... num2:
+ #define OP8(base, func_name) case (base): case (base)+1: case (base)+2: case (base)+3: case (base)+4: case (base)+5: case (base)+6: case (base)+7:
  #define OP_EPILOGUE break
 
 switch(opcode)
@@ -474,7 +474,7 @@ OP( 0x61, i_popa  ) {
 	POP(I.regs.w[IY]);
 	POP(I.regs.w[IX]);
 	POP(I.regs.w[BP]);
-    POP(tmp);
+    	POP(tmp); (void)tmp;
 	POP(I.regs.w[BW]);
 	POP(I.regs.w[DW]);
 	POP(I.regs.w[CW]);
@@ -804,12 +804,12 @@ OP( 0xd3, i_rotshft_wcl ) {
 	}
 } OP_EPILOGUE;
 
-OP( 0xd4, i_aam    ) { uint32 mult=FETCH; mult=0; I.regs.b[AH] = I.regs.b[AL] / 10; I.regs.b[AL] %= 10; SetSZPF_Word(I.regs.w[AW]); CLK(17); } OP_EPILOGUE;
-OP( 0xd5, i_aad    ) { uint32 mult=FETCH; mult=0; I.regs.b[AL] = I.regs.b[AH] * 10 + I.regs.b[AL]; I.regs.b[AH] = 0; SetSZPF_Byte(I.regs.b[AL]); CLK(6); } OP_EPILOGUE;
+OP( 0xd4, i_aam    ) { (void)FETCH; I.regs.b[AH] = I.regs.b[AL] / 10; I.regs.b[AL] %= 10; SetSZPF_Word(I.regs.w[AW]); CLK(17); } OP_EPILOGUE;
+OP( 0xd5, i_aad    ) { (void)FETCH; I.regs.b[AL] = I.regs.b[AH] * 10 + I.regs.b[AL]; I.regs.b[AH] = 0; SetSZPF_Byte(I.regs.b[AL]); CLK(6); } OP_EPILOGUE;
 OP( 0xd6, i_setalc ) { I.regs.b[AL] = (CF)?0xff:0x00; CLK(3);  } OP_EPILOGUE;
 OP( 0xd7, i_trans  ) { uint32 dest = (I.regs.w[BW]+I.regs.b[AL])&0xffff; I.regs.b[AL] = GetMemB(DS0, dest); CLK(5); } OP_EPILOGUE;
 
-OP_RANGE(0xd8, 0xdf, i_fpo) { /*printf("FPO1, Op:%02x\n", opcode);*/ GetModRM; CLK(1); } OP_EPILOGUE;
+OP8(0xd8, i_fpo) { /*printf("FPO1, Op:%02x\n", opcode);*/ GetModRM; (void)ModRM; CLK(1); } OP_EPILOGUE;
 
 OP( 0xe0, i_loopne ) { int8 disp = (int8)FETCH; I.regs.w[CW]--; if (!ZF && I.regs.w[CW]) { I.pc = (uint16)(I.pc+disp);  CLK(6); ADDBRANCHTRACE(I.sregs[PS], I.pc); } else CLK(3); } OP_EPILOGUE;
 OP( 0xe1, i_loope  ) { int8 disp = (int8)FETCH; I.regs.w[CW]--; if ( ZF && I.regs.w[CW]) { I.pc = (uint16)(I.pc+disp);  CLK(6); ADDBRANCHTRACE(I.sregs[PS], I.pc); } else CLK(3); } OP_EPILOGUE;
@@ -1134,7 +1134,7 @@ void v30mz_debug(void (*CPUHook)(uint32), uint8 (*ReadHook)(uint32), void (*Writ
 }
 #endif
 
-int v30mz_StateAction(StateMem *sm, int load, int data_only)
+void v30mz_StateAction(StateMem *sm, const unsigned load, const bool data_only)
 {
  uint16 PSW;
 
@@ -1154,15 +1154,12 @@ int v30mz_StateAction(StateMem *sm, int load, int data_only)
 
  PSW = CompressFlags();
 
- if(!MDFNSS_StateAction(sm, load, data_only, StateRegs, "V30"))
-  return(0);
+ MDFNSS_StateAction(sm, load, data_only, StateRegs, "V30");
 
  if(load)
  {
   ExpandFlags(PSW);
  }
-
- return(1);
 }
 
 }
