@@ -1,9 +1,9 @@
 #ifndef __WSWAN_H
 #define __WSWAN_H
 
-#include "../mednafen.h"
-#include "../state.h"
-#include "../general.h"
+#include <mednafen/mednafen.h>
+#include <mednafen/state.h>
+#include <mednafen/general.h>
 
 #include "interrupt.h"
 
@@ -12,6 +12,7 @@ namespace MDFN_IEN_WSWAN
 
 #define  mBCD(value) (((value)/10)<<4)|((value)%10)
 
+extern		uint32 WS_InDebug;
 extern          uint32 rom_size;
 extern          int wsc;
 
